#! /bin/bash

# GentooLinux common functions
source /etc/init.d/functions.sh

function is_cf_storage {
	local result

	ebegin "Searching CompactFlash Storage"
	pccardctl ident | grep "fixed disk" &> /dev/null
	result=$?
	eend ${result}

	return ${result}
}

function find_mounted {
	local mounted

	mounted=`cat /proc/mounts|egrep hda[0-9]+|cut -d' ' -f 1`
	echo ${mounted}
}

function unmount_all {
	local msg=$1
	local mounted_devices
	local dev
	local result=1

	mounted_devices=`find_mounted`

	if [ "${mounted_devices}" == "" ]; then
		return 0
	fi

	for dev in ${mounted_devices}; do
		ebegin "Unmounting CF partition ${dev}"
		umount ${dev} &> /dev/null
		eend $?
	done

	mounted_devices=`find_mounted`

	if [ "${mounted_devices}" == "" ]; then
		result=0
	fi

	return ${result}
}

# main

# check CF storage device
is_cf_storage
if [ "$?" != "0" ]; then
	echo " * CF(Storage) not found, exit."
	exit 0
fi

# unmount all CF partitions
unmount_all
if [ "$?" != "0" ]; then
	unmount_all "(retry)"
fi

if [ "$?" != "0" ]; then
	echo "Unmount failed, exit."
fi

ebegin "Eject CompactFlash card"
pccardctl eject 0 &> /dev/null
eend $?
