#!/bin/sh

# This script is run by pppd after the link is established.
# It executes all the scripts available in /etc/ppp/ip-up.d directory,
# with the following parameters:
# $1 = interface name (e.g. ppp0)
# $2 = tty device
# $3 = speed
# $4 = local IP address
# $5 = remote IP address
# $6 = ipparam (user specified parameter, see man pppd)

PRGNAME="991-route.sh"

LOGGER=/usr/bin/logger
STATEPATH=/var/state
SCRIPTSPATH=/etc/ppp/scripts
PPPSCRIPT=$SCRIPTSPATH/ppp_function
NETSCRIPT=$SCRIPTSPATH/net_function

BASEFILE="ROUTE..."

INTERFACE=$1
DEVICE=$2
SPEED=$3
LOCALIP=$4
REMOTEIP=$5
IPPARAM=$5

if [ -f $PPPSCRIPT ]; then
  . $PPPSCRIPT
else
  exit 1
fi

if [ -f $NETSCRIPT ]; then
  . $NETSCRIPT
else
  exit 1
fi

# 自分の設定がデフォルトルートを要求していなければ終了。
check_ppp_options $INTERFACE defaultroute
if [ $? -eq 0 ]; then
  $LOGGER -p user.info "ip-up.d: $PRGNAME: Default route does not set it."
  exit 0
fi

# イーサネットにスタティックデフォルトルートが設定されていれば終了。
# イーサネットのDHCPにデフォルトルートを受け入れる設定になっていれば終了。
check_net_route
if [ $? -eq 1 ]; then
  $LOGGER -p user.info "ip-up.d: $PRGNAME: Default route is set by Ether."
  exit 0
fi

# 自分はPPPoEでなく、他のPPPoE設定にデフォルトルートの設定があれば終了。
check_pppoe $INTERFACE
if [ $? -eq 0 ]; then
  check_pppoe_all_options defaultroute
  if [ $? -eq 1 ]; then
    $LOGGER -p user.info "ip-up.d: $PRGNAME: Default route is set by PPPoE"
    exit 0
  fi
fi

# 誰もデフォルトルートを書き換えていなければ設定を保存
# (デフォルトルートが自分のI/Fの場合は、保存せずかつ削除してはならない）
CHECK_STATE=`ls ${STATEPATH}/${BASEFILE}*`
if [ -z $CHECK_STATE ] || [ $CHECK_STATE == "" ]; then
  route -n | grep UG > /tmp/${BASEFILE}${INTERFACE}
  COUNT=0
  MAXCOUNT=`cat /tmp/${BASEFILE}${INTERFACE} | wc | awk '{ print $1}'`
  while [ $COUNT -lt $MAXCOUNT ]; do
    VALUE=`lines -l $COUNT -f /tmp/${BASEFILE}${INTERFACE}`
    [ $? -ne 0 ] && break
    DESTINATION=`echo $VALUE | awk '{ print $1 }'`
    GENMASK=`echo $VALUE | awk '{ print $3 }'`
    PPPIF=`echo $VALUE | awk '{ print $8 }'`
    if [ "$DESTINATION" == "0.0.0.0" ] && [ "$GENMASK" == "0.0.0.0" ] && [ "$PPPIF" != ${INTERFACE} ]; then
      echo $VALUE >> ${STATEPATH}/${BASEFILE}${INTERFACE}
    fi
    COUNT=$(($COUNT+1))
  done
fi

# デフォルトルートを削除
COUNT=0
MAXCOUNT=`cat ${STATEPATH}/${BASEFILE}${INTERFACE} | wc | awk '{ print $1}'`
while [ $COUNT -lt $MAXCOUNT ]; do
  VALUE=`lines -l $COUNT -f ${STATEPATH}/${BASEFILE}${INTERFACE}`
  [ $? -ne 0 ] && break
  IFACE=`echo $VALUE | awk '{print $8}'`
  route del default dev $IFACE
  COUNT=$(($COUNT+1))
done

# デフォルトルートを設定
$LOGGER -p user.info "ip-up.d: $PRGNAME: set default route. gw $5 dev $INTERFACE"
route add default gw $5 dev $INTERFACE

exit 0
