#!/bin/sh
# Copyright (c) 1999-2007 Gentoo Foundation
# Copyright (c) 2007-2009 Roy Marples <roy@marples.name>
# All rights reserved. Released under the 2-clause BSD license.

# This basically mounts $RC_SVCDIR as a ramdisk.
# The tricky part is finding something our kernel supports
# tmpfs and ramfs are easy, so force one or the other.
mount_svcdir()
{
	# mount from fstab if we can
	fstabinfo --mount "$RC_SVCDIR" && return 0

	local fs= fsopts="-o rw,noexec,nodev,nosuid"
	local svcsize=${rc_svcsize:-1024}

	# Some buggy kernels report tmpfs even when not present :(
	if grep -Eq "[[:space:]]+tmpfs$" /proc/filesystems; then
		mount -n -t tmpfs $fsopts,mode=755,size=${svcsize}k \
			rc-svcdir "$RC_SVCDIR" && return 0
	fi
	
	if grep -Eq "[[:space:]]+ramfs$" /proc/filesystems; then
		fs="ramfs"
		# ramfs has no special options
	elif [ -e /dev/ram0 ] \
		&& grep -Eq "[[:space:]]+ext2$" /proc/filesystems; then
		devdir="/dev/ram0"
		fs="ext2"
		dd if=/dev/zero of="$devdir" bs=1k count="$svcsize"
		mkfs -t "$fs" -i 1024 -vm0 "$devdir" "$svcsize"
	else
		echo
		eerror "OpenRC requires tmpfs, ramfs or a ramdisk + ext2"
		eerror "compiled into the kernel"
		echo
		return 1
	fi

	mount -n -t "$fs" $fsopts rc-svcdir "$RC_SVCDIR"
}

. "$RC_LIBEXECDIR"/sh/functions.sh
[ -r /etc/rc.conf ] && . /etc/rc.conf

# By default VServer already has /proc mounted, but OpenVZ does not!
# However, some of our users have an old proc image in /proc
# NFC how they managed that, but the end result means we have to test if
# /proc actually works or not. We to this by comparing uptime to one a second
# ago
mountproc=true
if [ -e /proc/uptime ]; then
	up="$(cat /proc/uptime)"
	sleep 1
	if [ "$up" = "$(cat /proc/uptime)" ]; then
		eerror "You have cruft in /proc that should be deleted"
	else
		einfo "/proc is already mounted, skipping"
		mountproc=false
	fi
fi

if $mountproc; then
	procfs="proc"
	[ "$RC_UNAME" = "GNU/kFreeBSD" ] && proc="linprocfs"
	ebegin "Mounting /proc"
	if ! fstabinfo --mount /proc; then
		mount -n -t "$procfs" -o noexec,nosuid,nodev proc /proc
	fi
	eend $?
fi

# Try to mount xenfs as early as possible, otherwise rc_sys() will always
# return RC_SYS_XENU and will think that we are in a domU while it's not.
if grep -Eq "[[:space:]]+xenfs$" /proc/filesystems; then
	ebegin "Mounting xenfs"
	if ! fstabinfo --mount /proc/xen; then
		mount -n -t xenfs xenfs /proc/xen -o nosuid,nodev,noexec
	fi
	eend $?
fi

# MA-E2xx

ledctrl()
{
        local leddir='/sys/class/leds/led_'$1
        local mode=$2
        
        if [ "${mode}" == "on" ]; then
                echo 'default-on' > ${leddir}/trigger
                echo 1 > ${leddir}/brightness
        elif [ "${mode}" == "off" ]; then
                echo 'default-on' > ${leddir}/trigger
                echo 0 > ${leddir}/brightness
        elif [ "${mode}" == "blink" ]; then
                local ontime=$3
                local offtime=$4

                echo 'timer' > ${leddir}/trigger
                echo $ontime > ${leddir}/delay_on
                echo $offtime > ${leddir}/delay_off
        fi
}

/sbin/hwclock --hctosys --local

ebegin "Mounting tmpfs at /tmp"
mount -n /tmp
result=$?
eend ${result}

ebegin "Mounting /sys"
mount -n -t sysfs -o noexec,nosuid,nodev sysfs /sys
eend $?

# Check if pushsw_init pressed
ebegin "Checking InitSwitch state"
initsw_status=`cat /proc/driver/psw_init`
eend ${initsw_status}

erase_settings=0
if [ "${initsw_status}" != "0" ]; then
        #echo "BLINK:0 R:200,G:200" > /tmp/ledctrl
        ledctrl g0 blink 200 200
        sleep 0.2
        ledctrl r0 blink 200 200

        echo " ### InitSwitch is pressed, checking 5 seconds ..."
        check_count=5

        while [ "${check_count}" -gt "0" ]; do
                initsw_status=`cat /proc/driver/psw_init`

                if [ "${initsw_status}" == "0" ]; then
                        #echo "SET:0 K" > /tmp/ledctrl
                        ledctrl g0 off
                        ledctrl r0 off
                        break
                fi

                check_count=$((check_count-1))
                echo " InitSwitch is pressed, ${check_count} seconds left..."
                sleep 1
        done

        if [ "${check_count}" == "0" ]; then
                echo " ### Erase current settings."
                #echo "BLINK:0 R:200,K:500" > /tmp/ledctrl
                ledctrl g0 off
                ledctrl r0 blink 200 500
                erase_settings=1
        fi
fi
                        
ebegin "Mounting /usr"
mount -n /usr
eend $?

ebegin "Mounting tmpfs at /.etc"
mount -n /.etc
result=$?
eend ${result}

if [ "${result}" == "0" ] && [ "${erase_settings}" == "0" ]; then
        archive_file=/tmp/etc.tar.gz
        ebegin "Loading /etc overlay contents from mtd"
        dd if=/dev/mtdblock2 of=$archive_file bs=65536 count=2 &> /dev/null
        if file $archive_file | grep "gzip compressed data" &> /dev/null; then
                cd /.etc &> /dev/null
                gzip -dcq $archive_file | tar xvfp - --atime-preserve &> /dev/null
                res=$?
                cd - &> /dev/null
                mount -n /etc
        else
                res=1
        fi
        eend ${res}
fi

if [ "${erase_settings}" == "1" ]; then
        ebegin "Clearing current settings"
        sleep 5
        #echo "SET:0 K" > /tmp/ledctrl
        ledctrl g0 off
        ledctrl r0 off
        eend 0
fi

ebegin "Update LED status"
#echo -e "SET:0 G\nSET:1 G\nSET:2 G\nSET:3 G\n" > /tmp/ledctrl
ledctrl g0 blink 250 250
ledctrl g1 off
ledctrl r0 off
ledctrl r1 off
eend 0

ebegin "FOMA power-on"
/opt/mae2xx/tools/foma_ctrl on &> /dev/null
eend 0

. "$RC_LIBEXECDIR"/sh/init-common-post.sh
