#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   Sendmail module
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
# Copyright (C) 2007 Century Systems Co.,Ltd.
#
# Last Modified: 2010/05/26 09:30:34 kikuchi
#

import smtplib
import poplib
import syslog

from email.MIMEText import MIMEText
from email.Header   import Header
from email          import Utils


#===============================================================================
#
# E-Mail libraly
#
#===============================================================================
class Maillib:
    def __init__(self):
        self.__setup_done = False

    def __log(self, msg):
        str = "(Maillib) %s" % msg
        syslog.syslog(syslog.LOG_INFO, str)
        print str
    #---------------------------------------------
    # アカウント情報設定
    #---------------------------------------------
    def setup_account(self, arg):
        try:
            info = {}

            info['enable'] = bool(arg['enable'])

            if info['enable'] == True:
                info['smtp_server'] = arg['smtp_server']
                info['name']        = arg['name']
                info['mail_addr']   = arg['mail_addr']
                info['auth_type']   = arg['auth_type']
                try:
                    info['smtp_port']   = int(arg['smtp_port'])
                except:
                    info['smtp_port']   = 25

                if info['auth_type'] == 'auth_pop':
                    info['pop3_server'] = arg['pop3_server']
                    try:
                        info['pop3_port']   = int(arg['pop3_port'])
                    except:
                        info['pop3_port']   = 110
                else:
                    info['pop3_server'] = ''
                    info['pop3_port']   = 0

                if (info['auth_type'] == 'auth_smtp' or
                    info['auth_type'] == 'auth_ssl' or
                    info['auth_type'] == 'auth_pop'):

                    info['account']     = arg['account']
                    info['password']    = arg['password']
                else:
                    info['account']     = ''
                    info['password']    = ''

            self.__account_info = info
        except Exception, msg:
            self.__log("setup_account() error: %s" % str(msg))
            self.__setup_done = False
            return False

        self.__log("setup_account(): setup is done.")
        self.__setup_done = True
        return True

    #---------------------------------------------
    # Send Email
    #---------------------------------------------
    def send_email(self, param):
        if not self.__setup_done:
            self.__log("send_email(): setup not done.")
            return False

        encoding = 'utf-8'

        msg = MIMEText(param['Content'].encode(encoding), 'plain', encoding)

        msg['Subject'] = param['Subject']
        msg['From']    = Utils.formataddr((Header(self.__account_info['name'], encoding).encode(),
                                          self.__account_info['mail_addr']))
        msg['To']      = ', '.join(param['To'])
        msg['Date']    = Utils.formatdate(None, True)

        host      = self.__account_info['smtp_server']
        port      = self.__account_info['smtp_port']
        from_addr = self.__account_info['mail_addr']
        user      = self.__account_info['account']
        password  = self.__account_info['password']
        auth_type = self.__account_info['auth_type']

        self.__log("smtp_server: %s, smtp_port: %d" % (host, port))

        try:
            s = smtplib.SMTP(host, port)

            if auth_type == 'auth_pop':
                # POP before SMTP認証
                self.__log("Auth: POP before SMTP")
                self.pop3_authenticate()
                #s.connect()
            elif auth_type == 'auth_smtp':
                # SMTP認証
                self.__log("Auth: SMTP Auth")
                s.ehlo()
                s.login(user, password)
            elif auth_type == 'auth_ssl':
                # SMTP over TLS認証 (Gmail)
                self.__log("Auth: SMTP over TLS")
                s.ehlo()
                s.starttls()
                s.ehlo()
                s.login(user, password)
            else:
                # 認証無し
                res = s.ehlo()

                # workaround: old MTA (BlackjumboDog)
                if res[0] != 250:
                    s.docmd('RSET')
                    s.helo()

            self.__log("sendmail")
            s.sendmail(from_addr, param['To'], msg.as_string())

            if host != 'smtp.gmail.com':
                self.__log("quit")
                s.quit()
            else:
                self.__log("close")
                s.close()
        except Exception, errMsg:
            self.__log("Maillib Error: '%s'" % str(errMsg))
            return False

        return True

    #---------------------------------------------
    # POP Before SMTP
    #---------------------------------------------
    def pop3_authenticate(self):
        try:
            host     = self.__account_info['pop3_server']
            port     = self.__account_info['pop3_port']
            user     = self.__account_info['account']
            password = self.__account_info['password']

            M = poplib.POP3(host, port)
            M.user(user)
            M.pass_(password)
            numMessages = len(M.list()[1])
            self.__log("MessageNums: %d" % numMessages)

            return True
        except Exception, msg:
            self.__log("POP_before_SMTP Error: '%s'" % str(msg))
            raise
