#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   Monitor (死活監視) Application module
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
# Copyright (C) 2007 Century Systems Co.,Ltd.
#
# Last Modified: 2010/05/26 10:30:47 kikuchi
#

import time
import Queue
import threading
import syslog

import MonitorThread
import SendMailThread
import NotifierThread


#===============================================================================
#
# Monitor Application module (I/F)
#
#===============================================================================
class MonitorApp(threading.Thread):
    def __init__(self):
        self.__running = False
        self.__monitorThread  = MonitorThread.MonitorThread()
        self.__notifierThread = NotifierThread.PeriodicNotifier()
        self.__sendmailThread = SendMailThread.SendMailThread()

        self.__email_info    = None
        self.__host_info     = None
        self.__email_target  = None
        self.__poll_info     = None
        self.__periodic_info = None
        self.__queue = Queue.Queue(20)

        self.__monitorThread.start()
        self.__notifierThread.start()
        self.__sendmailThread.start()

        self.__queue_monitor = self.__monitorThread.get_queue()
        self.__queue_notifier = self.__notifierThread.get_queue()
        self.__queue_mailer  = self.__sendmailThread.get_queue()

        self.__queue_monitor.put(('setup_mailer', self.__queue_mailer))
        self.__queue_notifier.put(('setup_mailer', self.__queue_mailer))

        threading.Thread.__init__(self)
        self.setDaemon(True)

    def __log(self, msg):
        str = "(MonitorApp) %s" % msg
        syslog.syslog(syslog.LOG_INFO, str)
        print str

    #-------------------------------------------------------
    # E-Mail アカウント設定
    #-------------------------------------------------------
    def setup_email_account(self, msg):
        self.__log("setup_email_account(): %s" % str(msg))

        param = msg
        if not 'email_info' in param:
            return False

        self.__email_info = param['email_info']
        self.__queue_mailer.put(['setup_email', param['email_info']])

        return True

    #-------------------------------------------------------
    # E-Mail 通知先アドレス設定
    #-------------------------------------------------------
    def setup_notify_email(self, msg):
        self.__log("setup_notify_email(): %s" % str(msg))

        param = msg
        if not 'email' in param:
            self.__log("setup_notify_email: not found 'email'.")
            return False

        self.__email_target = param['email']
        self.__queue_mailer.put(['setup_notify_email', self.__email_target])

    #-------------------------------------------------------
    # 監視対象ホスト設定
    #-------------------------------------------------------
    def setup_hosts(self, msg):
        self.__log("setup_hosts(): %s" % str(msg))

        param = msg
        if not 'host_info' in param:
            return False

        self.__host_info = param['host_info']
        self.__queue_monitor.put(['setup_hosts', self.__host_info])

        return True

    #-------------------------------------------------------
    # 監視条件設定
    #-------------------------------------------------------
    def setup_polling(self, msg):
        self.__log("setup_polling(): %s" % str(msg))

        param = msg
        if not 'poll_info' in param:
            return False

        self.__poll_info = param['poll_info']
        self.__queue_monitor.put(['setup_polling', self.__poll_info])

        return True

    #-------------------------------------------------------
    # 定期送信設定
    #-------------------------------------------------------
    def setup_periodic_notifier(self, msg):
        self.__log("setup_periodic_notifier(): %s" % str(msg))

        if not 'param' in msg:
            return False
        self.__periodic_info = msg['param']
        self.__queue_notifier.put(['setup_param', self.__periodic_info])
        return True

    #-------------------------------------------------------
    # 監視開始
    #-------------------------------------------------------
    def start_monitoring(self):
        self.__log("start_monitoring():")

        if (self.__email_info == None or self.__host_info == None or
            self.__email_target == None or self.__poll_info == None or
            self.__periodic_info == None):
            # 未設定情報あり、開始できません
            self.__log("start_monitoring(): !! can't start")
            return False

        # 既に開始している
        if self.__running == True:
            self.__log("start_monitoring(): Already started")
            return True

        self.__queue_monitor.put(['start_monitoring', ''])
        return True

    #-------------------------------------------------------
    # 監視停止
    #-------------------------------------------------------
    def stop_monitoring(self):
        self.__log("stop_monitoring():")

        # 停止中
        if self.__running == False:
            self.__log("stop_monitoring(): Already stopped")
            return True

        self.__queue_monitor.put(['stop_monitoring', ''])
        return True

    #-------------------------------------------------------
    # 監視状態取得
    #-------------------------------------------------------
    def get_monitoring_stat(self):
        return self.__running

    #-------------------------------------------------------
    # Queue
    #-------------------------------------------------------
    def get_queue(self):
        return self.__queue

    #-------------------------------------------------------
    # Kill Monitor,Email thread
    #-------------------------------------------------------
    def kill_threads(self):
        self.__queue_monitor.put(['exit_thread', ''])
        self.__queue_notifier.put(['exit_thread', ''])
        self.__queue_mailer.put(['exit_thread', ''])

        self.__monitorThread.join()
        self.__notifierThread.join()
        self.__sendmailThread.join()

    #-------------------------------------------------------
    # EntryPoint
    #-------------------------------------------------------
    def run(self):
        self.__log("start thread...")

        while True:
            msg = self.__queue.get()
            self.__log("message received: %s" % str(msg))

            cmd = msg[0]
            try:
                param = msg[1]
            except:
                param = None

            if cmd == 'setup_email_account':
                self.setup_email_account(param)
            elif cmd == 'setup_notify_email':
                self.setup_notify_email(param)
            elif cmd == 'setup_hosts':
                self.setup_hosts(param)
            elif cmd == 'setup_polling':
                self.setup_polling(param)
            elif cmd == 'setup_periodic_notifier':
                self.setup_periodic_notifier(param)
            elif cmd == 'start_monitoring':
                self.start_monitoring()
            elif cmd == 'stop_monitoring':
                self.stop_monitoring()
            elif cmd == 'exit_thread':
                self.__log("exit thread.")
                self.kill_threads()
                break
            else:
                self.__log("CMD '%s' is ignored" % cmd)

        self.__log("*** MonitoringApplication stop...")
