#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   Monitor (死活監視) MonitoringThread
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
# Copyright (C) 2007-2008 Century Systems Co.,Ltd.
#
# Last Modified: 2010/05/26 10:31:30 kikuchi
#

import sys
import time
import copy
import threading
import Queue
import syslog

from email.MIMEText import MIMEText
from email.Header   import Header
from email          import Utils

import Pinglib

#===============================================================================
#
# Monitor Application Main Thread module
#
#===============================================================================
class MonitorThread(threading.Thread):
    def __init__(self):
        self.__running = True
        self.__observe = False
        self.__timeout = 0

        self.__pingModule = Pinglib.Pinglib()

        self.__hostList    = None
        self.__poll_info   = None

        self.__queue = Queue.Queue(20)

        # 監視対象リスト
        self.__observeList = None

        # SendmailThread Receiver
        self.__sendmail    = None
        self.__name = 'MonitorThread'

        threading.Thread.__init__(self)
        self.setDaemon(True)

    def __log(self, msg):
        str = "(MonitorThread) %s" % msg
        syslog.syslog(syslog.LOG_INFO, str)
        print str

    #-------------------------------------------------------
    # Message Handler (setup Host list)
    #-------------------------------------------------------
    def get_queue(self):
        return self.__queue

    #-------------------------------------------------------
    # Message Handler (setup Host list)
    #-------------------------------------------------------
    def setup_hosts(self, msg):
        self.__log("setup_hosts: %s" % str(msg))
        self.__hostList = msg

    #-------------------------------------------------------
    # Message Handler (setup Polling info)
    #-------------------------------------------------------
    def setup_polling(self, msg):
        self.__log("setup_polling: %s" % str(msg))
        self.__poll_info = msg

    #-------------------------------------------------------
    # Message Handler (setup SendMail candygram obj)
    #-------------------------------------------------------
    def setup_mailer(self, msg):
        self.__log("setup_mailer: %s" % str(msg))
        self.__sendmail = msg

    #-------------------------------------------------------
    # Message Handler (start monitoring)
    #-------------------------------------------------------
    def start_monitoring(self):
        self.__log("start_monitoring:")

        if self.__hostList == []:
            self.__log("start_monitoring: Target host not registered.")
            return

        self.__observeList = []
        print self.__hostList

        # Search "Observe Enabled" host
        i = 0

        for host in self.__hostList:
            self.__log("No.%d: address = '%s'" %
                   (i + 1, host['address']))
            if host['enable'] == True:
                self.__log("  ---> Enabled")
                self.__observeList.append(copy.copy(host))
            else:
                self.__log("  ---> Disabled")
            i += 1

        self.__poll_info['hostNums'] = len(self.__observeList)

        #interval_sel = self.__poll_info['interval_sel']
        interval = self.__poll_info['interval']

        self.__poll_info['ping_interval'] = int(interval / max(self.__poll_info['hostNums'], 1))

        self.__log("# EnableHostNums:  %d" % self.__poll_info['hostNums'])
        self.__log("# PollingInterval: %d [s]" % interval)
        self.__log("# Ping Interval:   %d [s]" % self.__poll_info['ping_interval'])

        for host in self.__observeList:
            self.__log("host: %s" % host['address'])
            host['counter'] = self.__poll_info['retry_max']

        #print self.__observeList

        self.__timeout = int(self.__poll_info['ping_interval'])
        self.__poll_info['idx'] = 0

    #-------------------------------------------------------
    # Message Handler (stop monitoring)
    #-------------------------------------------------------
    def stop_monitoring(self):
        self.__log("stop_monitoring:")
        self.__timeout = 0

    #-------------------------------------------------------
    # EntryPoint
    #-------------------------------------------------------
    def run(self):
        self.__log("start...")

        while self.__running:
            if self.__timeout == 0:
                self.__log("timeout = 0...")
                res = self.__queue.get(True)
            else:
                try:
                    self.__log("timeout = %s..." % self.__timeout)
                    res = self.__queue.get(True, self.__timeout)
                except:
                    res = None

            if res == None:
                # Timeout...
                self.__log("timeout...")
                self.observe()
            else:
                self.__log("msg received. '%s'" % str(res))
                cmd = res[0]
                try:
                    msg = res[1]
                except:
                    msg = None

                if cmd == 'setup_hosts':
                    self.setup_hosts(msg)
                elif cmd == 'setup_polling':
                    self.setup_polling(msg)
                elif cmd == 'setup_mailer':
                    self.setup_mailer(msg)
                elif cmd == 'start_monitoring':
                    self.start_monitoring()
                elif cmd == 'stop_monitoring':
                    self.stop_monitoring()
                elif cmd == 'exit_thread':
                    break
                else:
                    self.__log("!! command '%s' is ignored" % cmd)

        self.__log("*** MonitorThread stop...")

    #-------------------------------------------------------
    # 監視メイン
    #-------------------------------------------------------
    def observe(self):
        idx = self.__poll_info['idx']
        self.__log("observe() called, idx = %d" % idx)

        if len(self.__observeList) == 0:
            return

        host = self.__observeList[idx]

        # Ping! (次リリースでは Impacket に変更)
        result = self.__pingModule.observe(host['address'])

        if result == False:
            # Timeout
            if host['counter'] > 0:
                host['counter'] -= 1

                self.__log("host: %s counter left: %d" % \
                        (host['address'], host['counter']))

                if host['counter'] <= 0:
                    self.__log("Host (%s) down detected. --> %s" %
                              (host['address'], self.__sendmail))

                    self.__sendmail.put(['sendmail', (host, False)])
        else:
            # Ping OK
            if host['counter'] <= 0:
                self.__log("Host (%s) recover detected. --> %s" %
                          (host['address'], self.__sendmail))

                self.__sendmail.put(['sendmail', (host, True)])

            # Reset Retry Counter
            host['counter'] = self.__poll_info['retry_max']

        self.__poll_info['idx'] += 1

        if self.__poll_info['idx'] >= len(self.__observeList):
            self.__poll_info['idx'] = 0

        return
