#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   NotifierThread
# Copyright (C) 2007-2009 Century Systems Co.,Ltd.
#

import sys
import time
import copy
import threading
import Queue
import syslog

from email.MIMEText import MIMEText
from email.Header   import Header
from email          import Utils

class PeriodicNotifier(threading.Thread):
    def __init__(self):
        self.__running = True
        self.__timeout = 0
        self.__queue = Queue.Queue(20)
        self.__name = 'PeriodicNotifier'
        self.__param = None
        self.__sendmail = None
        threading.Thread.__init__(self)
        self.setDaemon(True)

    def __log(self, msg):
        str = "(%s) %s" % (self.__name, msg)
        syslog.syslog(syslog.LOG_INFO, str)
        print str

    def get_queue(self):
        return self.__queue

    def setup_param(self, param):
        self.__log("set_param(): %s" % str(param))
        self.__param = param
        if self.__param.get('enable', False) == True:
            interval = self.__param.get('interval', 1440)
            self.__log("timer enabled. interval = %d." % interval)
            self.__timeout = int(interval) * 60
        else:
            self.__log("timer disabled.")
            self.__timeout = 0

    def setup_mailer(self, mailer):
        self.__sendmail = mailer

    def run(self):
        self.__log("start thread")

        while self.__running:
            if self.__timeout == 0:
                res = self.__queue.get(True)
            else:
                try:
                    res = self.__queue.get(True, self.__timeout)
                except:
                    res = None

            if res == None:
                self.__log("timeout")
                self.__sendmail.put(['sendmail', 'PeriodicNotifier'])
            else:
                self.__log("msg received. '%s'" % str(res))
                cmd = res[0]
                try:
                    msg = res[1]
                except:
                    msg = None

                if cmd == 'setup_param':
                    self.setup_param(msg)

                elif cmd == 'setup_mailer':
                    self.setup_mailer(msg)

                elif cmd == 'exit_thread':
                    break

                else:
                    self.__log("unknown command: '%s'" % cmd)

        self.__log("finish thread")
