#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   Monitor (死活監視) SendMailThread
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
# Copyright (C) 2007-2008 Century Systems Co.,Ltd.
#
# Last Modified: 2010/05/26 13:59:39 kikuchi
#

import sys
import threading
import time
import Queue
import syslog
import commands

from email.MIMEText import MIMEText
from email.Header   import Header
from email          import Utils

import Maillib

EXT_CONTENT_COMMAND = '/etc/opt/mailcontent.sh'


#===============================================================================
#
# Monitor Application SendMail Thread module
#
#===============================================================================
class SendMailThread(threading.Thread):
    def __init__(self):
        self.__running = True
        self.__mailModule = Maillib.Maillib()

        self.__email_info    = None
        self.__notify_target = None

        self.__monitorFormat = ''.join(( "ActionEvent: %s\r\n",
                                  "HostName:    %s\r\n",
                                  "NodeAddress: %s\r\n",
                                  "Timestamp:   %s\r\n"))

        self.__notifierFormat = ''.join(("%s\r\n",
                                    "This is a periodic notification.\r\n"))

        self.__queue = Queue.Queue(20)

        threading.Thread.__init__(self)
        self.setDaemon(True)

    def __log(self, msg):
        str = "(SendMailThread) %s" % msg
        syslog.syslog(syslog.LOG_INFO, str)
        print str

    #-------------------------------------------------------
    # Message Handler (setup Host list)
    #-------------------------------------------------------
    def get_queue(self):
        return self.__queue

    #-------------------------------------------------------
    # Message Handler (setup E-Mail account)
    #-------------------------------------------------------
    def setup_email(self, msg):
        self.__log("setup_email: %s" % str(msg))

        self.__email_info = msg
        self.__mailModule.setup_account(self.__email_info)

    #-------------------------------------------------------
    # Message Handler (setup E-Mail send Address)
    #-------------------------------------------------------
    def setup_notify_email(self, msg):
        self.__log("setup_notify_email: %s" % str(msg))

        # 送信先をリストで受け取る
        self.__notify_target = msg

    #-------------------------------------------------------
    # Message Handler (sendmail)
    #-------------------------------------------------------
    def sendmail(self, msg):
        self.__log("sendmail: %s" % str(msg))

        try:
            content = msg

            # メール送信先
            addr_sendto = []

            for entry in self.__notify_target:
                # 有効なメール通知先検索
                if entry.get('enable', False) == True and \
                       entry.get('mail_addr', '') != '':
                    addr_sendto.append(entry['mail_addr'])

            if len(addr_sendto) == 0:
                self.__log("E-mail enabled host isn't found, sendmai() is aborted.")
                return

            if content == "PeriodicNotifier":
                # 定期送信
                param = {}
                try:
                    res = commands.getoutput(EXT_CONTENT_COMMAND)
                    param['Content'] = res
                except:
                    param['Content'] = self.__notifierFormat % \
                                        (Utils.formatdate(None, True))

                param['Subject'] = "[MA-E2xx Notifier] Periodic Notification"
                param['To'] = addr_sendto

            else:
                # 死活監視
                # ダウンを検知したホストの (IPアドレス, ホスト名) を
                # 受け取り、メッセージを作ってメール送信
                addr   = content[0]['address']
                host   = content[0]['name']

                if content[1] == True:
                    updown = 'up'
                else:
                    updown = 'down'

                mail_target = []

                param = {}
                param['Content'] = self.__monitorFormat % (updown, host,
                                                addr, Utils.formatdate(None, True))
                param['Subject'] = "[MA-E2xx Monitor] %s %s" % (host, updown)
                param['To']      = addr_sendto

            self.__log("---> sendmail: param = %s" % str(param))
            self.__mailModule.send_email(param)

        except Exception, err:
            self.__log("sendmail error: '%s'" % str(err))

    #-------------------------------------------------------
    # EntryPoint
    #-------------------------------------------------------
    def run(self):
        self.__log("start...")

        while self.__running:
            res = self.__queue.get()

            cmd = res[0]
            try:
                param = res[1]
            except:
                param = None

            if cmd == 'setup_email':
                self.setup_email(param)
            elif cmd == 'setup_notify_email':
                self.setup_notify_email(param)
            elif cmd == 'sendmail':
                self.sendmail(param)
            elif cmd == 'exit_thread':
                break
            else:
                self.__log("cmd '%s' is ignored" % cmd)

        self.__log("*** SendMailThread stop...")
