#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx Web Application Server Module
#   Database Models
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
#
# Copyright (C) 2007-2008 Century Systems Co.,Ltd.
# Last Modified: 2012/12/10 11:57:00 kikuchi
#

import syslog

from sqlalchemy import *
from sqlalchemy import sql
from sqlalchemy.orm import *
from dbconn import Session, metadata, engine

# YAML
import yaml
try:
    from yaml import CLoader as Loader
    from yaml import CDumper as Dumper
    print "*** PyYAML: CLoader and CDumper loaded."
except ImportError:
    from yaml import Loader, Dumper
    print "*** PyYAML: CLoader and/or CDumper load failed."


#===============================================================================
#
# Base Model class
#
#===============================================================================
class BaseModel(object):
    def __init__(self, args):
        for key in self.c.keys():
            try:
                self.__dict__[key] = args[key]
            except:
                pass

    ## Object --> dictionary
    def get_dict(self):
        res = {}

        for key in self.c.keys():
            res[key] = self.__dict__.get(key, '')

        return res

    ## update content
    def update(self, args):
        if type(args) == self.__class__:
            new_args = args.get_dict()
        else:
            new_args = args

        for key in self.c.keys():
            if key in new_args:
                #self.__dict__[key] = new_args[key]
                exec("self.%s = new_args[key]" % key)

    def __repr__(self):
        res = self.get_dict()
        return "<%s: '%s'>" % (self.__class__.__name__, str(res))

    ## insert or update (call from derived class)
    @classmethod
    def _insert_or_update(self, arg, filter, primary_key):
        if type(arg) == self:
            dict_arg = arg.get_dict()
        elif type(arg) == dict:
            dict_arg = arg
        else:
            raise TypeError, ("class %s or dictionary required." % self.__name__)

        # select by primary_key
        try:
            pk = dict_arg[primary_key]
            cmdline = "q = Session.query(self).filter(\"%s = '%s'\")" % (filter, pk)
            exec(cmdline)
            count = q.count()
        except:
            count = 0

        try:
            if count == 1:
                mode = 'update'
                new_eth = q.one()
                new_eth.update(dict_arg)
            else:
                mode = 'insert'
                new_eth = self(dict_arg)
                Session.save(new_eth)

            Session.commit()
            return (True, mode)
        except Exception, msg:
            Session.close()
            return (False, msg)

    ## delete one entry (call from derived class)
    @classmethod
    def _delete(self, arg, filter, primary_key):
        if type(arg) == self:
            dict_arg = arg.get_dict()
        elif type(arg) == dict:
            dict_arg = arg
        else:
            raise TypeError, ("class %s or dictionary required." % self.__name__)

        # select by primary_key
        try:
            pk = dict_arg[primary_key]
            cmdline = "q = Session.query(self).filter(\"%s = '%s'\")" % (filter, pk)
            exec(cmdline)
            count = q.count()
        except:
            count = 0

        if count == 1:
            # DELETE
            Session.delete(q.one())
            Session.commit()
            return True
        else:
            return False

    @classmethod
    def select_all(self):
        q = Session.query(eval(self.__name__)).all()
        Session.close()

        res = []
        for entry in q:
            res.append(entry.get_dict())

        return res

    @classmethod
    def delete_all(self):
        tuples = Session.query(eval(self.__name__)).all()
        nums = len(tuples)

        for entry in tuples:
            Session.delete(entry)

        Session.commit()
        return nums

    # load from list of dictionary
    @classmethod
    def load(self, arg):
        if type(arg) != list and type(data) != tuple:
            raise TypeError, "list or tuple required."

        for entry in arg:
            if type(entry) == dict:
                obj = eval("%s(entry)" % self.__name__)
                Session.save(obj)

        Session.commit()


#===============================================================================
#
# Table definitions
#
#===============================================================================

#---------------------------------------------------------------------
# Interfaces table
#---------------------------------------------------------------------
interfaces = Table('interfaces', metadata,
                   Column('name', String, primary_key = True),
                   #--- 0: Ethernet / 1: PPP / 2: GRE
                   Column('type', Integer, nullable = False),
                   Column('info', String, default = ''),
                   Column('erasable', Boolean, default = True))

class Interface(BaseModel):
    @classmethod
    def select_all(self):
        q = Session.query(Interface).order_by(interfaces.c.name).all()
        Session.close()

        res = []
        for entry in q:
            res.append(entry.get_dict())

        return res

    @classmethod
    def select_by_intf(self, intf):
        q = Session.query(Interface).filter(interfaces.c.name == intf)
        Session.close()

        if q.count() != 1:
            return None

        if_info = q.one()

        if if_info.type == 0:
            # Ethernet
            if_type = 'Ethernet'
            if_conf = if_info.ethernet[0]
        elif if_info.type == 1:
            # PPP
            if_type = 'PPP'
            if_conf = if_info.ppp[0]
        else:
            # ???
            return None

        return {'type' : if_type, 'conf' : if_conf.get_dict()}


#---------------------------------------------------------------------
# Ethernet configuration table
#---------------------------------------------------------------------
ethernet = Table('ethernet', metadata,
                 Column('eth_id', Integer, primary_key = True),
                 Column('name', String, ForeignKey('interfaces.name'), unique = True),
                 Column('address', String, nullable = False),
                 Column('prefixlen', Integer, nullable = False),
                 Column('gateway', String, default = ''),
                 Column('use_dhcp', Boolean, default = False),
                 # 2^0 : release / 2^1 : nodns
                 # 2^2 : nontp   / 2^3 : nogateway
                 Column('dhcp_opts', Integer, default = 0),
                 Column('up', Boolean, default = True))

class Ethernet(BaseModel):
    @classmethod
    def select_all(self):
        q = Session.query(Ethernet).order_by(ethernet.c.name).all()
        Session.close()

        res = []
        for entry in q:
            res.append(entry.get_dict())

        return res

    @classmethod
    def select_by_intf(self, intf):
        q = Session.query(Ethernet).filter(ethernet.c.name == intf)

        if q.count() == 1:
            return q.one()
        else:
            return None

    @classmethod
    def insert_or_update(self, arg):
        return self._insert_or_update(arg, 'ethernet.name', 'name')

#---------------------------------------------------------------------
# PPP(callout) configuration table
#---------------------------------------------------------------------
ppp = Table('ppp', metadata,
            Column('unit', Integer, primary_key = True),
            Column('name', String, ForeignKey('interfaces.name'), unique = True),
            Column('is_modem', Boolean, default = True),
            Column('conn_name', String),
            Column('domain', String, default = ''),
            Column('dev_name', String, nullable = False),
            Column('dev_node', String, nullable = False),
            #--- 0: normal / 1: persist / 2:demand
            Column('conn_type', Integer, nullable = False),
            Column('idle_time', Integer, default = 0,),
            Column('ip_local', String, default = '0.0.0.0'),
            Column('ip_remote', String, default = '0.0.0.0'),
            Column('lcp_check', Boolean, default = True),
            Column('lcp_interval', Integer, default = 30),
            Column('lcp_failure', Integer, default = 3),
            Column('defaultroute', Boolean, default = False),
            Column('usepeerdns', Boolean, default = True),
            Column('modeminit', String, default = 'AT&FS0=0'),
            Column('phone_number', String, default = ''),
            Column('pulseok', Boolean, default = True),
            Column('tone', Boolean, default = True),
            Column('vjcomp', Boolean, default = True),
            Column('deflatecomp', Boolean, default = True),
            Column('mtu', Integer, default = 1454),
            Column('apn', String, default = ''),
            Column('pdptype', Integer, default = 0),
            Column('ip_netmask', String, default = '0.0.0.0'),
            Column('useaccount', Boolean, default = True),
            Column('need_apn', Boolean, default = False),
            Column('no_persist', Boolean, default = False),
            Column('force_lcp_check', Boolean, default = False),
            Column('cipl_usable', Boolean, default = False),
            Column('cipl_sms', Boolean, default = False),
            Column('cipl_domain', String, default = ''))

class PPP(BaseModel):
    @classmethod
    def select_all(self):
        q = Session.query(PPP).order_by(ppp.c.unit).all()
        Session.close()

        res = []
        for entry in q:
            res.append(entry.get_dict())

        return res

    @classmethod
    def insert_or_update(self, arg):
        return self._insert_or_update(arg, 'ppp.unit', 'unit')

    @classmethod
    def delete(self, arg):
        return self._delete(arg, 'ppp.unit', 'unit')


#---------------------------------------------------------------------
# PPP(dialin) configuration table
#---------------------------------------------------------------------
dialin = Table('dialin', metadata,
               Column('unit', Integer, default = 50, primary_key = True),
               Column('name', String, ForeignKey('interfaces.name'),
                      default = 'ppp50'),
               Column('dev_name', String, nullable = False),
               Column('dev_node', String, nullable = False),
               Column('enable', Boolean, default = False),
               Column('modeminit', String, default = 'AT&FS0=0'),
               Column('ip_local', String, default = '0.0.0.0'),
               Column('ip_remote', String, default = '0.0.0.0'),
               Column('ip_netmask', String, default = '0.0.0.0'),
               Column('idle_time', Integer, default = 0),
               Column('ipcp_dns', Integer, default = 0),
               Column('primary_dns', String, default = ''),
               Column('secondary_dns', String, default = ''),
               Column('defaultroute', Boolean, default = False),
               Column('lcp_check', Boolean, default = True),
               Column('lcp_interval', Integer, default = 30),
               Column('lcp_failure', Integer, default = 3),
               Column('auth_method', Boolean, default = True),
               Column('vjcomp', Boolean, default = True),
               Column('deflatecomp', Boolean, default = True),
               Column('mtu', Integer, default = 1500),
               Column('useaccount', Boolean, default = False),
               Column('user', String))

class DialIn(BaseModel):
    @classmethod
    def insert_or_update(self, arg):
        return self._insert_or_update(arg, 'dialin.unit', 'unit')


#---------------------------------------------------------------------
# PPP(OCN IPv6) configuration table
#---------------------------------------------------------------------
ocnipv6 = Table('ocnipv6', metadata,
                Column('unit', Integer, default = 100, primary_key = True),
                Column('name', String, default = 'ppp100'),
                Column('l2tp_server', String, nullable = False),
                Column('l2tp_port', Integer, default = 1701),
                Column('interface', String, ForeignKey('ethernet.name'),
                       nullable = False),
                Column('use_ocnv6', Boolean, default = False),
                Column('auto_connect', Boolean, default = False))

class OCNIPv6(BaseModel):
    @classmethod
    def insert_or_update(self, arg):
        return self._insert_or_update(arg, 'ocnipv6.unit', 'unit')

#---------------------------------------------------------------------
# GRE table
#---------------------------------------------------------------------
gre = Table('gre', metadata,
            Column('gre_id', Integer, primary_key = True),
            Column('name', String, ForeignKey('interfaces.name'), unique = True),
            Column('conn_name', String, default = ''),
            Column('remote_ip', String, nullable = False),
            Column('local_ip', String, default = ''),
            Column('if_address', String, default = ''),
            Column('ttl', Integer, default = 255),
            Column('mtu', Integer, default = 1500))

class Gre(BaseModel):
    @classmethod
    def select_all(self):
        q = Session.query(Gre).order_by(gre.c.gre_id).all()
        Session.close()

        res = []
        for entry in q:
            res.append(entry.get_dict())

        return res

    @classmethod
    def select_by_intf(self, intf):
        q = Session.query(Gre).filter(gre.c.name == intf)

        if q.count() == 1:
            return q.one()
        else:
            return None

    @classmethod
    def insert_or_update(self, arg):
        return self._insert_or_update(arg, 'gre.name', 'name')

#---------------------------------------------------------------------
# StaticRoute table
#---------------------------------------------------------------------
routes = Table('routes', metadata,
               Column('route_id', Integer, primary_key = True),
               Column('address', String, nullable = False),
               Column('prefixlen', Integer, nullable = False),
               Column('gw_address', String, nullable = False),
               Column('gw_dev', String, ForeignKey('interfaces.name')),
               ### for policy routing (future)
               Column('rule', String, default = 'main'),
               Column('proto', String, default = 'boot'))

class Route(BaseModel):
    @classmethod
    def select_all(self):
        q = Session.query(Route).order_by(routes.c.gw_dev).all()
        Session.close()

        res = []
        for entry in q:
            res.append(entry.get_dict())

        return res

    @classmethod
    def select_by_intf(self, intf):
        q = Session.query(Route).filter(routes.c.gw_dev == intf).all()
        Session.close()

        res = []
        for entry in q:
            res.append(entry.get_dict())

        return res

    @classmethod
    def insert_or_update(self, arg):
        return self._insert_or_update(arg, 'routes.route_id', 'route_id')

    @classmethod
    def delete(self, arg):
        return self._delete(arg, 'routes.route_id', 'route_id')


#---------------------------------------------------------------------
# PPP secrets (chap-secrets, pap-secrets) table
#---------------------------------------------------------------------
secrets = Table('secrets', metadata,
                Column('unit', Integer,
                       ForeignKey('ppp.unit'),
                       ForeignKey('dialin.unit'),
                       ForeignKey('ocnipv6.unit'),
                       nullable = False),
                Column('account', String, nullable = False),
                Column('password', String, default = ''),
                PrimaryKeyConstraint('unit', 'account'))

class Secret(BaseModel):
    @classmethod
    def select_all(self):
        q = Session.query(Secret).order_by(secrets.c.unit).all()
        Session.close()

        res = []
        for entry in q:
            res.append(entry.get_dict())

        return res

    @classmethod
    def select_by_unit(self, unit):
        q = Session.query(Secret).filter(secrets.c.unit==unit).\
            order_by(secrets.c.account).all()
        Session.close()

        res = []
        for entry in q:
            res.append({'account'  : entry.account,
                        'password' : entry.password})

        return res

    @classmethod
    def delete_by_unit(self, unit):
        count = 0
        q = Session.query(Secret).filter(secrets.c.unit==unit)

        if q.count() > 0:
            for entry in q.all():
                Session.delete(entry)
                count += 1

            Session.commit()

        return count

    @classmethod
    def insert(self, arg):
        secret = Secret(arg)

        Session.save(secret)
        Session.commit()


#---------------------------------------------------------------------
# Shorewall(Firewall) Interfaces table
#---------------------------------------------------------------------
fw_interfaces = Table('fw_interfaces', metadata,
                      Column('name', String, ForeignKey('interfaces.name'),
                             primary_key = True),
                      Column('zone', String, default = 'net'))

class FW_interface(BaseModel):
    @classmethod
    def select_all(self):
        q = Session.query(FW_interface).order_by(fw_interfaces.c.name).all()
        Session.close()

        res = []
        for entry in q:
            res.append(entry.get_dict())

        return res

    @classmethod
    def insert_or_update(self, arg):
        return self._insert_or_update(arg, 'fw_interfaces.name', 'name')

    @classmethod
    def delete(self, arg):
        return self._delete(arg, 'fw_interfaces.name', 'name')


#---------------------------------------------------------------------
# Shorewall(Firewall) masq(SNAT) table
#---------------------------------------------------------------------
fw_masqs = Table('fw_masqs', metadata,
                 Column('masq_id', Integer, primary_key = True),
                 Column('dst_if', String, ForeignKey('interfaces.name'),
                        nullable = False),
                 Column('src_if', String, ForeignKey('interfaces.name'),
                        nullable = False),
                 UniqueConstraint('dst_if', 'src_if'))

class FW_masq(BaseModel):
    @classmethod
    def select_all(self):
        q = Session.query(FW_masq).order_by(fw_masqs.c.masq_id).all()
        Session.close()

        res = []
        for entry in q:
            res.append(entry.get_dict())

        return res

    @classmethod
    def insert_or_update(self, arg):
        return self._insert_or_update(arg, 'fw_masqs.masq_id', 'masq_id')

    @classmethod
    def delete(self, arg):
        return self._delete(arg, 'fw_masqs.masq_id', 'masq_id')


#---------------------------------------------------------------------
# Shorewall(Firewall) rule table
#---------------------------------------------------------------------
fw_rules = Table('fw_rules', metadata,
                 Column('rule_id', Integer, primary_key = True),
                 Column('src_address', String, default = ''),
                 Column('src_port', Integer, default = 0),
                 Column('dst_address', String, default = ''),
                 Column('dst_port', Integer, default = 0),
                 Column('svc_macro', String, default = 'Port/Protocol'),
                 Column('svc_protocol', String, default = ''),
                 Column('svc_port', Integer, default = 0),
                 Column('action', String, nullable = False),
                 Column('original_dest', String, default = ''))

class FW_rule(BaseModel):
    @classmethod
    def select_all(self):
        q = Session.query(FW_rule).order_by(fw_rules.c.rule_id).all()
        Session.close()

        res = []
        for entry in q:
            res.append(entry.get_dict())

        return res

    @classmethod
    def insert_or_update(self, arg):
        return self._insert_or_update(arg, 'fw_rules.rule_id', 'rule_id')

    @classmethod
    def delete(self, arg):
        return self._delete(arg, 'fw_rules.rule_id', 'rule_id')

#---------------------------------------------------------------------
# Shorewall(Firewall) nat table
#---------------------------------------------------------------------
fw_nat = Table('fw_nat', metadata,
                Column('nat_id', Integer, primary_key = True),
                Column('ext_address', String, nullable = False),
                Column('interface', String, ForeignKey('interfaces.name'),
                    nullable = False),
                Column('int_address', String, nullable = False))

class FW_nat(BaseModel):
    @classmethod
    def select_all(self):
        q = Session.query(FW_nat).order_by(fw_nat.c.nat_id).all()
        Session.close()

        res = []
        for entry in q:
            res.append(entry.get_dict())

        return res

    @classmethod
    def insert_or_update(self, arg):
        return self._insert_or_update(arg, 'fw_nat.nat_id', 'nat_id')

    @classmethod
    def delete(self, arg):
        return self._delete(arg, 'fw_nat.nat_id', 'nat_id')

#---------------------------------------------------------------------
# Monitor Host table
#---------------------------------------------------------------------
mon_hosts = Table('mon_hosts', metadata,
                  Column('address', String, primary_key = True),
                  Column('name', String),
                  Column('enable', Boolean, default = False))

class Mon_host(BaseModel):
    @classmethod
    def insert_or_update(self, arg):
        return self._insert_or_update(arg, 'mon_hosts.address', 'address')

    @classmethod
    def delete(self, arg):
        return self._delete(arg, 'mon_hosts.address', 'address')


#---------------------------------------------------------------------
# Monitor mail table
#---------------------------------------------------------------------
mon_mails = Table('mon_mails', metadata,
                  Column('mail_addr', String, primary_key = True),
                  Column('enable', Boolean, default = False))

class Mon_mail(BaseModel):
    @classmethod
    def insert_or_update(self, arg):
        return self._insert_or_update(arg, 'mon_mails.mail_addr', 'mail_addr')

    @classmethod
    def delete(self, arg):
        return self._delete(arg, 'mon_mails.mail_addr', 'mail_addr')


#---------------------------------------------------------------------
# PING test target table
#---------------------------------------------------------------------
ping_hosts = Table('ping_hosts', metadata,
                   Column('address', String, primary_key = True),
                   Column('name', String))

class PING_host(BaseModel):
    @classmethod
    def insert_or_update(self, arg):
        return self._insert_or_update(arg, 'ping_hosts.address', 'address')

    @classmethod
    def delete(self, arg):
        return self._delete(arg, 'ping_hosts.address', 'address')


#---------------------------------------------------------------------
# ser2net port table
#---------------------------------------------------------------------
ser2net = Table('ser2net', metadata,
                Column('ser2net_seq', Integer, primary_key = True),
                Column('port', Integer, unique = True),
                Column('state', Integer, nullable = False),
                Column('timeout', Integer, default = 600),
                Column('dev_node', String, nullable = False),
                Column('dev_name', String, nullable = False),
                Column('bps', Integer, default = 115200),
                Column('parity', String, default = 'none'),
                Column('stopbits', Integer, default = 1),
                Column('databits', Integer, default = 8),
                Column('flow', String, default = 'none'),
                Column('local', Boolean, default = False),
                Column('remctl', Boolean, default = False))

class Ser2net(BaseModel):
    @classmethod
    def select_all(self):
        q = Session.query(Ser2net).order_by(ser2net.c.ser2net_seq).all()
        Session.close()

        res = []
        for entry in q:
            res.append(entry.get_dict())

        return res

    @classmethod
    def insert_or_update(self, arg):
        return self._insert_or_update(arg, 'ser2net.ser2net_seq', 'ser2net_seq')

    @classmethod
    def delete(self, arg):
        return self._delete(arg, 'ser2net.ser2net_seq', 'ser2net_seq')

#---------------------------------------------------------------------
# Generic configurations table
#---------------------------------------------------------------------
configs = Table('configs', metadata,
                Column('kind', String, nullable = False, default = 'Generic'),
                Column('key', String, nullable = False),
                Column('value', String),
                PrimaryKeyConstraint('kind', 'key'))

class ConfigDB(BaseModel):
    @classmethod
    def select_all(self):
        entries = Session.query(ConfigDB).all()
        Session.close()

        res = {}
        for entry in entries:
            kind = str(entry.kind)

            if not kind in res:
                res[kind] = {}

            res[kind][str(entry.key)] = entry.value

        return res

    @classmethod
    def select_all_casted(self):
        entries = Session.query(ConfigDB).all()
        Session.close()

        res = {}
        for entry in entries:
            kind = entry.kind

            if not kind in res:
                res[kind] = {}

            try:
                value = int(entry.value)
            except ValueError:
                value = entry.value
            except TypeError:
                value = ''

            res[kind][entry.key] = value

        return res

    @classmethod
    def load(self, arg):
        inserted = 0

        for (kind, entries) in arg.iteritems():
            for (key, value) in entries.iteritems():
                entry = ConfigDB({'kind' : kind, 'key' : key, 'value' : value})
                Session.save(entry)
                inserted += 1

        if inserted > 0:
            Session.commit()

    @classmethod
    def get_kinds(self):
        conn = engine.connect()

        s = sql.text("select distinct kind from configs")
        res = conn.execute(s)

        kinds = []
        for entry in res.fetchall():
            kinds.append(entry[0])

        return kinds

    @classmethod
    def select_by_kind(self, kind):
        entries = Session.query(ConfigDB).filter(configs.c.kind == kind)

        if entries.count() == 0:
            return {}

        res = {}
        for entry in entries.all():
            try:
                val = int(entry.value)
            except ValueError:
                val = entry.value
            except TypeError:
                val = ''

            res[entry.key] = val

        return res
        
    @classmethod
    def select_by_kind_raw(self, kind):
        entries = Session.query(ConfigDB).filter(configs.c.kind == kind)

        if entries.count() == 0:
            return {}

        res = {}
        for entry in entries.all():
            res[str(entry.key)] = entry.value

        return res

    @classmethod
    def select_by_kind_key(self, kind, key):
        entry = Session.query(ConfigDB).filter(configs.c.kind == kind).filter(configs.c.key == key)

        if entry.count() == 0:
            return None

        try:
            value = int(entry.one().value)
        except ValueError:
            value = entry.one().value
        except TypeError:
            val = ''

        return value

    @classmethod
    def select_by_kind_key_raw(self, kind, key):
        entry = Session.query(ConfigDB).filter(configs.c.kind == kind).filter(configs.c.key == key)

        if entry.count() == 0:
            return None

        return entry.one().value

    @classmethod
    def set(self, kind, key, value):
        entry = Session.query(ConfigDB).filter(configs.c.kind == kind).filter(configs.c.key == key)

        if value == None:
            value = ''

        if entry.count() > 0:
            #-- update
            obj = entry.one()
            print "*** UPDATE"
            #print ("*** UPDATE from '%s' --> (kind: %s, key: %s)" % (obj, kind, key))
            obj.value = value
            Session.update(obj)
        else:
            #-- insert
            print "*** INSERT"
            new_entry = ConfigDB({'kind' : kind, 'key' : key, 'value' : value})
            Session.save(new_entry)

        Session.commit()

    @classmethod
    def delete(self, kind, key):
        entry = Session.query(ConfigDB).filter(configs.c.kind == kind).filter(configs.c.key == key)
        result = False

        if entry.count() == 1:
            obj = entry.one()
            Session.delete(obj)
            Session.commit()
            result = True

        return result


#===============================================================================
#
# Joined View
#
#===============================================================================

#---------------------------------------------------------------------
# Interface --> Ethernet
#---------------------------------------------------------------------
join_ethernet = join(interfaces, ethernet).join(fw_interfaces)

class vEthernet(BaseModel):
    @classmethod
    def select_by_intf(self, intf):
        q = Session.query(vEthernet).filter(ethernet.c.name == intf)
        Session.close()

        if q.count() == 1:
            return q.one()
        else:
            return None

    @classmethod
    def insert_or_update(self, arg):
        return self._insert_or_update(arg, 'ethernet.name', 'name')


#---------------------------------------------------------------------
# Interface --> Gre
#---------------------------------------------------------------------
join_gre = join(interfaces, gre).join(fw_interfaces)

class vGre(BaseModel):
    @classmethod
    def select_all(self):
        q = Session.query(vGre).order_by(gre.c.gre_id).all()
        Session.close()

        res = []
        for entry in q:
            res.append(entry.get_dict())

        return res

    @classmethod
    def select_by_intf(self, intf):
        q = Session.query(vGre).filter(gre.c.name == intf)
        Session.close()

        if q.count() == 1:
            return q.one()
        else:
            return None

    @classmethod
    def insert_or_update(self, arg):
        return self._insert_or_update(arg, 'gre.name', 'name')

    @classmethod
    def delete(self, arg):
        return self._delete(arg, 'gre.name', 'name')


#---------------------------------------------------------------------
# PPP --> Secret
#---------------------------------------------------------------------
join_ppp = interfaces.join(ppp).join(secrets).join(fw_interfaces)

class vPPP(BaseModel):
    @classmethod
    def select_all(self):
        q = Session.query(vPPP).order_by(ppp.c.unit).all()
        Session.close()

        res = []
        for entry in q:
            res.append(entry.get_dict())

        return res

    @classmethod
    def select_by_intf(self, intf):
        if type(intf) == int:
            name = "ppp%d" % intf
        elif type(intf) == str:
            name = intf
        else:
            raise TypeError, "PPP UnitNo. or name required."

        q = Session.query(vPPP).filter(ppp.c.name == name)
        Session.close()

        if q.count() == 1:
            return q.one()
        else:
            return None

    @classmethod
    def insert_or_update(self, arg):
        return self._insert_or_update(arg, 'ppp.name', 'name')

    @classmethod
    def delete(self, arg):
        return self._delete(arg, 'ppp.name', 'name')


#---------------------------------------------------------------------
# DialIn --> Secret
#---------------------------------------------------------------------
join_dialin = join(dialin, secrets)

class vDialIn(BaseModel):
    @classmethod
    def insert_or_update(self, arg):
        return self._insert_or_update(arg, 'dialin.unit', 'unit')


#---------------------------------------------------------------------
# OCNIPv6 --> Secret
#---------------------------------------------------------------------
join_ocnipv6 = join(ocnipv6, secrets)

class vOCNIPv6(BaseModel):
    @classmethod
    def insert_or_update(self, arg):
        return self._insert_or_update(arg, 'ocnipv6.unit', 'unit')


#===============================================================================
#
# Mapper configurations
#
#===============================================================================

#--- tables
mapper(Interface, interfaces, properties = {
    'ethernet' : relation(Ethernet, backref = 'interface', cascade = 'all, delete'),
    'gre' : relation(Gre, backref = 'interface', cascade = 'all, delete'),
    'ppp' : relation(PPP, backref = 'interface', cascade = 'all, delete'),
    'dialin' : relation(DialIn, backref = 'interface', cascade = 'all, delete'),
    'route' : relation(Route, backref = 'interface', cascade = 'all, delete'),
    'fw_if' : relation(FW_interface, backref = 'interface', cascade = 'all, delete'),
    'fw_masq_src' : relation(FW_masq, primaryjoin = and_(interfaces.c.name == fw_masqs.c.src_if),
                             backref = 'src_if_ref', cascade = 'all, delete'),
    'fw_masq_dst' : relation(FW_masq, primaryjoin = and_(interfaces.c.name == fw_masqs.c.dst_if),
                             backref = 'dst_if_ref', cascade = 'all, delete')
    })

mapper(Ethernet, ethernet, properties = {
    'ocnipv6' : relation(OCNIPv6, backref = 'ethernet', cascade = 'all')})

mapper(Gre, gre)

mapper(PPP, ppp, properties = {
    'secrets' : relation(Secret, backref = 'ppp', cascade = 'all, delete')})

mapper(DialIn, dialin, properties = {
    'secrets' : relation(Secret, backref = 'dialin', cascade = 'all, delete')})

mapper(OCNIPv6, ocnipv6, properties = {
    'secrets' : relation(Secret, backref = 'ocnipv6', cascade = 'all, delete')})

mapper(Secret, secrets)

mapper(Route, routes)

# shorewall
mapper(FW_interface, fw_interfaces)

mapper(FW_masq, fw_masqs)

mapper(FW_rule, fw_rules)

mapper(FW_nat, fw_nat)

mapper(ConfigDB, configs)

# Monitor
mapper(Mon_host, mon_hosts)

mapper(Mon_mail, mon_mails)

# PING test
mapper(PING_host, ping_hosts)

# ser2net
mapper(Ser2net, ser2net)

#--- views
mapper(vEthernet, join_ethernet, properties = {
    'name' : [interfaces.c.name, ethernet.c.name, fw_interfaces.c.name]})

mapper(vGre, join_gre, properties = {
    'name' : [interfaces.c.name, gre.c.name, fw_interfaces.c.name]})

mapper(vPPP, join_ppp, properties = {
    'name' : [interfaces.c.name, ppp.c.name, fw_interfaces.c.name],
    'unit' : [ppp.c.unit, secrets.c.unit]})

mapper(vDialIn, join_dialin, properties = {
    'unit' : [dialin.c.unit, secrets.c.unit]})

mapper(vOCNIPv6, join_ocnipv6, properties = {
    'unit' : [ocnipv6.c.unit, secrets.c.unit]})


all_tables = {'Net' : {'Interface' : Interface,
                       'Ethernet'  : Ethernet,
                       'Gre'       : Gre,
                       'OCNIPv6'   : OCNIPv6,
                       'PPP'       : {'DialUp'  : PPP,
                                      'DialIn'  : DialIn,
                                      'Secret'  : Secret},
                       'Route'     : Route,
                       'Firewall'  : {'Interface' : FW_interface,
                                      'Masq'      : FW_masq,
                                      'Rule'      : FW_rule,
                                      'Nat'       : FW_nat}
                       },
              'Monitor'   : {'Host'      : Mon_host,
                             'E-Mail'    : Mon_mail},
              'PING_test' : PING_host,
              'Ser2net'   : Ser2net,
              'Configs'   : ConfigDB}


#===============================================================================
#
# support functions
#
#===============================================================================

#---------------------------------------
#
#---------------------------------------
def delete_all(tables=all_tables):
    print "*** Delete from tables..."
    for (name, classname) in tables.iteritems():
        if type(classname) == dict:
            delete_all(classname)
        else:
            classname.delete_all()

#---------------------------------------
#
#---------------------------------------
def drop_tables():
    print "*** Drop all tables..."
    metadata.drop_all()

#---------------------------------------
#
#---------------------------------------
def create_tables():
    print "*** Create all tables..."
    metadata.create_all()

#---------------------------------------
#
#---------------------------------------
def get_entries(arg):
    res = {}
    for (key, classname) in arg.iteritems():
        if type(classname) == dict:
            res[key] = get_entries(classname)
        else:
            res[key] = classname.select_all()

    return res

#---------------------------------------
#
#---------------------------------------
def get_all():
    data = get_entries(all_tables)
    return data

#---------------------------------------
#
#---------------------------------------
def restore_table(key, classname, obj, count):
    if type(classname) == dict:
        for (n_key, n_classname) in classname.iteritems():
            if n_key in obj:
                count = restore_table(n_key, n_classname, obj[n_key], count)
        return count
    else:
        print "restore_table: %s" % key
        classname.load(obj)
        return count + 1

#---------------------------------------
#
#---------------------------------------
def dump_YAML(obj=None, encoding='utf-8', line_break='\n'):
    if obj == None:
        obj = get_all()

    res = yaml.safe_dump(obj, default_flow_style=False,
                         encoding=encoding, allow_unicode=True,
                         indent=2, line_break=line_break)
    return res

#---------------------------------------
#
#---------------------------------------
def load_YAML(yamltxt):
    obj = yaml.safe_load(yamltxt)
    delete_all()

    restored_count = 0
    for (key, classname) in all_tables.iteritems():
        if key in obj:
            print "try to restore_table(%s) ..." % key
            try:
                restored_count += restore_table(key, classname, obj[key], 0)
            except Exception, msg:
                print "restore_table() Error: %s" % str(msg)
                return 0

            print "restored_count: %d" % restored_count

    print "### restored_count: %d" % restored_count
    return restored_count

#---------------------------------------
#
#---------------------------------------
def safe_load_YAML(yamltxt):
    print "** safe_load_YAML()"
    yaml_backup = dump_YAML()

    if load_YAML(yamltxt) == 0:
        # restore
        print "!!! load_YAML() failed, restore backup data..."
        load_YAML(yaml_backup)
        return False

    return True


__all__ = ["Session",
           ### tables
           "Interface", "Ethernet", "Gre", "PPP", "DialIn",
           "OCNIPv6", "Route", "Secret",
           "FW_interface", "FW_masq", "FW_rule", "FW_nat",
           "Mon_host", "Mon_mail", "PING_host", "Ser2net", "ConfigDB",
           ### views
           "vEthernet", "vPPP", "vDialIn", "vOCNIPv6",
           ### data
           "all_tables",
           ### functions
           "delete_all", "drop_tables", "create_tables",
           "get_all",
           "dump_YAML", "load_YAML"]


#===============================================================================
#
# for self test
#
#===============================================================================
def test():
    drop_tables()
    create_tables()

    session = Session()

    #-----------------------------------
    # insert table
    #-----------------------------------

    # Ethernet 'eth0'
    intf = Interface({'name' : 'eth0', 'type' : 0, 'erasable' : False})
    session.save(intf)
    session.commit()

    eth0 = Ethernet({'name' : 'eth0', 'address' : '192.168.253.253',
                     'prefixlen' : 24})
    session.save(eth0)
    session.commit()

    # PPP 'ppp0'
#   intf = Interface({'name' : 'ppp0', 'type' : 1, 'erasable' : True})
#   session.save(intf)
#   session.commit()

#   ppp0 = PPP({'unit' : 0, 'name' : 'ppp0', 'conn_name' : 'Willcom',
#               'dev_name' : 'Willcom(PCMCIA)', 'dev_node' : '/dev/ttyWillcomPCMCIA',
#               'conn_type' : 1, 'pulseok' : False,
#               'modeminit' : 'AT&FS0=0', 'phone_number' : '*99#'})
#   session.save(ppp0)
#   session.commit()

#   secret0 = Secret({'unit' : 0, 'account' : 'HogeUser', 'password' : 'fuga'})
#   session.save(secret0)
#   session.commit()

    # DialIn 'ppp50'
    intf = Interface({'name' : 'ppp50', 'type' : 1, 'erasable' : False})
    session.save(intf)
    session.commit()

    #-----------------------------------
    # insert from view
    #-----------------------------------
#   vEthernet.insert_or_update({'name' : 'eth1', 'address' : '192.168.254.254',
#                               'prefixlen' : 24, 'type' : 0, 'erasable' : False})

#   vGre.insert_or_update({'name' : 'gtn0',
#                           'conn_name' : u'トンネル0',
#                           'remote_ip' : '172.16.0.253',
#                           'local_ip' : '192.168.253.253',
#                           'if_address' : '192.168.3.1',
#                           'ttl' : '255',
#                           'mtu' : '1400',
#                           'type' : 2,
#                           'zone' : 'loc'})

#   vPPP.insert_or_update({'unit' : 1, 'name' : 'ppp1', 'conn_name' : u'ドコモUSB',
#                          'dev_name' : 'NTT DoCoMo(USB)', 'dev_node' : '/dev/ttyNTTDoCoMoUSB',
#                          'conn_type' : 1, 'type' : 1, 'pulseok' : False,
#                          'modeminit' : 'AT&FS0=0', 'phone_number' : '*99#',
#                          'account' : 'docomo', 'password' : 'docomo'})

    mgetty = vDialIn({'dev_name' : 'Modem(RS1)', 'dev_node' : '/dev/ttyS1',
                      'account' : '', 'password' : '',
                      'name' : 'ppp50', 'type' : 1})
    session.save(mgetty)
    session.commit()

#   mgetty = Secret({'unit' : 50, 'account' : 'dialin2', 'password' : 'hoge2'})
#   session.save(mgetty)
#   session.commit()

    #---------------
    # Static Routes
    #---------------
#   Route.insert_or_update({'address' : '172.16.0.0', 'prefixlen' : 24,
#                           'gw_address' : '192.168.254.1', 'gw_dev' : 'eth1'})

    #---------------
    # OCN IPv6
    #---------------
    #ocn_ipv6 = vOCNIPv6({'l2tp_server'  : 'v6tffd1.ocn.ne.jp',
    #                    'interface'    : 'eth1',
    #                    'use_ocnv6'    : True,
    #                    'auto_connect' : False,
    #                    'account'      : 'hoge.ocn.ne.jp',
    #                    'password'     : 'fuga'})
    #session.save(ocn_ipv6)
    #session.commit()

    #---------------
    # firewall
    #---------------

    # interface
    fwzone = FW_interface({'name' : 'eth0', 'zone' : 'loc'})
    session.save(fwzone)
    session.commit()

    #FW_interface.insert_or_update({'name' : 'ppp0', 'zone' : 'net'})
    FW_interface.insert_or_update({'name' : 'ppp50', 'zone' : 'loc'})

    # masq
#   fwmasq = FW_masq({'dst_if' : 'eth1', 'src_if' : 'eth0'})
#   session.save(fwmasq)
#   session.commit()

    # rule
#   FW_rule.insert_or_update({'svc_macro' : 'SSH', 'action' : 'ACCEPT'})
#   FW_rule.insert_or_update({'svc_macro' : 'HTTP', 'src_address' : '172.16.0.0/24',
#                             'action' : 'ACCEPT'})

    # nat
#   FW_nat.insert_or_update({'ext_address' : '130.252.100.18',
#       'interface' : 'eth0', 'int_address' : '10.1.1.2'})
#   FW_nat.insert_or_update({'ext_address' : '130.252.100.19',
#       'interface' : 'eth0', 'int_address' : '10.1.1.3'})

    #---------------
    # PING test
    #---------------
#   ping = PING_host({'address' : '192.168.100.1', 'name' : 'ThinkPad'})
#   session.save(ping)
#   session.commit()

    #---------------
    # ser2net
    #---------------
#   s2n = Ser2net({
#       'port': 8000,
#       'state': 3,
#       'timeout': 600,
#       'dev_node': '/dev/ttyS2',
#       'dev_name': 'RS2',
#       'bps': 115200,
#       'parity': 'NONE',
#       'stopbits': 1,
#       'databits': 8,
#       'flow': 'NONE',
#       'local': False,
#       'remctl': False})
#   session.save(s2n)
#   session.commit()

    #---------------
    # Configurations
    #---------------

    # WebUI
    ConfigDB.set('WebUI', 'account', 'admin')
    ConfigDB.set('WebUI', 'password', 'd033e22ae348aeb5660fc2140aec35850c4da997')

    # DHCPServer (dnsmasq)
    ConfigDB.set('DHCPServer', 'use_dhcp', False)
    ConfigDB.set('DHCPServer', 'use_dns', False)
    ConfigDB.set('DHCPServer', 'interface', 'eth0')
    ConfigDB.set('DHCPServer', 'lease_from', '')
    ConfigDB.set('DHCPServer', 'lease_to', '')

    # DNSServer
    ConfigDB.set('DNSServer', 'primary', '')
    ConfigDB.set('DNSServer', 'secondary', '')

    # Firewall
    ConfigDB.set('Firewall', 'use', True)

    # Monitor
    #  - Email
    ConfigDB.set('Monitor_Mail', 'enable', False)
    ConfigDB.set('Monitor_Mail', 'smtp_server', '')
    ConfigDB.set('Monitor_Mail', 'pop3_server', '')
    ConfigDB.set('Monitor_Mail', 'smtp_port', 25)
    ConfigDB.set('Monitor_Mail', 'pop3_port', 110)
    ConfigDB.set('Monitor_Mail', 'account', '')
    ConfigDB.set('Monitor_Mail', 'password', '')
    ConfigDB.set('Monitor_Mail', 'name', 'MA-E2x0 Default')
    ConfigDB.set('Monitor_Mail', 'mail_addr', '')
    ConfigDB.set('Monitor_Mail', 'auth_type', 'none')

    #  - SNMP Trap
    ConfigDB.set('Monitor_SNMP', 'enable', False)
    ConfigDB.set('Monitor_SNMP', 'manager_addr', '')
    ConfigDB.set('Monitor_SNMP', 'trapcom', '')

    #  - Polling Info
    ConfigDB.set('Monitor_Poll', 'interval', 30)
    ConfigDB.set('Monitor_Poll', 'retry_max', 1)

    #  - ser2net
    ConfigDB.set('ser2net', 'service', False)
    ConfigDB.set('ser2net', 'controller', False)
    ConfigDB.set('ser2net', 'port', 3000)

if __name__ == "__main__":
    test()
