#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   Account module
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
# Copyright (C) 2007-2008 Century Systems Co.,Ltd.
#
# Last Modified: 2010/05/10 15:18:35 kikuchi
#

import copy
import sha
import cherrypy
import syslog

# Configuration DB
from DB import config, models

from Tools import jsonlib
from Tools import wsse_auth

from Handlers import BaseModule

ConfigKey = "WebUI"


#===============================================================================
#
# Account module
#
#===============================================================================
class Account(BaseModule.BaseModule):
    exposed = True

    def __init__(self, debug=False, prefix='output_test/'):
        # SuperClass (BaseModule)
        self.__super = BaseModule.BaseModule

        # モジュール内部データ管理クラス
        self.__account = InternalData(self)

        # Superclass Constructor
        self.__super.__init__(self)

        self.log_header = "Account"

    #-----------------------------------
    # HTTP method handler (GET)
    #-----------------------------------
    def GET(self, *args, **kwargs):
        return self.__account.handle_GET(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (PUT)
    #-----------------------------------
    def PUT(self, *args, **kwargs):
        return self.__account.handle_PUT(*args, **kwargs)

    #---------------------------------------------
    # 設定ファイル書きだし
    #---------------------------------------------
    def write_data(self):
        pass

    #---------------------------------------------
    # Debug: 設定データ(Config) dump
    #---------------------------------------------
    def dump_data(self):
        try:
            encoding = config.encoding
        except:
            encoding = 'shift-jis'

        return models.dump_YAML(encoding=encoding, line_break='\r\n')

    #---------------------------------------------
    # Account --> password
    #---------------------------------------------
    def getPassword(self, account):
        return self.__account.getPassword(account)


#-------------------------------------------------------------------------------
# 内部データ管理
#-------------------------------------------------------------------------------
class InternalData:
    #-----------------------------------
    # Constructor
    #-----------------------------------
    def __init__(self, module):
        self.__module = module

        # アカウント情報
        try:
            account_info = models.ConfigDB.select_by_kind(ConfigKey)
        except KeyError:
            self.__module.log(syslog.LOG_ERR,
                    "No Account Info found, initializing default...")

            account_info = {}
            account_info['account'] = 'admin'
            account_info['password'] = sha.sha('admin').hexdigest()

            for (key, val) in account_info:
                models.ConfigDB.set(ConfigKey, key, val)

    #=======================================================
    # GET (HTTP Method handler)
    #=======================================================
    def handle_GET(self, *args, **kwargs):
        # **kwargs is ignored
        self.__module.log(syslog.LOG_INFO,
            "'GET' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        account_info = models.ConfigDB.select_by_kind(ConfigKey)

        self.__module.log(syslog.LOG_INFO,
            "Account:     '%s'" % account_info['account'])
        self.__module.log(syslog.LOG_INFO,
            "Pass_Digest: '%s'" % account_info['password'])

        # Debug only
        return self.__module.dump_data()

    #=======================================================
    # PUT (HTTP Method handler)
    #=======================================================
    def handle_PUT(self, *args, **kwargs):
        self.__module.log(syslog.LOG_INFO,
            "'PUT' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        try:
            new_account  = kwargs['account']
            new_password = kwargs['password']
        except Exception:
            raise cherrypy.HTTPError(400)

        self.__module.log(syslog.LOG_INFO, "### Change Account, Password")

        account_info = {}

        account_info['account']  = new_account
        account_info['password'] = new_password

        for (key, val) in account_info.iteritems():
            models.ConfigDB.set(ConfigKey, key, val)

        return {'message' : 'Ok'}

    #=======================================================
    # Account -> Password
    #=======================================================
    def getPassword(self, account):
        account_info = models.ConfigDB.select_by_kind(ConfigKey)

        reg_account  = account_info['account']
        reg_password = account_info['password']

        if account == reg_account:
            print "Account OK, password = '%s'" % reg_password
            return reg_password
        else:
            raise KeyError, "Account '%s' isn't valid account" % account


#-------------------------------------------------------------------------------
# Self Test
#-------------------------------------------------------------------------------
if __name__ == '__main__':
    class Empty(BaseModule.BaseModule):
        exposed = True
        def __init__(self):
            BaseModule.BaseModule.__init__(self)

        def write_config(self):
            return {'result'  : 0,
                    'message' : 'Test OK'}

    print "## Account Module test"
    #res = Resource.Resource()
    #conf = Config.Config()

    account = Account(res, conf)

    def get_password(username):
        try:
            password = account.getPassword(username)
            print "wsse callback: password = '%s'" % password
            return password
        except:
            raise cherrypy.HTTPError(401)

    # /conf/datetime
    root = Empty()
    root.admin = Empty()
    root.admin.account = account

    cp_conf = {'global' :
               {'request.show_tracebacks' : False},
               '/' :
               {'request.dispatch' : cherrypy.dispatch.MethodDispatcher(),

                'tools.wsse_auth.on'           : True,
                'tools.wsse_auth.realm'        : 'This Site',
                'tools.wsse_auth.get_password' : get_password,

                'tools.dejsonifier.on' : True,
                'tools.jsonifier.on'   : True,
                }}

    cherrypy.quickstart(root, '/', config = cp_conf)
