#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   Base module
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
#
# Copyright (C) 2007-2008 Century Systems Co.,Ltd.
# Last Modified: 2010/05/10 15:18:42 kikuchi
#

import syslog

#===============================================================================
#
#
#
#===============================================================================
class BaseModule(object):
    def __init__(self):
        self.__dict__['__this_is_module'] = True
        self.__submodules = {}
        self.__parent = None
        self.log_header = ''

    def __setattr__(self, name, value):
        self.__dict__[name] = value
        if hasattr(value, '__this_is_module'):
            self.__submodules[name] = value
            value.__dict__['_BaseModule__parent'] = self

    def log(self, level, msg):
        str = ""
        if self.log_header:
            str = "(%s) " % self.log_header
        str += msg
        syslog.syslog(level, str)
        print str

    def get_confData(self):
        self.confData = self.config.get_item(self.configKey)

        if self.confData == None:
            self.confData = self.config.register_item(self.configKey)

    #-----------------------------------
    #
    #-----------------------------------
    def get_childmodule_names(self):
        modules = []
        for module in self.__submodules.items():
            modules.append(module[0])
        return modules

    #-----------------------------------
    #
    #-----------------------------------
    def get_childmodule(self, name):
        if self.__submodules.has_key(name):
            return self.__submodules[name]
        else:
            return None

    #-----------------------------------
    #
    #-----------------------------------
    def get_parentmodule(self):
        return self.__parent

    #-----------------------------------
    #
    #-----------------------------------
    def get_description(self):
        return "This is BaseModule, please override it."

    #-----------------------------------
    #
    #-----------------------------------
    def get_config(self):
        print "This is BaseModule, please override it."
        return None

    #-----------------------------------
    #
    #-----------------------------------
    def write_data(self):
        msg = "This is BaseModule, please override it."
        return { 'result'  : -1,
                 'command' : 'write_config',
                 'message' : msg }

    #-----------------------------------
    #
    #-----------------------------------
    def export(self):
        self.config.export()


#-------------------------------------------------------------------------------
#
# Self Test
#
#-------------------------------------------------------------------------------
if __name__ == '__main__':
    class NetConfig(BaseModule):
        pass

    class EthernetConfig(BaseModule):
        pass

    class PPPConfig(BaseModule):
        pass

    # Parent module
    net = NetConfig()
    print "dir(net) : %s\n" % dir(net)

    # Sub module
    ethernet = EthernetConfig()
    ppp      = PPPConfig()
    print "dir(ethernet) : %s\n" % dir(ethernet)
    print "dir(ppp)      : %s\n" % dir(ppp)

    # relation
    net.ethernet = ethernet
    net.ppp      = ppp

    net_childmodules = net.get_childmodule_names()
    print "net.get_childmodule_names() --> %s" % str(net_childmodules)

    for childmod_name in net_childmodules:
        print " # childmodule: '%s'" % childmod_name
        childmodule = net.get_childmodule(childmod_name)

        parent = childmodule.get_parentmodule()
        print " --> parent module: '%s'" % str(parent)
