#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#
# Copyright (C) 2008 Century Systems Co.,Ltd.
#

import cherrypy
import commands
from DB import config, models
from Tools import jsonlib
from Tools import wsse_auth

from Utils import PPPstat
from Utils import DaemonTools
from Utils import RCscript
from Handlers import BaseModule
from Applications import Kcmv200Monitor


#===============================================================================
#
# 通信モジュールWebapi
#
#===============================================================================
class ComModule(BaseModule.BaseModule):
    exposed = True

    def __init__(self, debug=False, prefix='output_text/'):
        # SuperClass (BaseModule)
        self.__super = BaseModule.BaseModule

        # モジュール内部データ管理クラス
        self.__commodule = InternalData(self)

        # Debug / Production
        self.__debug = debug

        # Superclass Constructor
        self.__super.__init__(self)

    #-----------------------------------
    # HTTP method handler (GET)
    #-----------------------------------
    def GET(self, *args, **kwargs):
        return self.__commodule.handle_GET(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (POST)
    #-----------------------------------
    def POST(self, *args, **kwargs):
        return self.__commodule.handle_POST(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (PUT)
    #-----------------------------------
    def PUT(self, *args, **kwargs):
        return self.__commodule.handle_PUT(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (DELETE)
    #-----------------------------------
    def DELETE(self, *args, **kwargs):
        return self.__commodule.handle_DELETE(*args, **kwargs)

    #---------------------------------------------
    # 設定ファイル書きだし
    #---------------------------------------------
    def write_data(self):
        pass

    #---------------------------------------------
    # Kill monitor application thread
    #---------------------------------------------
    def kill_monitor(self):
        self.__commodule.kill_monitor()

#-------------------------------------------------------------------------------
# 内部データ管理
#-------------------------------------------------------------------------------
class InternalData:
    #-----------------------------------
    #
    #-----------------------------------
    def __init__(self, module):
        self.__module = module

        # Monitor Application
        self.__monitorApp = Kcmv200Monitor.Kcmv200Monitor()
        self.__monitorApp.start()

    #---------------------------------------------
    # Kill monitor application thread
    #---------------------------------------------
    def kill_monitor(self):
        self.__monitorApp.kill_thread()

    #=======================================================
    # GET
    #=======================================================
    def handle_GET(self, *args, **kwargs):
        # **kwargs is ignored
        print "'GET' : *args = %s, **kwargs = %s" % (str(args), str(kwargs))

        try:
            if len(args) == 1:
                if args[0] == 'info':
                    return self.__monitorApp.read_info()

                elif args[0] == 'stat':
                    return self.__monitorApp.read_status()

                if args[0] == 'infostat':
                    return {'info': self.__monitorApp.read_info(),
                            'stat': self.__monitorApp.read_status()}

        except Exception, msg:
            print "Error, %s" % str(msg)

        raise cherrypy.HTTPError('400', 'ComModule: Bad Request.')

    #=======================================================
    # POST
    #=======================================================
    def handle_POST(self, *args, **kwargs):
        print "'POST' : *args = %s, **kwargs = %s" % (str(args), str(kwargs))

        try:
            if len(args) == 1:
                if args[0] == 'otasp':
                    return self.__monitorApp.otasp()

                elif args[0] == 'otapa':
                    return self.__monitorApp.otapa()

        except Exception, err:
            print "Error, %s" % str(err)

        raise cherrypy.HTTPError('400', 'ComModule: Bad Request.')

    #=======================================================
    # PUT
    #=======================================================
    def handle_PUT(self, *args, **kwargs):
        print "'PUT' : *args = %s, **kwargs = %s" % (str(args), str(kwargs))
        raise cherrypy.HTTPError('400', 'ComModule: Bad Request.')

    #=======================================================
    # DELETE
    #=======================================================
    def handle_DELETE(self, *args, **kwargs):
        print "'DELETE' : *args = %s, **kwargs = %s" % (str(args), str(kwargs))
        raise cherrypy.HTTPError('400', 'ComModule: Bad Request.')

#-------------------------------------------------------------------------------
# Self Test
#-------------------------------------------------------------------------------
if __name__ == '__main__':
    pass
