#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   Config data module
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
#
# Copyright (C) 2007 Century Systems Co.,Ltd.
# Last Modified: 2010/05/10 15:19:01 kikuchi
#

import copy
import syslog
import syck
import pickle

# Exceptions
AlreadyRegistered = "Already registered."
ArgumentError = "Options argument error."

__author__  = "Takeyoshi Kikuchi"
__version__ = "0.01"

#===============================================================================
#
#
#
#===============================================================================
class Config:
    #-------------------------------------------------------
    #
    #-------------------------------------------------------
    def __init__(self, filename, initial = {}):
        #self.__config = copy.copy(initial)
        self.__config = copy.deepcopy(initial)
        self.__filename = filename

    #-------------------------------------------------------
    #
    #-------------------------------------------------------
    def clear(self):
        self.__config = {}

    #-------------------------------------------------------
    #
    #-------------------------------------------------------
    def load(self, obj):
        #self.__config = copy.copy(obj)
        self.__config = copy.deepcopy(obj)

    #-------------------------------------------------------
    #
    #-------------------------------------------------------
    def register_item(self, name):
        # 'root' : reserved
        if name == 'root':
            return None
        if not self.__config.has_key(name):
            self.__config[name] = {}

        return self.__config[name]

    #-------------------------------------------------------
    #
    #-------------------------------------------------------
    def get_item(self, name):
        if name == 'root':
            item = copy.deepcopy(self.__config)
        elif self.__config.has_key(name):
            item = self.__config[name]
        else:
            item = None

        return item

    #-------------------------------------------------------
    #
    #-------------------------------------------------------
    def dump(self, format = 'pickle'):
        format = format.lower()
        result = ''

        if format == 'yaml':
            result = syck.dump(self.__config)
        else:
            result = pickle.dumps(self.__config)

        return result

    #-------------------------------------------------------
    #
    #-------------------------------------------------------
    def export(self, path = ""):
        if path == '':
            path = self.__filename

        fd = open(path, 'w')
        fd.write(syck.dump(self.__config))
        fd.close()


#===============================================================================
# Self Test
#===============================================================================
if __name__ == '__main__':
    print "*** class Config test\n"

    conf = Config()

    conf.register_item('Ethernet')
    conf.register_item('PPP')

    ether = conf.get_item('Ethernet')

    ether['eth0'] = { 'modules' : [ 'dhcpcd', 'ifconfig' ],
                      'config' : { 'dhcp' : True,
                                   'addr' : '192.168.253.253',
                                   'mask' : '255.255.255.0',
                                   }
                      }
    ether['eth1'] = { 'modules' : [ 'ifconfig' ],
                      'config' : { 'dhcp' : False,
                                   'addr' : '192.168.253.253',
                                   'mask' : '255.255.255.0',
                                   }
                      }

    ether2 = conf.get_item('Ethernet')

    print "ether(org)  : %s" % str(ether)
    print "ether(copy) : %s" % str(ether2)

    root = conf.get_item('root')

    print "root obj : %s" % str(root)

