#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   設定データ module
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
# Copyright (C) 2007-2010 Century Systems Co.,Ltd.
#
# Last Modified: 2010/05/26 15:14:03 kikuchi
#

import sha
import commands
import cherrypy
import tempfile
import os
import shutil
import syslog
from datetime import datetime

# Configuration DB
from DB import config, models

from Tools import jsonlib
from Tools import wsse_auth
from Tools import postlib

from Handlers import BaseModule
from Handlers import Account
from Utils import MyTimer

ConfigKey = "yaml"


#===============================================================================
#
# ConfigData module
#
#===============================================================================
class ConfigData(BaseModule.BaseModule):
    exposed = True

    def __init__(self, debug=False, prefix='output_test/'):
        self.__super = BaseModule.BaseModule

        # クライアントモジュール
        # [(key, module),...]
        self.modules = []

        if debug == False:
            self.yamlFile_etc = '/etc/MA_conf.yaml'
            self.yamlFile_tmp = '/tmp/MA_conf_uploaded.yaml'
        else:
            self.yamlFile_etc = '/tmp/MA_conf.yaml'
            self.yamlFile_tmp = '/tmp/MA_conf_uploaded.yaml'

        # Superclass Constructor
        self.__super.__init__(self)

        self.log_header = "ConfigData"

    #-----------------------------------
    # HTTP method handler (GET)
    #-----------------------------------
    def GET(self, *args, **kwargs):
        self.log(syslog.LOG_INFO,
                "'GET': args = %s, kwargs = %s" % (str(args), str(kwargs)))

        if not kwargs.has_key('X-WSSE'):
            self.log(syslog.LOG_ERR, "GET: X-WSSE is not found.")
            cherrypy.HTTPError('401', "ConfigData: Authentication Required.")

        try:
            wsse_auth.wsse_auth_lib("This Site",
                self.__accountModule.getPassword, kwargs['X-WSSE'])
        except Exception, err:
            self.log(syslog.LOG_ERR, "GET: Authentication Fail.")
            raise

        try:
            encoding = config.YAML_encoding
        except:
            encoding = 'shift-jis'

        config_yaml = models.dump_YAML(encoding=encoding, line_break='\r\n')

        header_list = [u"#" * 80,
                       u"# MA-E2xx 設定データ",
                       u"#",
                       u"#  バージョン : '%s'" % config.MA8xx_Version,
                       u"#  出力日     : %s" % datetime.now().isoformat(' '),
                       u"#" * 80,
                       u""]

        header = ('\r\n'.join(header_list)).encode(encoding)
        configfile = '\r\n'.join([header, config_yaml])

        cherrypy.response.headers['Content-Type'] = 'text/plain'
        cherrypy.response.headers['Content-Disposition'] = \
                'attachment; filename="MA_conf.yaml"'

        self.log(syslog.LOG_INFO,
                "output Configuration File (%d bytes)." % len(configfile))

        return configfile

    #-----------------------------------
    # HTTP method handler (POST)
    #-----------------------------------
    def POST(self, *args, **kwargs):
        self.log(syslog.LOG_INFO,
                "'POST': args = %s, kwargs = %s" % (str(args), str(kwargs)))
        lcHDRS = {}

        for key, val in cherrypy.request.headers.iteritems():
            lcHDRS[key.lower()] = val

        try:
            formFields = postlib.storage()(
                        fp=cherrypy.request.rfile,
                        headers=lcHDRS,
                        environ={'REQUEST_METHOD' : 'POST'},
                        keep_blank_values=True)
        except Exception, err:
            self.log(syslog.LOG_ERR,
                "POST: myFieldStorage error: '%s'" % str(err))

        if not formFields.has_key('X-WSSE'):
            self.log(syslog.LOG_ERR, "POST: X-WSSE is not found.")
            cherrypy.HTTPError('401', "ConfigData: Authentication Required.")

        # postBody の X-WSSE で認証
        try:
            wsse_auth.wsse_auth_lib("This Site",
                    self.__accountModule.getPassword,
                    formFields['X-WSSE'].value)
        except Exception, err:
            self.log(syslog.LOG_ERR, "POST: Authentication Fail.")
            raise

        #--- 'POST' でのコマンド処理
        # save, all_clear
        if formFields.has_key('command'):
            command = formFields['command'].value
            self.log(syslog.LOG_INFO, "POST: command='%s'" % command)
            return self.__POST_process_command(command)

        #--- これ以降、ファイルアップロードの処理
        #print formFields
        uploadFile = formFields['configfile']
        self.log(syslog.LOG_INFO,
                "X-WSSE in body : '%s'" % formFields['X-WSSE'].value)
        self.log(syslog.LOG_INFO,
                "Callback Function : '%s'" % formFields['callback'].value)
        callbackFunc = formFields['callback'].value
        #yamlFile = '/tmp/MA_conf_uploaded.yaml'

        try:
            os.unlink(self.yamlFile_tmp)
        except Exception:
            pass

        if hasattr(uploadFile.file, 'name'):
            os.link(uploadFile.file.name, self.yamlFile_tmp)
        else:
            f = open(self.yamlFile_tmp, 'w')
            f.write(uploadFile.file.read())
            f.close()

        result = self.__load_YAML(self.yamlFile_tmp)

        self.log(syslog.LOG_INFO, "load YAML result='%s'" % str(result))

        if result == True:
            msg = "設定を更新しました。"
            shutil.copy(self.yamlFile_tmp, self.yamlFile_etc)
            self.update_config()
        else:
            msg = "設定ファイルが不正です。"

        res = '\n'.join(["<head>",
                         "<meta http-equiv='content-type' content='text/html; charset=utf-8'></meta>",
                         "</head>",
                         "<script type='text/javascript'><!--",
                         "  parent.%s('%s');",
                         "-->",
                         "</script>"]) % (callbackFunc, msg)
        return res

    #-----------------------------------
    # Reload configuration (YAML)
    #-----------------------------------
    def __load_YAML(self, yamlFile):
        try:
            fd = open(yamlFile, 'r')
        except Exception, msg:
            syslog.syslog("__load_YAML: open(%s) failed, '%s'." %
                          (yamlFile, str(msg)))
            return False

        try:
            encoding = config.YAML_encoding
        except:
            encoding = 'shift-jis'

        try:
            buf = fd.read().decode(encoding)
            result = models.safe_load_YAML(buf)
        except Exception, msg:
            syslog.syslog("__load_YAML: failed, '%s'." % str(msg))
            result = False
        finally:
            fd.close()

        return result

    #-----------------------------------
    # POST command
    #-----------------------------------
    def __POST_process_command(self, command):

        if command == "save":
            self.log(syslog.LOG_INFO, "Save Configuration data to mtd.")
            cmdline = "/opt/mae2xx/tools/save_config"
            result = commands.getstatusoutput(cmdline)
            if result[0] == 0:
                msg = u"設定データをFlashメモリへ保存しました。"
            else:
                msg = u"Flashメモリへの保存に失敗しました。"

            return {'msg' : msg}

        elif command == "all_clear":
            self.log(syslog.LOG_INFO, "Clear Configuration data on mtd.")
            fd = open("/tmp/clear_config", 'w+')
            fd.write('clear')
            fd.close()
            ret = {
                'result': True,
                'msg': u"設定データを初期化しました。再起動して下さい。"
            }
            timer = MyTimer.MyTimer(3, self.__reboot)
            timer.start()
            return ret

        elif command == "reload":
            self.log(syslog.LOG_INFO, "Reload Configuration.")
            result = self.__load_YAML("/etc/MA_conf.yaml")
            return {'result' : result}

        else:
            cherrypy.HTTPError('400', "ConfigData: Bad command.")

    def __reboot(self):
        commands.getstatusoutput('shutdown -r now')

    #-----------------------------------
    # HTTP method handler (PUT)
    #-----------------------------------
    def PUT(self, *args, **kwargs):
        cherrypy.HTTPError('400', "ConfigData: Bad Request.")

    def set_Auth(self, accountModule):
        self.__accountModule = accountModule

    #-------------------------------------------------------
    # Register client modules for Configuration
    #-------------------------------------------------------
    def register_module(self, key, module):
        self.modules.append((key, module))
        self.log(syslog.LOG_INFO, "Module '%s' registered." % key)

    #-------------------------------------------------------
    # Configure registered modules
    #-------------------------------------------------------
    def update_config(self):
        for module in self.modules:
            self.log(syslog.LOG_INFO,
                    "call Write Configuration function for module '%s'" % module[0])
            module[1].write_data()


#-------------------------------------------------------------------------------
# Self Test
#-------------------------------------------------------------------------------
if __name__ == '__main__':
    class Empty(BaseModule.BaseModule):
        exposed = True
        def __init__(self):
            BaseModule.BaseModule.__init__(self)

        def write_config(self):
            return { 'result'  : 0,
                     'message' : 'Test OK' }

    print "## Account Module test"
    #res = Resource.Resource()
    #conf = ConfigData.Config()

    account = Account(res, conf)

    def get_password(username):
        try:
            password =account.getPassword(username)
            print "wsse callback: password = '%s'" % password
            return password
        except:
            raise cherrypy.HTTPError(401)

    # /conf/datetime
    root = Empty()
    root.admin = Empty()
    root.admin.account = account

    cp_conf = {'global' :
               {'request.show_tracebacks' : False},
               '/' :
               {'request.dispatch' : cherrypy.dispatch.MethodDispatcher(),
                'tools.wsse_auth.on'           : True,
                'tools.wsse_auth.realm'        : 'This Site',
                'tools.wsse_auth.get_password' : get_password,

                'tools.dejsonifier.on' : True,
                'tools.jsonifier.on'   : True,}}

    cherrypy.quickstart(root, '/', config = cp_conf)
