#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx Web Application Server Module
#   DNSServer module
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
# Copyright (C) 2007-2008 Century Systems Co.,Ltd.
#
# Last Modified: 2010/04/30 15:12:57 kikuchi
#

import cherrypy
import syslog

# Configuration DB
from DB import config, models

from Tools import jsonlib
from Tools import wsse_auth

from Utils import RCscript

from Handlers import BaseModule

ConfigKey = "DNSServer"


#===============================================================================
#
# DNS Server module
#
#===============================================================================
class DNSServer(BaseModule.BaseModule):
    exposed = True

    def __init__(self, debug=False, prefix='output_test/'):
        # SuperClass (BaseModule)
        self.__super = BaseModule.BaseModule

        self.debug = debug

        # モジュール内部データ管理クラス
        self.__dnsserver = InternalData(self)

        # Superclass Constructor
        self.__super.__init__(self)

        self.log_header = "DNSServer"

    #-----------------------------------
    # HTTP method handler (GET)
    #-----------------------------------
    def GET(self, *args, **kwargs):
        return self.__dnsserver.handle_GET(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (POST)
    #-----------------------------------
    def POST(self, *args, **kwargs):
        return self.__dnsserver.handle_POST(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (PUT)
    #-----------------------------------
    def PUT(self, *args, **kwargs):
        return self.__dnsserver.handle_PUT(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (DELETE)
    #-----------------------------------
    def DELETE(self, *args, **kwargs):
        raise cherrypy.HTTPError('400', 'DNSServer: Bad Request.')

    #---------------------------------------------
    # 設定ファイル書きだし
    #---------------------------------------------
    def write_data(self):
        pass


#-------------------------------------------------------------------------------
# 内部データ管理
#-------------------------------------------------------------------------------
class InternalData:
    #-----------------------------------
    #
    #-----------------------------------
    def __init__(self, module):
        self.__module = module
        # 設定が無い場合のデフォルト
        if models.ConfigDB.select_by_kind_key('DNSServer', 'primary') == None:
            models.ConfigDB.set('DNSServer', 'primary', '')

        if models.ConfigDB.select_by_kind_key('DNSServer', 'secondary') == None:
            models.ConfigDB.set('DNSServer', 'secondary', '')

    #=======================================================
    # GET
    #=======================================================
    def handle_GET(self, *args, **kwargs):
        # **kwargs is ignored
        self.__module.log(syslog.LOG_INFO,
                "'GET' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        dns_info = models.ConfigDB.select_by_kind('DNSServer')

        result = {}
        result['DNS_Servers'] = [dns_info.get('primary', ''),
                                 dns_info.get('secondary', '')]

        return result

    #=======================================================
    # PUT
    #=======================================================
    def handle_PUT(self, *args, **kwargs):
        self.__module.log(syslog.LOG_INFO,
                "'PUT' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        try:
            dns_servers = kwargs['dns_servers']
        except Exception:
            cherrypy.HTTPException(400)

        if type(dns_servers) == list and len(dns_servers) <= 2:
            try:
                primary = dns_servers[0]
                models.ConfigDB.set('DNSServer', 'primary', primary)
                secondary = dns_servers[1]
                models.ConfigDB.set('DNSServer', 'secondary', secondary)
            except:
                pass

            return {'result' : 'OK'}
        else:
            cherrypy.HTTPException(400)


#-------------------------------------------------------------------------------
# Self Test
#-------------------------------------------------------------------------------
if __name__ == '__main__':
    class Empty(BaseModule.BaseModule):
        exposed = True
        def __init__(self):
            BaseModule.BaseModule.__init__(self)

        def write_config(self):
            return {'result'  : 0,
                    'message' : 'Test OK'}

    def get_password(username):
        return 'admin'

    print "## DNS Server Module test"
    #res = Resource.Resource()
    #conf = Config.Config()

    dnsserver = DNSServer(res, conf)

    # /conf/datetime
    root = Empty()
    root.conf = Empty()
    root.conf.net = Empty()
    root.conf.net.dnsserver = dnsserver

    cp_conf = {'global' :
               {'request.show_tracebacks' : False},
               '/' :
               {'request.dispatch' : cherrypy.dispatch.MethodDispatcher(),
                'tools.wsse_auth.on'           : True,
                'tools.wsse_auth.realm'        : 'This Site',
                'tools.wsse_auth.get_password' : get_password,

                'tools.dejsonifier.on' : True,
                'tools.jsonifier.on'   : True,}
               }

    cherrypy.quickstart(root, '/', config=cp_conf)
