#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx Web Application Server Module
#   Date/Time module
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
# Copyright (C) 2007-2008 Century Systems Co.,Ltd.
#
# Last Modified: 2010/04/30 15:06:11 kikuchi
#

from datetime import datetime

import cherrypy
import commands
import syslog

# Configuration DB
from DB import config, models

from Tools import jsonlib
from Tools import wsse_auth

from Utils import RCscript

from Handlers import BaseModule

ConfigKey = "DateTime"


#===============================================================================
#
# DateTime module
#
#===============================================================================
class DateTime(BaseModule.BaseModule):
    exposed = True

    def __init__(self, debug=False, prefix='output_test/'):
        # SuperClass (BaseModule)
        self.__super = BaseModule.BaseModule
        self.debug = debug

        # モジュール内部データ管理クラス
        self.__datetime = InternalData(self)

        # Superclass Constructor
        self.__super.__init__(self)

        self.log_header = "DateTime"

    #-----------------------------------
    # HTTP method handler (GET)
    #-----------------------------------
    def GET(self, *args, **kwargs):
        return self.__datetime.handle_GET(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (POST)
    #-----------------------------------
    def POST(self, *args, **kwargs):
        return self.__datetime.handle_POST(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (PUT)
    #-----------------------------------
    def PUT(self, *args, **kwargs):
        return self.__datetime.handle_PUT(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (DELETE)
    #-----------------------------------
    def DELETE(self, *args, **kwargs):
        raise cherrypy.HTTPError('400', 'DateTime: Bad Request.')

    #---------------------------------------------
    # 設定ファイル書きだし
    #---------------------------------------------
    def write_data(self):
        pass


#-------------------------------------------------------------------------------
# 内部データ管理
#-------------------------------------------------------------------------------
class InternalData:
    #-----------------------------------
    #
    #-----------------------------------
    def __init__(self, module):
        self.__module = module

    #=======================================================
    # GET
    #=======================================================
    def handle_GET(self, *args, **kwargs):
        self.__module.log(syslog.LOG_INFO, 
                "'GET' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))
        # **kwargs is ignored
        now = datetime.now()

        date = "%04d/%02d/%02d" % (now.year, now.month, now.day)
        time = "%02d:%02d:%02d" % (now.hour, now.minute, now.second)

        return {'date' : date,
                'time' : time,
                'datetime' : [now.year,
                              now.month,
                              now.day,
                              now.hour,
                              now.minute,
                              now.second]}

    #=======================================================
    # PUT
    #=======================================================
    def handle_PUT(self, *args, **kwargs):
        self.__module.log(syslog.LOG_INFO, 
                "'PUT' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        try:
            time_to_set = [int(x) for x in kwargs['datetime']]
            cmdline = 'date -s "%04d/%02d/%02d %02d:%02d:%02d"' % tuple(time_to_set)
            self.__module.log(syslog.LOG_INFO, cmdline);

            if self.__module.debug == False:
                res = commands.getstatusoutput(cmdline)

                if res[0] != 0:
                    syslog.syslog("DateTime: cmd '%s' failed." % cmdline)

            return {'result': True}

        except Exception:
            raise cherrypy.HTTPError('400', 'DateTime: Bad argument.')


#-------------------------------------------------------------------------------
# Self Test
#-------------------------------------------------------------------------------
if __name__ == '__main__':
    class Empty(BaseModule.BaseModule):
        exposed = True
        def __init__(self):
            BaseModule.BaseModule.__init__(self)

        def write_config(self):
            return {'result'  : 0,
                    'message' : 'Test OK'}

    def get_password(username):
        return 'password1'

    print "## DateTime Module test"
    #res = Resource.Resource()
    #conf = Config.Config()

    DateTime = DateTime(res, conf)

    # /conf/datetime
    root = Empty()
    root.conf = Empty()
    root.conf.datetime = DateTime

    cp_conf = {'global' :
               {'request.show_tracebacks' : False},
               '/' :
               {'request.dispatch' : cherrypy.dispatch.MethodDispatcher(),

                'tools.wsse_auth.on'           : True,
                'tools.wsse_auth.realm'        : 'This Site',
                'tools.wsse_auth.get_password' : get_password,

                'tools.dejsonifier.on' : True,
                'tools.jsonifier.on'   : True,
                }}

    cherrypy.quickstart(root, '/', config=cp_conf)
