#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx Web Application Server Module
#   Firewall (Shorewall) module
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
#
# Copyright (C) 2007-2008 Century Systems Co.,Ltd.
# Last Modified: 2010/05/26 11:37:50 kikuchi
#

import copy
import cherrypy
import syslog
import commands

# Configuration DB
from DB import config, models

from Tools import jsonlib
from Tools import wsse_auth

from Utils import RCscript

from Handlers import BaseModule

# TemplateModule FrontEnd
from Templates.Template_Shorewall import Template_Shorewall

ConfigKey = "Firewall"


#===============================================================================
#
# Firewall (Shorewall) module
#
#===============================================================================
class Firewall(BaseModule.BaseModule):
    exposed = True

    def __init__(self, debug=False, prefix='output_test/'):
        # SuperClass (BaseModule)
        self.__super = BaseModule.BaseModule

        # モジュール内部データ管理クラス
        self.__firewall = InternalData(self)

        # Template FrontEnd
        self.__template = Template_Shorewall(debug, prefix)

        # Debug / Production
        self.debug = debug

        # Superclass Constructor
        self.__super.__init__(self)

        self.log_header = "Firewall"

    #-----------------------------------
    # HTTP method handler (GET)
    #-----------------------------------
    def GET(self, *args, **kwargs):
        return self.__firewall.handle_GET(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (POST)
    #-----------------------------------
    def POST(self, *args, **kwargs):
        if self.debug == False:
            return self.__firewall.handle_POST(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (PUT)
    #-----------------------------------
    def PUT(self, *args, **kwargs):
        return self.__firewall.handle_PUT(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (DELETE)
    #-----------------------------------
    def DELETE(self, *args, **kwargs):
        raise cherrypy.HTTPError('400', 'Firewall: Bad Request.')

    #---------------------------------------------
    # 設定ファイル書きだし
    #---------------------------------------------
    def write_data(self):
        fw_interface = models.FW_interface.select_all()
        fw_masq      = models.FW_masq.select_all()
        fw_rule      = models.FW_rule.select_all()
        fw_nat       = models.FW_nat.select_all()
        use_fw       = models.ConfigDB.select_by_kind_key('Firewall', 'use')

        if use_fw == None:
            use_fw = False

        params = {'fw_interface' : fw_interface,
                  'fw_masq'      : fw_masq,
                  'fw_rule'      : fw_rule,
                  'fw_nat'       : fw_nat,
                  'use_fw'       : use_fw}

        self.log(syslog.LOG_INFO,
                "write_data: params='%s'" % str(params))

        self.__template.write_config(params)

        if self.debug == False:
            # when production only
            self.__firewall.update_daemon(params)


#-------------------------------------------------------------------------------
# 内部データ管理
#-------------------------------------------------------------------------------
class InternalData:
    #-----------------------------------
    #
    #-----------------------------------
    def __init__(self, module):
        self.__module = module

        self.__rc = RCscript.RCscript()
        self.__script = {'firewall' : 'shorewall',
                         'iptables' : 'iptables'}

    #=======================================================
    # GET
    #=======================================================
    def handle_GET(self, *args, **kwargs):
        # **kwargs is ignored
        print "'GET' : *args = %s, **kwargs = %s" % (str(args), str(kwargs))

        if len(args) == 1:
            if args[0] == 'interface':
                return self.__get_fw_interface()
            elif args[0] == 'masq':
                return self.__get_fw_masq()
            elif args[0] == 'rule':
                return self.__get_fw_rule()
            elif args[0] == 'nat':
                return self.__get_fw_nat()
            elif args[0] == 'conf':
                return self.__get_fw_conf()
        elif len(args) == 0:
            return self.__get_fw_conf()

        raise cherrypy.HTTPError('400', 'Firewall: Bad Request.')

    #---
    #
    def __get_fw_interface(self):
        result = models.FW_interface.select_all()

        return {'interface' : result}

    #---
    #
    def __get_fw_masq(self):
        result = models.FW_masq.select_all()

        return {'masq' : result}

    #---
    #
    def __get_fw_rule(self):
        result = models.FW_rule.select_all()

        return {'rule' : result}

    #---
    #
    def __get_fw_conf(self):
        use_fw = models.ConfigDB.select_by_kind_key('Firewall', 'use')

        if use_fw == None:
            use_fw = False

        return {'conf' : {'use_fw' : use_fw}}

    #---
    #
    def __get_fw_nat(self):
        result = models.FW_nat.select_all()

        return {'nat' : result}


    #=======================================================
    # PUT
    #=======================================================
    def handle_PUT(self, *args, **kwargs):
        self.__module.log(syslog.LOG_INFO,
            "'PUT' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        if len(args) == 1:
            if args[0] == 'interface':
                return self.__put_fw_interface(kwargs['interface'])
            elif args[0] == 'masq':
                return self.__put_fw_masq(kwargs['masq'])
            elif args[0] == 'rule':
                return self.__put_fw_rule(kwargs['rule'])
            elif args[0] == 'conf':
                return self.__put_fw_conf(kwargs['conf'])
            elif args[0] == 'nat':
                return self.__put_fw_nat(kwargs['nat'])
            else:
                raise cherrypy.HTTPError('400', 'Firewall: Bad Request.')
        elif len(args) == 0:
            # All
            firewall_info = {}

            try:
                firewall_info['interface'] = kwargs['interface']
                firewall_info['masq']      = kwargs['masq']
                firewall_info['rule']      = kwargs['rule']
                firewall_info['conf']      = kwargs['conf']
                firewall_info['nat']       = kwargs['nat']
            except Exception, msg:
                raise cherrypy.HTTPError('400', "Firewall: Bad Request, '%s'" % str(msg))

            self.__put_fw_interface(firewall_info['interface'])
            self.__put_fw_masq(firewall_info['masq'])
            self.__put_fw_rule(firewall_info['rule'])
            self.__put_fw_conf(firewall_info['conf'])
            self.__put_fw_nat(firewall_info['nat'])

            return {'result' : True}

    #------
    # FW_interface 更新
    def __put_fw_interface(self, interface):
        for entry in interface:
            self.__module.log(syslog.LOG_INFO,
                    "update FW_interface: %s" % str(entry))
            models.FW_interface.insert_or_update(entry)

        return {'result' : True}

    #------
    # FW_masq 更新
    def __put_fw_masq(self, masq):
        models.FW_masq.delete_all()

        for entry in masq:
            self.__module.log(syslog.LOG_INFO, 
                    "insert FW_masq: %s" % str(entry))
            models.FW_masq.insert_or_update(entry)

        return {'result' : True}

    #------
    # FW_rule 更新
    def __put_fw_rule(self, rule):
        models.FW_rule.delete_all()

        for entry in rule:
            try:
                del(entry['rule_id'])
            except:
                pass
            self.__module.log(syslog.LOG_INFO,
                    "insert FW_rule: %s" % str(entry))
            models.FW_rule.insert_or_update(entry)

        return {'result' : True}

    #------
    # 
    def __put_fw_conf(self, conf):
        self.__module.log(syslog.LOG_INFO,
                "__put_fw_conf: conf= '%s'" % str(conf))

        use_fw = conf.get('use_fw', False)

        models.ConfigDB.set('Firewall', 'use', use_fw)
        self.__module.write_data()

        return {'result' : True}

    #------
    # FW_nat 更新
    def __put_fw_nat(self, nat):
        models.FW_nat.delete_all()

        for entry in nat:
            try:
                del(entry['nat_id'])
            except:
                pass
            self.__module.log(syslog.LOG_INFO,
                "insert FW_nat: %s" % str(entry))
            models.FW_nat.insert_or_update(entry)

        return {'result' : True}

    #=======================================================
    # POST
    #=======================================================
    def handle_POST(self, *args, **kwargs):
        self.__module.log(syslog.LOG_INFO,
            "'POST' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        arg_nums = len(args)

        if arg_nums == 0:
            if kwargs.has_key('command'):
                command = kwargs['command']

                # "適用"
                if command == 'update':
                    # /etc/init.d/shorewall 再起動, /etc/init.d/iptables 設定
                    ret = self.update_daemon()
                    return {'result': ret}
                else:
                    raise cherrypy.HTTPError('400', "Shorewall: Bad command '%s'" % command)
            else:
                raise cherrypy.HTTPError('400', "Shorewall: 'command' required.")
        else:
            raise cherrypy.HTTPError('400', "Shorewall: Bad Request URL")

    #-----------------------------------
    # Update daemon
    #-----------------------------------
    def update_daemon(self, params):
        use_fw = models.ConfigDB.select_by_kind_key('Firewall', 'use')

        if use_fw == None:
            use_fw = False
            models.ConfigDB.set('Firewall', 'use', False)

        # 起動時有効 ?
        if use_fw == True:
            # forwarding停止
            self.__module.log(syslog.LOG_INFO, "disable forwarding")
            ip_forward = "/proc/sys/net/ipv4/ip_forward"
            f = open(ip_forward, 'w')
            f.write('0')
            f.close()

            self.__module.log(syslog.LOG_INFO,
                              "rc-update add %s default" % self.__script['iptables'])
            self.__rc.add_default(self.__script['iptables'])

            # shorewall設定更新
            # forwardingはshorewallが再開してくれる。
            cmd = "/sbin/shorewall restart"
            res = commands.getstatusoutput(cmd)
            if res[0] != 0:
                self.__module.log(syslog.LOG_INFO,
                                  "update_daemon(): '%s' failed. '%s'" %
                                  (cmd, str(res)))
                return False

            self.__rc.save(self.__script['iptables'])
            self.__rc.start(self.__script['iptables'])

            # Flush connection tracking table
            cmd = "/usr/sbin/conntrack -F"
            res = commands.getstatusoutput(cmd)
            if res[0] != 0:
                self.__module.log(syslog.LOG_INFO,
                                  "update_daemon(): '%s' failed. '%s'" % \
                                  (cmd, str(res)))

            return True
        else:
            self.__module.log(syslog.LOG_INFO,
                    "rc-update del %s default" % self.__script['iptables'])
            self.__rc.stop(self.__script['iptables'])
            self.__rc.save(self.__script['iptables'])
            self.__rc.del_default(self.__script['iptables'])
            return True


#-------------------------------------------------------------------------------
# Self Test
#-------------------------------------------------------------------------------
if __name__ == '__main__':
    class Empty(BaseModule.BaseModule):
        exposed = True
        def __init__(self):
            BaseModule.BaseModule.__init__(self)

        def write_config(self):
            return {'result'  : 0,
                    'message' : 'Test OK'}

    def get_password(username):
        return 'admin'

    print "## Firewall Module test"
    #res = Resource.Resource()
    #conf = Config.Config()

    firewall = Firewall(res, conf)

    # /conf/datetime
    root = Empty()
    root.conf = Empty()
    root.conf.net = Empty()
    root.conf.net.firewall = firewall

    cp_conf = {'global' :
               {'request.show_tracebacks' : False},
               '/' :
               {'request.dispatch' : cherrypy.dispatch.MethodDispatcher(),

                'tools.wsse_auth.on'           : True,
                'tools.wsse_auth.realm'        : 'This Site',
                'tools.wsse_auth.get_password' : get_password,

                'tools.dejsonifier.on' : True,
                'tools.jsonifier.on'   : True,
                }}

    cherrypy.quickstart(root, '/', config=cp_conf)
