#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   Firmware Update module
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
#
# Copyright (C) 2007-2010 Century Systems Co.,Ltd.
# Last Modified: 2010/05/26 09:20:36 kikuchi
#

import cherrypy
import cgi
import tempfile
import os
import stat
import commands
import Queue
import syslog
import tempfile
import threading

# Configuration DB
from DB import config, models

from Tools import jsonlib
from Tools import wsse_auth
from Tools import postlib

from Utils import RCscript
from Utils import Utils
from Utils import MyTimer

from Handlers import BaseModule
from Handlers import Account

ConfigKey = "Firmup"

FirmwareFileName = 'MAE2xxFirm.img'
FirmupFile       = '/tmp/FIRM.UP'
ClearConfigFile  = '/tmp/clear_config'


#===============================================================================
#
# Firmware update module
#
#===============================================================================
class Firmup(BaseModule.BaseModule):
    exposed = True

    WAITING_UPLOAD = 'waiting_upload'
    UPLOADING = 'uploading'
    WAITING_COMMAND = 'waiting_command'
    CANCEL = 'cancel'
    PENDING_TIMEOUT = 300

    def __init__(self, debug=False, prefix='output_test/'):
        # SuperClass (BaseModule)
        self.__super = BaseModule.BaseModule

        # モジュール内部データ管理クラス
        self.__firmup = InternalData(self)

        # 多重アップロード排他
        self.__mutex = threading.Lock()

        # firmware ファイル情報
        self.__firmInfo = {
                'ticket': 0,
                'status': self.WAITING_UPLOAD,
                'client': '',
                'tmpfp': None,
                'fileName': '',
                'fileSize': '',
                'md5sum': '',
                'mode': ''}

        # 書き込み待ちタイマ
        self.__pendingTimer = None

        # Superclass Constructor
        self.__super.__init__(self)

        self.log_header = "Firmup"

    def __initSession(self, client):
        ret = False

        self.__mutex.acquire()
        try:
            if self.__firmInfo['client'] == "":
                self.__firmInfo['ticket'] += 1
                self.__firmInfo['status'] = self.WAITING_UPLOAD
                self.__firmInfo['client'] = client
                self.__firmInfo['tmpfp'] = tempfile.NamedTemporaryFile()
                ret = True
        except Exception, err:
            pass
        self.__mutex.release()

        return ret

    def __startUpload(self):
        self.__mutex.acquire()
        self.__firmInfo['status'] = self.UPLOADING
        self.__mutex.release()

    def __finishUpload(self, name, size, md5sum):
        self.__mutex.acquire()
        try:
            self.__firmInfo['status'] = self.WAITING_COMMAND
            self.__firmInfo['fileName'] = name
            self.__firmInfo['fileSize'] = size
            self.__firmInfo['md5sum'] = md5sum
            self.__firmInfo['tmpfp'].close()

        except Exception, err:
            self.log(syslog.LOG_ERR, "__finishUpload error(1): '%s'" % str(err))

        try:
            if self.__pendingTimer:
                self.__pendingTimer.cancel()

            self.__pendingTimer = MyTimer.MyTimer(self.PENDING_TIMEOUT,
                                    self.__pendingTimeoutHandler)
            self.__pendingTimer.start()

        except Exception, err:
            self.log(syslog.LOG_ERR, "__finishUpload error(2): '%s'" % str(err))

        self.__mutex.release()

    def __cancelSession(self, cause=None):
        self.__mutex.acquire()
        try:
            # コマンド待ちタイマをキャンセル
            if self.__pendingTimer:
                self.__pendingTimer.cancel()
                self.__pendingTimer = None

            self.__firmInfo['status'] = self.CANCEL
            self.__firmInfo['client'] = ''

            # 一時ファイルを消す
            if self.__firmInfo['tmpfp']:
                try:
                    self.__firmInfo['tmpfp'].close()
                except:
                    pass
                if os.path.exists(self.__firmInfo['tmpfp'].name):
                    os.unlink(self.__firmInfo['tmpfp'].name)
                self.__firmInfo['tmpfp'] = None

            if self.__firmInfo['fileName'] != '':
                if os.path.exists(self.__firmInfo['fileName']):
                    os.unlink(self.__firmInfo['fileName'])

        except Exception, err:
            self.log(syslog.LOG_ERR, "__cancelSession: '%s'" % str(err))

        self.__mutex.release()

    def __pendingTimeoutHandler(self):
        self.log(syslog.LOG_INFO, "__pendingTimeoutHandler: timeout.")
        self.__cancelSession()

    def __update(self, mode):
        self.__mutex.acquire()
        try:
            # コマンド待ちタイマをキャンセル
            if self.__pendingTimer:
                self.__pendingTimer.cancel()
                self.__pendingTimer = None

            srcFile = self.__firmInfo['fileName']

            if srcFile == "":
                self.__cancelSession()
                raise cherrypy.HTTPError('400', 'Firmup: Firmware File not found.')

            dstFile = '/tmp/%s' % FirmwareFileName
            try:
                os.rename(srcFile, dstFile)
            except OSError:
                self.__cancelSession()
                raise cherrypy.HTTPError('400', 'Firmup: Rename Firmware File failed.')

            # Firmware Update キックファイル作成 → /etc/init.d/halt が実行
            kickFile = open(FirmupFile, 'w')
            kickFile.close()

            # /etc overlay preserve or clear
            if mode == 'clear':
                clearConfig = open(ClearConfigFile, 'w')
                clearConfig.close()
            else:
                try:
                    # もしあった場合でも消しておく
                    os.unlink(ClearConfigFile)
                except:
                    pass

            timer = MyTimer.MyTimer(3, self.__reboot)
            timer.start()

        except:
            self.__mutex.release()
            raise

        self.__mutex.release()

    def __reboot(self):
        cmdline = "shutdown -r now"
        commands.getstatusoutput(cmdline)

    #-----------------------------------
    # HTTP method handler (GET)
    #-----------------------------------
    def GET(self, *args, **kwargs):
        self.log(syslog.LOG_DEBUG, "GET called, args = %s" % str(args))

        if len(args) >= 1:
            if args[0] == 'version':
                try:
                    f = open('/etc/version')
                    version_str = f.read()
                    f.close()
                    result = True
                except:
                    version_str = u"'/etc/version' ファイルがありません。"
                    result = False

                return {'result'  : result,
                        'version' : version_str}

            if args[0] == 'get_ticket':
                try:
                    ret = self.__initSession(cherrypy.request.remote.ip)
                    if ret:
                        return {'result': True,
                                'ticket': self.__firmInfo['ticket']}
                    else:
                        return {'result': False,
                                'owner': self.__firmInfo['client']}
                except Exception, err:
                    self.log(syslog.LOG_ERR,
                            "GET get_ticket error: '%s'" % str(err))
                    return {'result': False}

            if args[0] == 'upload_poll':
                try:
                    ticket = int(args[1])
                    if self.__firmInfo['ticket'] != ticket:
                        self.log(syslog.LOG_ERR, "GET upload_poll ticket error.")
                        return {'result': False}

                    if self.__firmInfo['status'] == self.WAITING_UPLOAD:
                        return {'result' : True, 'size' : 0}

                    elif self.__firmInfo['status'] == self.UPLOADING:
                        name = self.__firmInfo['tmpfp'].name
                        size = os.path.getsize(name)
                        return {'result' : True, 'size' : size}

                    elif self.__firmInfo['status'] == self.WAITING_COMMAND:
                        return {'result': True,
                                'msg': {
                                    'fileName': self.__firmInfo['fileName'],
                                    'fileSize': self.__firmInfo['fileSize'],
                                    'md5sum': self.__firmInfo['md5sum']
                                    }
                                }

                    else:
                        return {'result' : False}

                except Exception, err:
                    self.log(syslog.LOG_ERR,
                            "GET upload_poll error: '%s'" % str(err))
                    return {'result' : False}

        else:
            raise cherrypy.HTTPError('400', 'Firmup: Bad URL')

    #-----------------------------------
    # HTTP method handler (POST)
    #-----------------------------------
    def POST(self, *args, **kwargs):
        lcHDRS = {}

        for key, val in cherrypy.request.headers.iteritems():
            lcHDRS[key.lower()] = val

        # POSTする前に、get_ticketで一時ファイルが作成されていること。
        if not self.__firmInfo['tmpfp']:
            self.log(syslog.LOG_ERR, "POST tmpfile is not found.")
            raise cherrypy.HTTPError('400', 'Firmup: tmpfile is not found.')

        # 状態変更
        self.__startUpload()

        self.log(syslog.LOG_INFO,
                "POST tmpfp name = '%s'" % self.__firmInfo['tmpfp'].name)

        # fieldStorageに一時ファイルポインタを渡して受信開始。
        self.log(syslog.LOG_INFO, "POST: myFieldStorage start.")
        try:
            formFields = postlib.storage(self.__firmInfo['tmpfp'])(
                    fp = cherrypy.request.rfile,
                    headers = lcHDRS,
                    environ = {'REQUEST_METHOD' : 'POST'},
                    keep_blank_values = True)
        except Exception, err:
            self.log(syslog.LOG_ERR,
                    "POST: myFieldStorage error: '%s'" % str(err))
            self.__cancelSession()
            raise cherrypy.HTTPError('500', "Firmup: internal error.")

        self.log(syslog.LOG_INFO, "POST: myFieldStorage finish.")

        if not formFields.has_key('X-WSSE'):
            self.log(syslog.LOG_ERR, "POST: X-WSSE is not found.")
            self.__cancelSession()
            raise cherrypy.HTTPError('401', "Firmup: Authentication Required.")

        # postBody の X-WSSE で認証
        try:
            wsse_auth.wsse_auth_lib("This Site",
                    self.__accountModule.getPassword,
                    formFields['X-WSSE'].value)
        except Exception, err:
            self.log(syslog.LOG_ERR, "POST: Authentication Fail.")
            raise

        if "firmfile" in formFields:
            uploadFile = formFields['firmfile']

            if not uploadFile.filename:
                self.log(syslog.LOG_ERR, "POST: filename is null.")
                self.__cancelSession()
                raise cherrypy.HTTPError('400', 'Firmup: filename is null.')

            self.log(syslog.LOG_INFO,
                    "POST: filename = '%s" % str(uploadFile.filename))
            fileName = '/tmp/' + uploadFile.filename

            try:
                os.unlink(fileName)
            except Exception:
                pass

            if hasattr(uploadFile.file, 'name'):
                os.link(uploadFile.file.name, fileName)
            else:
                f = open(fileName, 'w')
                f.write(uploadFile.file.read())
                f.close()

            fileSize = os.stat(fileName)[stat.ST_SIZE]
            md5sum = Utils.md5sum_file(fileName)

            self.log(syslog.LOG_INFO,
                    "POST: size = %d bytes, md5sum = '%s'" % (fileSize, md5sum))

            self.__finishUpload(fileName, fileSize, md5sum)

        if "command" in formFields:
            command = formFields['command'].value
            mode = formFields['mode'].value
            self.log(syslog.LOG_INFO,
                    "POST: command='%s', mode='%s'" % (command, mode))

            # ファームアップモードのチェック
            if mode != 'save' and mode != 'clear' and mode != 'dummy':
                raise cherrypy.HTTPError('400', 'Firmup: bad mode.')

            if command == 'update':
                self.__update(mode)
                return {'result': True,
                        'msg': u'ファームウェアアップデートを行います。完了後、再起動されます。'}
            elif command == 'cancel':
                self.__cancelSession()
                return {'result': True,
                        'msg': u'ファームウェア更新を中断しました。'}

    #-----------------------------------
    # HTTP method handler (PUT)
    #-----------------------------------
    def PUT(self, *args, **kwargs):
        self.log(syslog.LOG_DEBUG, "PUT called, args = %s" % str(args))
        raise cherrypy.HTTPError('400', 'Firmup: Bad Request.')

    #-----------------------------------
    # HTTP method handler (DELETE)
    #-----------------------------------
    def DELETE(self, *args, **kwargs):
        self.log(syslog.LOG_DEBUG, "DELETE called, args = %s" % str(args))
        raise cherrypy.HTTPError('400', 'Firmup: Bad Request.')

    def set_Auth(self, accountModule):
        self.__accountModule = accountModule

#-------------------------------------------------------------------------------
# 内部データ管理
#-------------------------------------------------------------------------------
class InternalData:
    #-----------------------------------
    #
    #-----------------------------------
    def __init__(self, module):
        self.__module = module

    #=======================================================
    # GET
    #=======================================================
    def handle_GET(self, *args, **kwargs):
        # **kwargs is ignored
        print "'GET' : *args = %s, **kwargs = %s" % (str(args), str(kwargs))

    #=======================================================
    # POST
    #=======================================================
    def handle_POST(self, *args, **kwargs):
        print "'POST' : *args = %s, **kwargs = %s" % (str(args), str(kwargs))

    #=======================================================
    # PUT
    #=======================================================
    def handle_PUT(self, *args, **kwargs):
        cherrypy.HTTPError(400)


#-------------------------------------------------------------------------------
# Self Test
#-------------------------------------------------------------------------------
if __name__ == '__main__':
    class Empty(BaseModule.BaseModule):
        exposed = True
        def __init__(self):
            BaseModule.BaseModule.__init__(self)

        def write_config(self):
            return {'result'  : 0,
                    'message' : 'Test OK'}

    def get_password(username):
        return 'admin'

    print "## Firmup Module test"
    firmupmod = Firmup()

    # /conf/datetime
    root = Empty()
    root.admin = Empty()
    root.admin.firmup = firmupmod

    cp_conf = {'global' :
               {'request.show_tracebacks' : False},
               '/' :
               {'request.dispatch' : cherrypy.dispatch.MethodDispatcher(),

                'tools.wsse_auth.on'           : True,
                'tools.wsse_auth.realm'        : 'This Site',
                'tools.wsse_auth.get_password' : get_password,

                'tools.dejsonifier.on' : True,
                'tools.jsonifier.on'   : True,
                }}

    cherrypy.quickstart(root, '/', config=cp_conf)
