#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#  GRE
# Copyright (C) 2009 - 2010 Century Systems Co.,Ltd.
#

import cherrypy
import os
import glob
import re
import syslog
from DB import models
from Tools import jsonlib
from Tools import wsse_auth
from Utils import RCscript
from Handlers import BaseModule

ConfigKey = "Gre"

#===============================================================================
#
# GRE Webapi
#
#===============================================================================
class Gre(BaseModule.BaseModule):
    exposed = True

    def __init__(self, debug=False, prefix='output_test/'):
        # SuperClass (BaseModule)
        self.__super = BaseModule.BaseModule

        # モジュール内部データ管理クラス
        self.__gre = InternalData(self, debug)

        # Debug / Production
        self.__debug = debug

        # Superclass Constructor
        self.__super.__init__(self)

        self.log_header = "Gre"

    #-----------------------------------
    # HTTP method handler (GET)
    #-----------------------------------
    def GET(self, *args, **kwargs):
        return self.__gre.handle_GET(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (POST)
    #-----------------------------------
    def POST(self, *args, **kwargs):
        return self.__gre.handle_POST(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (PUT)
    #-----------------------------------
    def PUT(self, *args, **kwargs):
        return self.__gre.handle_PUT(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (DELETE)
    #-----------------------------------
    def DELETE(self, *args, **kwargs):
        return self.__gre.handle_DELETE(*args, **kwargs)

    #---------------------------------------------
    # 設定ファイル書きだし
    #---------------------------------------------
    def write_data(self, intf='all'):
        self.log(syslog.LOG_INFO, "write_data: intf='%s'" % intf)
        parent = self.get_parentmodule()
        parent.write_data("gre")

        if self.__debug == False:
            if intf == 'all':
                self.__gre.update_interface_all()
            else:
                self.__gre.update_interface(intf)


#-------------------------------------------------------------------------------
# 内部データ管理
#-------------------------------------------------------------------------------
class InternalData:
    #-----------------------------------
    #
    #-----------------------------------
    def __init__(self, module, debug):
        self.__module = module
        self.__debug = debug
        self.__rc = RCscript.RCscript()

    #=======================================================
    # GET
    #=======================================================
    def handle_GET(self, *args, **kwargs):
        # **kwargs is ignored
        self.__module.log(syslog.LOG_INFO,
                "'GET' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        try:
            if len(args) == 0:
                interfaces = models.vGre.select_all()
                return {'gre': interfaces}

            elif len(args) == 1:
                if args[0] == 'all':
                    interfaces = models.vGre.select_all()
                    return {'gre': interfaces}

                else:
                    intf = models.vGre.select_by_intf(arg[0]).get_dict()
                    if intf == None:
                        raise cherrypy.HTTPError(404,
                                "Gre: '%s' isn't found." % arg[0])
                    return intf

        except Exception, msg:
            self.__module.log(syslog.LOG_ERR, "GET Error: %s" % str(msg))

        raise cherrypy.HTTPError('400', 'Gre: Bad Request.')

    #=======================================================
    # POST
    #=======================================================
    def handle_POST(self, *args, **kwargs):
        self.__module.log(syslog.LOG_INFO,
            "'POST' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))
        raise cherrypy.HTTPError('400', 'Gre: Bad Request.')

    #=======================================================
    # PUT
    #=======================================================
    def handle_PUT(self, *args, **kwargs):
        self.__module.log(syslog.LOG_INFO,
            "'PUT' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        if len(args) == 2:
            if args[0] == 'conf':
                intf = args[1]

                try:
                    models.vGre.insert_or_update(kwargs)
                    self.update_configuration(intf)
                    return {'result': True}
                except Exception, err:
                    self.__module.log(syslog.LOG_ERR, "PUT Error: %s" % str(msg))
                    return {'result': False}

        raise cherrypy.HTTPError('400', 'Gre: Bad Request.')

    #=======================================================
    # DELETE
    #=======================================================
    def handle_DELETE(self, *args, **kwargs):
        self.__module.log(syslog.LOG_INFO,
            "'DELETE' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        if len(args) == 2:
            if args[0] == 'conf':
                intf = args[1]

                try:
                    models.vGre.delete({'name': intf})
                    self.update_configuration(intf)
                    return {'result': True}
                except Exception, err:
                    self.__module.log(syslog.LOG_ERR, "DELETE Error: %s" % str(msg))
                    return {'result': False}

        raise cherrypy.HTTPError('400', 'Gre: Bad Request.')

    #=======================================================
    # Update GRE configuration
    #=======================================================
    def update_configuration(self, intf):
        if self.__debug == True:
            return

        self.__module.log(syslog.LOG_INFO,
            "update_configuration(): intf=%s" % intf)
        try:
            conf = models.vGre.select_by_intf(intf).get_dict()
        except:
            # GRE設定無し
            try:
                self.__rc.del_default("net.%s" % intf)
            except:
                pass

        self.__module.write_data(intf)

    #=======================================================
    # Update GRE interface
    #=======================================================
    def update_interface_all(self):
        self.__module.log(syslog.LOG_INFO, "update_interface_all()")

        try:
            names = []
            # DB内のI/F名
            conf_all = models.vGre.select_all()
            for conf in conf_all:
                names.append(conf['name'])

            # /etcに残っているI/F名
            links = glob.glob("/etc/init.d/net.gtn*")
            linkname_re = re.compile("/etc/init.d/net.(.+)")
            for link in links:
                name = linkname_re.match(link).groups()[0]
                if not name in names:
                    names.append(name)

            for name in names:
                self.update_interface(name)

        except Exception, err:
            self.__module.log(syslog.LOG_ERR,
                "update_interface_all() Error: '%s'" % str(err))

    def update_interface(self, intf):
        try:
            conf = models.vGre.select_by_intf(intf).get_dict()
            action = 'modify'
        except AttributeError:
            action = 'remove'

        rcfile = "net.%s" % intf

        if action == 'remove':
            self.__module.log(syslog.LOG_INFO,
                    "update_interface(): intf=%s remove" % intf)
            try:
                self.__rc.stop(rcfile)
                self.__rc.del_default(rcfile)
                os.unlink("/etc/init.d/%s" % rcfile)
            except Exception, err:
                self.__module.log(syslog.LOG_ERR,
                    "update_interface() Error: '%s'" % str(err))
        else:
            self.__module.log(syslog.LOG_INFO,
                    "update_interface(): intf=%s add" % intf)
            cwd = os.getcwd()
            #print "cwd --> %s" % cwd

            try:
                os.chdir("/etc/init.d")
                os.symlink("net.lo", rcfile)
                self.__rc.add_default(rcfile)
                self.__rc.start(rcfile)
                return
            except Exception, err:
                self.__module.log(syslog.LOG_ERR,
                        "update_interface() add Error: '%s'" % str(err))

            try:
                self.__rc.restart(rcfile)
            except Exception, err:
                self.__module.log(syslog.LOG_ERR,
                    "update_interface() restart Error: '%s'" % str(err))

