#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   Monitor (死活監視) module
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
# Copyright (C) 2007-2008 Century Systems Co.,Ltd.
#
# Last Modified: 2010/05/26 09:22:59 kikuchi
#

import copy
import commands
import cherrypy
import syslog

# Configuration DB
from DB import config, models

from Tools import jsonlib
from Tools import wsse_auth

from Utils import RCscript

from Handlers import BaseModule

from Applications import MonitorApp

ConfigKey = "Monitor"


#===============================================================================
#
# Monitor module
#
#===============================================================================
class Monitor(BaseModule.BaseModule):
    exposed = True

    def __init__(self, debug=False, prefix='output_test/'):
        # SuperClass (BaseModule)
        self.__super = BaseModule.BaseModule

        # Monitor Application
        self.__monitorApp = MonitorApp.MonitorApp()
        self.__monitorQueue = self.__monitorApp.get_queue()

        self.__monitorApp.start()

        # モジュール内部データ管理クラス
        self.__monitor = InternalData(self)

        # Superclass Constructor
        self.__super.__init__(self)

        self.log_header = "Monitor"

        self.setup_change()

    #-----------------------------------
    # HTTP method handler (GET)
    #-----------------------------------
    def GET(self, *args, **kwargs):
        return self.__monitor.handle_GET(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (POST)
    #-----------------------------------
    def POST(self, *args, **kwargs):
        return self.__monitor.handle_POST(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (PUT)
    #-----------------------------------
    def PUT(self, *args, **kwargs):
        return self.__monitor.handle_PUT(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (DELETE)
    #-----------------------------------
    def DELETE(self, *args, **kwargs):
        raise cherrypy.HTTPError('400', 'Monitor: Bad Request.')

    #---------------------------------------------
    # 設定ファイル書きだし
    #---------------------------------------------
    def write_data(self):
        pass

    #---------------------------------------------
    # MonitorApplication start
    #---------------------------------------------
    def start_monitoring(self):
        monitorApp = self.__monitorApp

        #email_info   = models.ConfigDB.select_by_kind('Monitor_Mail')
        email_info    = models.ConfigDB.select_by_kind_raw('Monitor_Mail')
        email_target  = models.Mon_mail.select_all()
        host_info     = models.Mon_host.select_all()
        poll_info     = models.ConfigDB.select_by_kind('Monitor_Poll')
        periodic_info = models.ConfigDB.select_by_kind('Monitor_Periodic')

        self.__monitorQueue.put(['setup_email_account', {'email_info' : email_info}])
        self.__monitorQueue.put(['setup_notify_email', {'email' : email_target}])
        self.__monitorQueue.put(['setup_hosts', {'host_info' : host_info}])
        self.__monitorQueue.put(['setup_polling', {'poll_info' : poll_info}])
        self.__monitorQueue.put(['setup_periodic_notifier', {'param': periodic_info}])

        self.__monitorQueue.put(['start_monitoring', ''])

    #---------------------------------------------
    # MonitorApplication stop
    #---------------------------------------------
    def stop_monitoring(self):
        self.__monitorQueue.put(['stop_monitoring', ''])

    #---------------------------------------------
    # Change Setup notify
    #---------------------------------------------
    def setup_change(self):
        self.stop_monitoring()

        # モニタモジュール用データ初期化は完了 ?
        if self.__monitor.get_setup_status() == True:
            # モニタを開始
            self.start_monitoring()

    #---------------------------------------------
    # Kill monitor application thread
    #---------------------------------------------
    def kill_monitor(self):
        self.__monitorApp.kill_threads()
        self.__monitorQueue.put(['exit_thread', ''])

        self.__monitorApp.join()


#-------------------------------------------------------------------------------
# 内部データ管理
#-------------------------------------------------------------------------------
class InternalData:
    #-----------------------------------
    #
    #-----------------------------------
    def __init__(self, module):
        self.__module = module
        self.__rc = RCscript.RCscript()

        result = {'host_info'  : False,
                  'alert_info' : False,
                  'email_info' : False,
                  'poll_info'  : False}

        # Monitor: 監視対象情報
        host_info = models.Mon_host.select_all()
        result['host_info'] = True

        # Monitor: 通知先設定情報
        alert_info = {}
        alert_info['email'] = models.Mon_mail.select_all()
        result['alert_info'] = True

        # Monitor: E-Mail設定
        email_info = models.ConfigDB.select_by_kind('Montor_Mail')
        result['email_info'] = True

        # Monitor: 監視間隔設定
        poll_info = models.ConfigDB.select_by_kind('Monitor_Poll')
        result['poll_info'] = True

        # 初期設定 OK ?
        self.__setup_done = result

    #=======================================================
    # GET
    #=======================================================
    def handle_GET(self, *args, **kwargs):
        # **kwargs is ignored
        self.__module.log(syslog.LOG_INFO,
                "'GET' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        if len(args) == 0:
            return {'resources' : ['hosts', 'alert', 'email', 'poll', 'schedule']}
        elif len(args) != 1:
            raise cherrypy.HTTPError('400', 'Monitor: Bad Request.')

        resource = args[0]

        if resource == 'hosts':
            poll_info = models.ConfigDB.select_by_kind('Monitor_Poll')
            poll_info['interval_opts'] = [30, 60, 90, 120]
            return {'hosts' : models.Mon_host.select_all(),
                    'poll'  : poll_info}

        elif resource == 'alert':
            emails = models.Mon_mail.select_all()
            return {'alert' : {'email' : emails}}

        elif resource == 'periodic':
            periodic_info = {}
            tmp = models.ConfigDB.select_by_kind('Monitor_Periodic')
            periodic_info['enable'] = tmp.get('enable', False)
            periodic_info['interval'] = tmp.get('interval', 1440)
            return {'periodic': periodic_info}

        elif resource == 'email':
            return {'email' : models.ConfigDB.select_by_kind('Monitor_Mail')}

        elif resource == 'poll':
            poll_info = models.ConfigDB.select_by_kind('Monitor_Poll')
            poll_info['interval_opts'] = [30, 60, 90, 120]
            return {'poll' : poll_info}

        else:
            raise cherrypy.HTTPError('400', 'Monitor: Bad Request.')

    #=======================================================
    # POST
    #=======================================================
    def handle_POST(self, *args, **kwargs):
        # **kwargs is ignored
        self.__module.log(syslog.LOG_INFO,
                "'POST' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        if not kwargs.has_key('command'):
            raise cherrypy.HTTPError('400', 'Monitor: Bad Request (command not set).')

        if self.__module.queue == None:
            raise cherrypy.HTTPError('500', 'Monitor: Monitor Application error.')

        command = kwargs['command']

        if command == 'setup':
            self.__setup_cmd()
        elif command == 'start':
            self.__start_cmd()
        elif command == 'stop':
            self.__stop_cmd()
        else:
            raise cherrypy.HTTPError('400', 'Monitor: Bad command.')

        return {'result': True}

    def get_setup_status(self):
        if (self.__setup_done['host_info']  == True and
            self.__setup_done['alert_info'] == True and
            self.__setup_done['email_info'] == True and
            self.__setup_done['poll_info']  == True):
            result = True
        else:
            result = False

        self.__module.log(syslog.LOG_INFO,
                "get_setup_status() = %s" % result)
        return result

    #-----------------------------------
    # Send 'setup' command
    def __setup_cmd(self):
        self.__module.setup_change()

    #-----------------------------------
    # Send 'start' command
    def __start_cmd(self):
        self.__module.start_monitoring()

    #-----------------------------------
    # Send 'stop' command
    def __stop_cmd(self):
        self.__module.stop_monitoring()

    #=======================================================
    # PUT
    #=======================================================
    def handle_PUT(self, *args, **kwargs):
        self.__module.log(syslog.LOG_INFO,
                "'PUT' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))
        errcode = ''

        if len(args) != 1:
            errcode = '400'

        if errcode == '':
            if args[0] == 'hosts':
                try:
                    host_info = kwargs.get('host_info', [])
                    poll_info = kwargs.get('poll_info', {})

                    self.__update_host(host_info, poll_info)
                except:
                    errcode = '400'
            elif args[0] == 'alert':
                try:
                    alert_info = kwargs['alert_info']

                    #print "alert_info = %s, type=%s" % (str(alert_info), type(alert_info))
                    # E-mail 通知先更新
                    alert_email = alert_info.get('email', None)

                    if alert_email != None:
                        self.__update_alert_email(alert_email)
                except Exception, msg:
                    self.__module.log(syslog.LOG_ERR,
                            "PUT: alert Error: '%s'" % str(msg))
                    errcode = 400

            elif args[0] == 'periodic':
                try:
                    periodic_info = kwargs['periodic']
                    for (key, val) in periodic_info.iteritems():
                        models.ConfigDB.set('Monitor_Periodic', key, val)
                except Exception, msg:
                    self.__module.log(syslog.LOG_ERR,
                        "PUT: periodic Error '%s'" % str(msg))
                    errcode = 400

            elif args[0] == 'email':
                try:
                    email_info = kwargs.get('email_info', {})

                    for (key, val) in email_info.iteritems():
                        #print "(key, val) <-- (%s, %s)" % (key, val)
                        models.ConfigDB.set('Monitor_Mail', key, val)
                except Exception, msg:
                    self.__module.log(syslog.LOG_ERR,
                        "PUT: email Error '%s'" % str(msg))
                    errcode = 400

        if errcode != '':
            raise cherrypy.HTTPError(errcode, 'Monitor: Bad Request.')
        else:
            # 設定変更通知
            self.__module.setup_change()
            return {'result': True}

    #----
    #
    def __update_host(self, host_info, poll_info):
        # モニタ対象ホスト全削除
        models.Mon_host.delete_all()

        for host in host_info:
            try:
                new_host = {'address' : host['address'],
                            'name'    : host['name'],
                            'enable'  : host['enable']}
                models.Mon_host.insert_or_update(new_host)
            except Exception, msg:
                self.__module.log(syslog.LOG_ERR,
                    "__update_host() Error: '%s'" % str(msg))

        interval  = poll_info.get('interval', 30)
        retry_max = poll_info.get('retry_max', 3)

        try:
            models.ConfigDB.set('Monitor_Poll', 'interval', interval)
            models.ConfigDB.set('Monitor_Poll', 'retry_max', retry_max)
        except Exception, msg:
            self.__module.log(syslog.LOG_ERR,
                "__update_host() Error: '%s'" % str(msg))

        return

    #----
    #
    def __update_alert_email(self, alert_email):
        models.Mon_mail.delete_all()

        count = 0

        for entry in alert_email:
            enable    = entry.get('enable', False)
            mail_addr = entry.get('mail_addr', '')

            models.Mon_mail.insert_or_update({'mail_addr' : mail_addr,
                                              'enable'    : enable})
            count += 1

        return count

#-------------------------------------------------------------------------------
# Self Test
#-------------------------------------------------------------------------------
if __name__ == '__main__':
    class Empty(BaseModule.BaseModule):
        exposed = True
        def __init__(self):
            BaseModule.BaseModule.__init__(self)

        def write_config(self):
            return {'result'  : 0,
                    'message' : 'Test OK'}

    def get_password(username):
        return 'admin'

    print "## Monitor Module test"
    #res = Resource.Resource()
    #conf = Config.Config()

    monitor = Monitor(res, conf)

    # /conf/datetime
    root = Empty()
    root.conf = Empty()
    root.conf.net = Empty()
    root.conf.net.ocnv6 = ocnv6

    cp_conf = {'global' :
               {'request.show_tracebacks' : False},
               '/' :
               {'request.dispatch' : cherrypy.dispatch.MethodDispatcher(),

                'tools.wsse_auth.on'           : True,
                'tools.wsse_auth.realm'        : 'This Site',
                'tools.wsse_auth.get_password' : get_password,

                'tools.dejsonifier.on' : True,
                'tools.jsonifier.on'   : True,
                }}

    cherrypy.quickstart(root, '/', config=cp_conf)
