#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   PPP(callout) module
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
#
# Copyright (C) 2008 Century Systems Co.,Ltd.
# Last Modified: 2010/05/26 09:25:52 kikuchi
#

import copy
import re
import cherrypy
import syslog
import glob
import os
import time
import commands

# Configuration DB
from DB import config, models

from Tools import jsonlib
from Tools import wsse_auth

from Utils import PPPstat
from Utils import RCscript

from Handlers import BaseModule

# TemplateModule FrontEnd
from Templates.Template_PPP_callout import Template_PPP_callout

ConfigKey = "PPP_callout"


#===============================================================================
#
# PPP(callout) module
#
#===============================================================================
class PPP_callout(BaseModule.BaseModule):
    exposed = True

    def __init__(self, debug=False, prefix='output_test/'):
        self.__super = BaseModule.BaseModule

        # モジュール内部データ管理クラス
        self.__ppp = InternalData(self, debug)

        # Template FrontEnd
        self.__template = Template_PPP_callout(debug, prefix)

        # Debug / Production
        self.__debug = debug

        # Superclass Constructor
        self.__super.__init__(self)

        self.log_header = "PPP_callout"

    #-----------------------------------
    # HTTP method handler (GET)
    #-----------------------------------
    def GET(self, *args, **kwargs):
        return self.__ppp.handle_GET(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (POST)
    #-----------------------------------
    def POST(self, *args, **kwargs):
        return self.__ppp.handle_POST(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (PUT)
    #-----------------------------------
    def PUT(self, *args, **kwargs):
        return self.__ppp.handle_PUT(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (DELETE)
    #-----------------------------------
    def DELETE(self, *args, **kwargs):
        return self.__ppp.handle_DELETE(*args, **kwargs)

    #---------------------------------------------
    # 設定ファイル書きだし
    #---------------------------------------------
    def write_data(self, intf='all'):
        self.log(syslog.LOG_INFO, "write_data: intf='%s'" % intf)

        ppp_callout = models.vPPP.select_all()
        secrets     = models.Secret.select_all()

        self.__template.write_config({'PPP_callout' : ppp_callout,
                                      'Secrets'     : secrets})

        # Daemon 設定変更
        if self.__debug == False:
            # when production only
            self.__ppp.update_daemon(intf)


#-------------------------------------------------------------------------------
# 内部データ管理
#-------------------------------------------------------------------------------
class InternalData:
    #-----------------------------------
    #
    #-----------------------------------
    def __init__(self, module, debug):
        self.__module = module
        self.__debug  = debug
        self.__rc     = RCscript.RCscript()

    #=======================================================
    # GET
    #=======================================================
    def handle_GET(self, *args, **kwargs):
        # **kwargs is ignored
        self.__module.log(syslog.LOG_INFO,
                "'GET' : *args = %s, **kwargs = %s" % \
                (str(args), str(kwargs)))

        if len(args) == 1:
            if args[0] == 'devices':
                # get available devices list
                devices = PPPstat.get_available_devices(dialin=False)

                return {'devices' : devices}
            elif args[0] == 'status' or args[0] == 'conf':
                # get PPP status (ALL)
                status = PPPstat.get_PPP_callout_status()

                return {args[0] : status}
        elif len(args) == 2:
            if args[0] == 'conf':
                pppname = args[1]

                try:
                    info = models.vPPP.select_by_intf(pppname).get_dict()
                except AttributeError:
                    info = "None"

                return {'conf' : info}
        else:
            raise cherrypy.HTTPError('400', 'PPP_callout: Bad Request.')

    #=======================================================
    # POST
    #=======================================================
    def handle_POST(self, *args, **kwargs):
        # **kwargs is ignored
        self.__module.log(syslog.LOG_INFO,
                "'POST' : *args = %s, **kwargs = %s" % \
                (str(args), str(kwargs)))

        if len(args) == 1:
            if args[0] == 'conf':
                # PPP追加
                intf = str(kwargs['name'])
                self.__module.log(syslog.LOG_INFO,
                        "add PPP: name = '%s'" % intf)
                # 既存設定チェック
                if models.vPPP.select_by_intf(intf) != None:
                    self.__module.log(syslog.LOG_ERR,
                            "%s: already exists." % intf)
                    result = {'result' : False,
                              'msg'    : u"PPP '%s' の設定が既に存在します" % intf}
                    return result

                try:
                    result = models.vPPP.insert_or_update(kwargs)
                    self.update_ppp_interface(intf)

                    return {'result' : result[0]}
                except Exception, msg:
                    result = {'result' : False,
                              'msg'    : '%s' % msg}
                    return result

        elif len(args) == 2:
            if args[0] == 'conf':
                intf = args[1]

                command = kwargs.get('command', None)
                if command == None:
                    return {'result' : False,
                            'msg'    : u"コマンド ('command') を指定して下さい"}

                # PPP発信状態チェック
                pppstat = PPPstat.get_PPP_callout_status(intf)

                if pppstat == None:
                    return {'result' : False,
                            'msg'    : u"PPP '%s' は設定されていません"}

                self.__module.log(syslog.LOG_INFO,
                        "pppstat = '%s'" % str(pppstat))

                status = pppstat['status']
                result = False

                if command == 'start':
                    # PPP接続開始
                    if status['value'] == 0:
                        # デバイス無し
                        msg = u"デバイスが存在しません"
                    elif status['value'] == 2:
                        # 未接続 --> PPP start
                        if self.__debug == False:
                            self.__rc.start("net.%s" % intf)

                            # pppd 起動 wait
                            time.sleep(2)

                        msg = u"接続を開始しました"
                        result = True
                    elif status['value'] == 4:
                        # 休止状態 --> Ping to remote
                        if self.__debug == False:
                            result = self.__ping_to_remote(intf, status)

                        if result == True:
                            msg = u"リモートへ PINGしました"
                        else:
                            msg = u"リモートのアドレスが不正です"
                    elif status['value'] == 5:
                        # 別の接続でデバイス使用中
                        msg = u"別のPPP接続で使用中です"
                    else:
                        # 接続中
                        msg = u"既に起動状態です"
                        result = True
                elif command == 'stop':
                    # PPP接続終了
                    if status['value'] == 0:
                        # デバイス無し
                        msg = u"デバイスが存在しません"
                    elif status['value'] == 2:
                        # 未接続
                        msg = u"接続していません"
                        result = True
                    elif status['value'] in [1, 3, 4, 6, 7]:
                        # デバイス無し待機中 / 接続処理中 / 休止状態 / 接続中 / 回線断
                        if self.__debug == False:
                            self.__rc.stop("net.%s" % intf)

                            # pppd 停止 wait
                            time.sleep(2)

                        msg = u"接続を終了しました"
                        result = True
                    else:
                        # 別の接続でデバイス使用中
                        msg = u"別のPPP接続で使用中です"
                else:
                    # コマンドエラー
                    msg = u"コマンド '%s' はサポートされていません" % command

                self.__module.log(syslog.LOG_INFO, "command: %s, result: %s" % \
                        (command, str(result)))

                return {'result' : result,
                        'msg'    : msg}
        else:
            raise cherrypy.HTTPError('400', 'PPP_callout: Bad Request.')

    #=======================================================
    # PUT
    #=======================================================
    def handle_PUT(self, *args, **kwargs):
        self.__module.log(syslog.LOG_INFO, 
                "'PUT' : *args = %s, **kwargs = %s" % \
                (str(args), str(kwargs)))

        if len(args) == 2:
            if args[0] == 'conf':
                # PPP設定更新
                intf = args[1]

                try:
                    models.vPPP.insert_or_update(kwargs)

                    self.update_ppp_interface(intf)

                    return {'result' : True}
                except:
                    return {'result' : False,
                            'msg'    : u"PPP '%s' の設定データに誤りがあります" % intf}

        raise cherrypy.HTTPError('400', 'PPP_callout: Bad Request.')

    #=======================================================
    # DELETE
    #=======================================================
    def handle_DELETE(self, *args, **kwargs):
        self.__module.log(syslog.LOG_INFO, 
                "'DELETE' : *args = %s, **kwargs = %s" % \
                (str(args), str(kwargs)))

        if len(args) == 2:
            if args[0] == 'conf':
                # PPP設定削除
                intf = args[1]

                try:
                    models.vPPP.delete({'name' : intf})

                    self.update_ppp_interface(intf)

                    return {'result' : True}
                except:
                    return {'result' : False,
                            'msg'    : u"PPP '%s' の設定削除に失敗しました" % intf}

        raise cherrypy.HTTPError('400', 'PPP_callout: Bad Request.')

    #-----------------------------------
    # Update PPP configuration
    #-----------------------------------
    def update_ppp_interface(self, intf):
        self.__module.log(syslog.LOG_INFO, "update_ppp_interface(%s)" % intf)
        if self.__debug == False:
            try:
                pppconf = models.vPPP.select_by_intf(intf).get_dict()
            except:
                # PPP設定無し
                try:
                    self.__rc.del_default("net.%s" % intf)

                    ppp_peerfile = "/etc/ppp/peers/%s" % intf
                    os.unlink(ppp_peerfile)
                    ppp_rcfile = "/etc/init.d/net.%s" % intf
                    os.unkink(ppp_rcfile)
                except:
                    pass

        self.__module.write_data(intf)

    #-----------------------------------
    # Update daemon
    #-----------------------------------
    def update_daemon(self, intf):
        self.__module.log(syslog.LOG_INFO, "update_daemon(%s)" % intf)

        if intf != 'all':
            return self.update_daemon_one(intf)

        ppps = models.vPPP.select_all()

        # unlink all init files
        for rcfile in glob.glob("/etc/init.d/net.ppp*"):
            try:
                os.unlink(rcfile)
            except:
                pass

        # unlink all init default
        for rcfile in glob.glob("/etc/runlevels/default/net.ppp*"):
            try:
                os.unlink(rcfile)
            except:
                pass

        for ppp in ppps:
            ppp_rcfile = "net.%s" % ppp['name']
            self.__module.log(syslog.LOG_INFO, "rcfile = %s" % ppp_rcfile)

            cwd = os.getcwd()

            try:
                os.chdir("/etc/init.d")
                os.symlink("net.lo", ppp_rcfile)
            except Exception, msg:
                self.__module.log(syslog.LOG_INFO,
                        "update_daemon() Error: '%s'" % str(msg))
            finally:
                os.chdir(cwd)

            if ppp['conn_type'] != 0:
                # Persist or Demand
                self.__rc.add_default(ppp_rcfile)

    #-----------------------------------
    # Update daemon (one interface)
    #-----------------------------------
    def update_daemon_one(self, intf):
        try:
            ppp = models.vPPP.select_by_intf(intf).get_dict()
            action = 'modify'
        except AttributeError:
            action = 'remove'

        ppp_rcfile = "net.%s" % intf

        self.__module.log(syslog.LOG_INFO, 
                "update_daemon_one: %s %s" % (action, ppp_rcfile))

        if action == 'remove':
            try:
                os.unlink("/etc/init.d/%s" % ppp_rcfile)
                os.unlink("/etc/runlevels/default/%s" % ppp_rcfile)
            except Exception, msg:
                self.__module.log(syslog.LOG_ERR, 
                        "update_daemon_one() error: %s" % str(msg))
        else:
            cwd = os.getcwd()
            try:
                os.chdir("/etc/init.d")
            except Exception, msg:
                self.__module.log(syslog.LOG_ERR,
                        "update_daemon_one() error: %s" % str(msg))

            try:
                os.symlink("net.lo", ppp_rcfile)
            except Exception, msg:
                self.__module.log(syslog.LOG_ERR,
                        "update_daemon_one() error: %s" % str(msg))
            finally:
                os.chdir(cwd)

            if ppp['conn_type'] == 0:
                # Normal
                try:
                    os.unlink("/etc/runlevels/default/%s" % ppp_rcfile)
                except Exception, msg:
                    self.__module.log(syslog.LOG_ERR,
                            "update_daemon_one() error: %s" % str(msg))
            else:
                # Persist or Demand
                try:
                    self.__rc.add_default(ppp_rcfile)
                except Exception, msg:
                    self.__module.log(syslog.LOG_ERR,
                            "update_daemon_one() error: %s" % str(msg))

    #-----------------------------------
    # 
    #-----------------------------------
    def __ping_to_remote(self, intf, stat):
        self.__module.log(syslog.LOG_INFO, 
                "__ping_to_remote: intf = %s, stat = %s" % \
                (intf, str(stat)))

        peer = stat.get('address', {}).get('peer', None)

        if peer != None:
            commands.getstatusoutput("/bin/ping -I %s -w 3 %s" % \
                    (intf, peer))
            return True

        return False

#-------------------------------------------------------------------------------
# Self Test
#-------------------------------------------------------------------------------
if __name__ == '__main__':
    class Empty(BaseModule.BaseModule):
        exposed = True
        def __init__(self):
            BaseModule.BaseModule.__init__(self)

        def write_config(self):
            return {'result'  : 0,
                    'message' : 'Test OK'}

    def get_password(username):
        return 'admin'

    print "## PPP Module test"
    #res = Resource.Resource()
    #conf = Config.Config()

    ppp_dialup = PPP_OCNv6(res, conf)

    # /conf/datetime
    root = Empty()
    root.conf = Empty()
    root.conf.net = Empty()
    root.conf.net.ocnv6 = ocnv6

    cp_conf = {'global' :
               {'request.show_tracebacks' : False},
               '/' :
               {'request.dispatch' : cherrypy.dispatch.MethodDispatcher(),

                'tools.wsse_auth.on'           : True,
                'tools.wsse_auth.realm'        : 'This Site',
                'tools.wsse_auth.get_password' : get_password,

                'tools.dejsonifier.on' : True,
                'tools.jsonifier.on'   : True,
                }}

    cherrypy.quickstart(root, '/', config=cp_conf)
