#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   Ping module
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
#
# Copyright (C) 2007-2008 Century Systems Co.,Ltd.
# Last Modified: 2010/05/26 09:25:27 kikuchi
#

import copy
import cherrypy
import commands
import syslog

# Configuration DB
from DB import config, models

from Tools import jsonlib
from Tools import wsse_auth

from Utils import RCscript

from Handlers import BaseModule

ConfigKey = "PING"


#===============================================================================
#
# Ping module
#
#===============================================================================
class Ping(BaseModule.BaseModule):
    exposed = True

    def __init__(self, debug=False, prefix='output_test/'):
        # SuperClass (BaseModule)
        self.__super = BaseModule.BaseModule

        # モジュール内部データ管理クラス
        self.__ping = InternalData(self)

        # Superclass Constructor
        self.__super.__init__(self)

        self.log_header = "Ping"

    #-----------------------------------
    # HTTP method handler (GET)
    #-----------------------------------
    def GET(self, *args, **kwargs):
        return self.__ping.handle_GET(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (POST)
    #-----------------------------------
    def POST(self, *args, **kwargs):
        return self.__ping.handle_POST(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (PUT)
    #-----------------------------------
    def PUT(self, *args, **kwargs):
        return self.__ping.handle_PUT(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (DELETE)
    #-----------------------------------
    def DELETE(self, *args, **kwargs):
        raise cherrypy.HTTPError('400', 'Ping: Bad Request.')

    #---------------------------------------------
    # 設定ファイル書きだし
    #---------------------------------------------
    def write_data(self):
        pass


#-------------------------------------------------------------------------------
# 内部データ管理
#-------------------------------------------------------------------------------
class InternalData:
    #-----------------------------------
    #
    #-----------------------------------
    def __init__(self, module):
        self.__module = module

        self.__rc = RCscript.RCscript()

    #=======================================================
    # GET (Retrieve)
    #=======================================================
    def handle_GET(self, *args, **kwargs):
        # **kwargs is ignored
        self.__module.log(syslog.LOG_INFO,
                "'GET' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        host_info = models.PING_host.select_all()

        return {'result' : host_info}

    #=======================================================
    # POST (Create, New, Command)
    #=======================================================
    def handle_POST(self, *args, **kwargs):
        # **kwargs is ignored
        self.__module.log(syslog.LOG_INFO,
            "'POST' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        if not kwargs.has_key('addr'):
            raise cherrypy.HTTPError('400', 'Ping: Bad Request (HostAddr not set).')

        addr_list = kwargs['addr'].split(';')

        if len(addr_list) != 1:
            raise cherrypy.HTTPError('400', 'Ping: Bad Request (HostAddr is invalid).')

        addr = addr_list[0]
        cmdline = "/bin/ping -c 3 %s" % addr
        ping_result = commands.getstatusoutput(cmdline)

        if ping_result[0] == 0:
            result = True
        else:
            result = False

        res = {'result'  : result,
               'message' : ping_result[1].split('\n')}

        return res

    #=======================================================
    # PUT
    #=======================================================
    def handle_PUT(self, *args, **kwargs):
        self.__module.log(syslog.LOG_INFO,
            "'PUT' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        count = 0
        try:
            hostlist = kwargs['hostlist']
            models.PING_host.delete_all()

            for host_info in hostlist:
                new_host = {'name'    : host_info.get('name', 'unknown'),
                            'address' : host_info.get('address', '192.168.0.1')}
                models.PING_host.insert_or_update(new_host)
                count += 1
        except Exception, msg:
            self.__module.log(syslog.LOG_ERR,
                    "PUT: Error '%s'" % str(msg))
            raise cherrypy.HTTPError('400', "Ping: '%s'" % str(msg))

        return {'result' : True, 'count' : count}


#-------------------------------------------------------------------------------
# Self Test
#-------------------------------------------------------------------------------
if __name__ == '__main__':
    class Empty(BaseModule.BaseModule):
        exposed = True
        def __init__(self):
            BaseModule.BaseModule.__init__(self)

        def write_config(self):
            return {'result'  : 0,
                    'message' : 'Test OK'}

    def get_password(username):
        return 'admin'

    print "## Ping Module test"
    #res = Resource.Resource()
    #conf = Config.Config()

    ping = Ping(res, conf)

    # /conf/datetime
    root = Empty()
    root.conf = Empty()
    root.conf.net = Empty()
    root.conf.net.ocnv6 = ocnv6

    cp_conf = {'global' :
               {'request.show_tracebacks' : False},
               '/' :
               {'request.dispatch' : cherrypy.dispatch.MethodDispatcher(),

                'tools.wsse_auth.on'           : True,
                'tools.wsse_auth.realm'        : 'This Site',
                'tools.wsse_auth.get_password' : get_password,

                'tools.dejsonifier.on' : True,
                'tools.jsonifier.on'   : True,
                }}

    cherrypy.quickstart(root, '/', config=cp_conf)
