#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   ser2net module
# Copyright (C) 2008 Century Systems Co.,Ltd.
#

import cherrypy
import syslog

from DB import config, models

from Tools import jsonlib
from Tools import wsse_auth

from Utils import DaemonTools

from Handlers import BaseModule

# TemplateModule FrontEnd
from Templates.Template_ser2net import Template_ser2net

ConfigKey = "ser2net"
ServiceName = "ser2net"


#===============================================================================
#
# Ser2net module
#
#===============================================================================
class Ser2net(BaseModule.BaseModule):
    exposed = True

    def __init__(self, debug=False, prefix='output_test/'):
        # SuperClass (BaseModule)
        self.__super = BaseModule.BaseModule

        # モジュール内部データ管理クラス
        self.__ser2net = InternalData(self)

        # Template FrontEnd
        self.__template = Template_ser2net(debug, prefix)

        # Debug / Production
        self.__debug = debug

        # Superclass Constructor
        self.__super.__init__(self)

        self.log_header = "Ser2net"

    #-----------------------------------
    # HTTP method handler (GET)
    #-----------------------------------
    def GET(self, *args, **kwargs):
        return self.__ser2net.handle_GET(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (POST)
    #-----------------------------------
    def POST(self, *args, **kwargs):
        return self.__ser2net.handle_POST(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (PUT)
    #-----------------------------------
    def PUT(self, *args, **kwargs):
        return self.__ser2net.handle_PUT(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (DELETE)
    #-----------------------------------
    def DELETE(self, *args, **kwargs):
        return self.__ser2net.handle_DELETE(*args, **kwargs)

    #---------------------------------------------
    # 設定ファイル書きだし
    #---------------------------------------------
    def write_data(self):
        server = models.ConfigDB.select_by_kind(ConfigKey)
        ports = models.Ser2net.select_all()
        self.__template.write_config({'Ser2net' :
                {'server': server, 'ports': ports}})

        if self.__debug == False:
            self.__ser2net.update_daemon()

#-------------------------------------------------------------------------------
# 内部データ管理
#-------------------------------------------------------------------------------
class InternalData:
    #-----------------------------------
    #
    #-----------------------------------
    def __init__(self, module):
        self.__module = module
        self.__daemon = DaemonTools.DaemonTools()

        if models.ConfigDB.select_by_kind(ConfigKey) == {}:
            # default
            models.ConfigDB.set(ConfigKey, 'service', False)
            models.ConfigDB.set(ConfigKey, 'controller', False)
            models.ConfigDB.set(ConfigKey, 'port', 3000)

    #=======================================================
    # GET
    #=======================================================
    def handle_GET(self, *args, **kwargs):
        # **kwargs is ignored
        self.__module.log(syslog.LOG_INFO,
                "'GET' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        if args[0] == 'devices':
            # XXX
            return {'devices' : []}

        elif args[0] == 'conf':
            server = models.ConfigDB.select_by_kind(ConfigKey)
            ports = models.Ser2net.select_all()
            return {'server': server, 'ports': ports}

        elif args[0] == 'server':
            server = models.ConfigDB.select_by_kind(ConfigKey)
            return {args[0] : server}

        elif args[0] == 'port':
            ports = models.Ser2net.select_all()
            return {args[0] : ports}

        else:
            raise cherrypy.HTTPError('400', 'PPP_callout: Bad Request.')

    #=======================================================
    # POST
    #=======================================================
    def handle_POST(self, *args, **kwargs):
        self.__module.log(syslog.LOG_INFO,
            "'POST' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        try:
            if len(args) == 1:
                if args[0] == 'port':
                    models.Ser2net.insert_or_update(kwargs)
                    self.__module.write_data()
                    return {'result': True}

        except Exception, msg:
            self.__module.log(syslog.LOG_ERR, "Error, %s" % str(msg))

        raise cherrypy.HTTPError('400', 'Ser2net: Bad Request.')

    #=======================================================
    # PUT
    #=======================================================
    def handle_PUT(self, *args, **kwargs):
        self.__module.log(syslog.LOG_INFO,
            "'PUT' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        try:
            if len(args) == 1:
                if args[0] == 'service':
                    keys = ('service', 'controller', 'port')

                    for key in keys:
                        models.ConfigDB.set(ConfigKey, key, kwargs[key])

                    self.__module.write_data()
                    return {'result': True}

                if args[0] == 'port':
                    models.Ser2net.insert_or_update(kwargs)
                    self.__module.write_data()
                    return {'result': True}

        except Exception, msg:
            self.__module.log(syslog.LOG_ERR, "Error, %s" % str(msg))

        raise cherrypy.HTTPError('400', 'Ser2net: Bad Request.')

    #=======================================================
    # DELETE
    #=======================================================
    def handle_DELETE(self, *args, **kwargs):
        self.__module.log(syslog.LOG_INFO,
            "'DELETE' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        if len(args) == 2:
            if args[0] == 'port':
                try:
                    seq = int(args[1])
                    models.Ser2net.delete({'ser2net_seq' : seq})
                    self.__module.write_data()
                    return {'result': True}

                except:
                    return {'result' : False,
                            'msg'    : u"port %d の設定削除に失敗しました" % port}

        raise cherrypy.HTTPError('400', 'Ser2net: Bad Request.')


    #=======================================================
    # Update daemon
    #=======================================================
    def update_daemon(self):
        server = models.ConfigDB.select_by_kind(ConfigKey)
        if server['service'] == True:
            self.__daemon.enable(ServiceName)
        else:
            self.__daemon.disable(ServiceName)

