#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx Web Application Server Module
#   Syslog module
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
# Copyright (C) 2007-2009 Century Systems Co.,Ltd.
#
# Last Modified: 2010/05/26 10:38:19 kikuchi
#

import copy
import glob
import cherrypy
import syslog

# Configuration DB
from DB import config, models

from Tools import jsonlib
from Tools import wsse_auth

from Utils import RCscript

from Handlers import BaseModule

ConfigKey = "Syslog"


#===============================================================================
#
# Syslog module
#
#===============================================================================
class Syslog(BaseModule.BaseModule):
    exposed = True

    def __init__(self, debug=False, prefix='output_test/'):
        # SuperClass (BaseModule)
        self.__super = BaseModule.BaseModule

        # モジュール内部データ管理クラス
        self.__syslog = InternalData(self)

        # Superclass Constructor
        self.__super.__init__(self)

        self.log_header = "Syslog"

    #-----------------------------------
    # HTTP method handler (GET)
    #-----------------------------------
    def GET(self, *args, **kwargs):
        return self.__syslog.handle_GET(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (POST)
    #-----------------------------------
    def POST(self, *args, **kwargs):
        return self.__syslog.handle_POST(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (PUT)
    #-----------------------------------
    def PUT(self, *args, **kwargs):
        return self.__syslog.handle_PUT(*args, **kwargs)

    #-----------------------------------
    # HTTP method handler (DELETE)
    #-----------------------------------
    def DELETE(self, *args, **kwargs):
        raise cherrypy.HTTPError('400', 'Syslog: Bad Request.')

    #---------------------------------------------
    # 設定ファイル書きだし
    #---------------------------------------------
    def write_data(self):
        pass


#-------------------------------------------------------------------------------
# 内部データ管理
#-------------------------------------------------------------------------------
class InternalData:
    #-----------------------------------
    #
    #-----------------------------------
    def __init__(self, module):
        self.__module = module

    #=======================================================
    # GET
    #=======================================================
    def handle_GET(self, *args, **kwargs):
        # **kwargs is ignored
        self.__module.log(syslog.LOG_INFO, 
                "'GET' : *args = %s, **kwargs = %s" % (str(args), str(kwargs)))

        if len(args) != 1:
            raise cherrypy.HTTPError('400', "Syslog: Bad Request.")

        #filename = "/var/log/%s/current" % args[0]
        #files = glob.glob('/var/log/%s/*' % args[0])

        files = glob.glob('/var/log/%s/log-*' % args[0])
        files.append("/var/log/%s/current" % args[0])

        logs = []
        for logfile in files:
            try:
                logs.append(open(logfile).read())
            except:
                pass

        if len(logs) > 0:
            content = '\n'.join(logs)
        else:
            content = "!!! couldn't find logfile at /var/log/%s/" % args[0]

        return {'content' : content}

    #=======================================================
    # PUT
    #=======================================================
    def handle_PUT(self, *args, **kwargs):
        raise cherrypy.HTTPError(400)


#-------------------------------------------------------------------------------
# Self Test
#-------------------------------------------------------------------------------
if __name__ == '__main__':
    class Empty(BaseModule.BaseModule):
        exposed = True
        def __init__(self):
            BaseModule.BaseModule.__init__(self)

        def write_config(self):
            return {'result'  : 0,
                    'message' : 'Test OK'}

    def get_password(username):
        return 'admin'

    print "## Syslog Module test"
    #res = Resource.Resource()
    #conf = Config.Config()

    syslogmod = Syslog(res, conf)

    # /conf/datetime
    root = Empty()
    root.conf = Empty()
    root.conf.net = Empty()
    root.conf.net.syslog = syslogmod

    cp_conf = {'global' :
               {'request.show_tracebacks' : False},
               '/' :
               {'request.dispatch' : cherrypy.dispatch.MethodDispatcher(),

                'tools.wsse_auth.on'           : True,
                'tools.wsse_auth.realm'        : 'This Site',
                'tools.wsse_auth.get_password' : get_password,

                'tools.dejsonifier.on' : True,
                'tools.jsonifier.on'   : True,
                }}

    cherrypy.quickstart(root, '/', config=cp_conf)
