#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-8xx/MA-E2xx Web Application Server
#
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
# Copyright (C) 2007-2012 Century Systems Co.,Ltd.
#
# Last Modified: 2013/07/09 18:22:17 kikuchi
#

import cherrypy
import cgi
import os
import time
import signal
import syslog
import re

# cherrypy tool
from Tools import jsonlib
from Tools import postlib
from Tools import wsse_auth

from Utils import RCscript

from Handlers import BaseModule
from Handlers.Empty import Empty

#-----------------------------
# Import Web modules
#-----------------------------

# /service/conf
# /service/conf/net/
from Handlers import Network
from Handlers import Ethernet
from Handlers import DHCPServer
from Handlers import DNSServer
from Handlers import Firewall
from Handlers import Route
# from Handlers import OCNv6
from Handlers import PPP_callout
from Handlers import PPP_dialin
#from Handlers import ComModule
from Handlers import Gre

# /service/conf/datetime/
from Handlers import DateTime

# /service/daemon/
from Handlers import NTP
#from Handlers import FTP

# /service/apps/
from Handlers import Ser2net
from Handlers import Monitor

# /service/admin/
from Handlers import Account
from Handlers import Syslog
from Handlers import Ping
from Handlers import Reboot

# /service_nowsse/
from Handlers import Firmup
from Handlers import ConfigData

#-----------------------------
# Monitoring Module
#-----------------------------
#from Applications import MonitorApp

from Applications import PPPMonitor

from DB import config


#===============================================================================
#
# MA-E2xx Web Application Server
#
#===============================================================================

# signal handler
def sig_handler(signum, frame):
    syslog.syslog("cherrypy received signal %d, KILL myself." % signum)
    os.kill(os.getpid(), signal.SIGTERM)

#-------------------------------------------------------------------------------
# Foma module class
#-------------------------------------------------------------------------------
class FomaUM01HW(object):
    #---------------------------------------------------------------------------
    #
    #---------------------------------------------------------------------------
    def __init__(self):
        self.__pat = {'SLEEP_OUT'  : re.compile(" +SLEEP_OUT:\t([01])"),
                      }

    #---------------------------------------------------------------------------
    # 戻り値: 0 or 1
    #---------------------------------------------------------------------------
    def __parse_status(self, key):
        try:
            if not key in self.__pat:
                return 1

            pat = self.__pat[key]
            f = open('/proc/driver/foma_status')
            lines = f.readlines()
            stat = -1
            for line in lines:
                match = pat.match(line)
                if match:
                    stat = int(match.groups()[0])
                    break

            f.close()
            if stat == -1:
                msg = "foma_status is out of value"
                syslog.syslog(syslog.LOG_INFO, "[%s] %s: %s" % \
                    (self.__class__.__name__, "__parse_status", str(msg)))
                raise msg

            return stat

        except Exception, msg:
            syslog.syslog(syslog.LOG_INFO, "[%s] %s: %s" % \
                (self.__class__.__name__, "__parse_status", str(msg)))
            return -1

    #---------------------------------------------------------------------------
    # 戻り値: 1=OPEN, 0=GND
    #---------------------------------------------------------------------------
    def sleep_out(self):
        return self.__parse_status('SLEEP_OUT')

    #---------------------------------------------------------------------------
    # 引数
    # value: 1=PWRKEY-OPEN, 0=PWRKEY-GND
    #---------------------------------------------------------------------------
    def pwrkey(self, value):
        try:
            f = open('/proc/driver/foma_pwrkey', 'w+')
            f.seek(0)
            f.write("%d\n" % value)
            f.flush()
            f.close()
            return value

        except Exception, msg:
            syslog.syslog(syslog.LOG_INFO, "[%s] %s: %s" % \
                (self.__class__.__name__, "pwrkey", str(msg)))
            return -1


    #---------------------------------------------------------------------------
    # 引数
    # value: 1=SYSRST-OPEN, 0=SYSRST-GND
    #---------------------------------------------------------------------------
    def sysrst(self, value):
        try:
            f = open('/proc/driver/foma_reset', 'w+')
            f.seek(0)
            f.write("%d\n" % value)
            f.flush()
            f.close()
            return value

        except Exception, msg:
            syslog.syslog(syslog.LOG_INFO, "[%s] %s: %s" % \
                (self.__class__.__name__, "sysrst", str(msg)))
            return -1

    #---------------------------------------------------------------------------
    # 電源ON(PWRKEY-GND 2.3s)
    # 戻り値: 0=success, -1=error
    #---------------------------------------------------------------------------
    def power_on(self):
        count = 0
        value = 0
        ret = None
        res = 0

        res = self.sleep_out()
        if res == 0:
            # 電源OFF状態の場合
            self.pwrkey(0)           # set Low
            time.sleep(0.5)
            self.pwrkey(1)           # set OPEN

            while count <= 8:
                value = self.sleep_out()
                if value == 1:           # SLEEP_OUTがHighなら電源ON確認
                    break
                time.sleep(0.5)
                count += 1

            if value == 1:
                ret = 0
            else:
                ret = -1
        elif res == 1:
            # 電源ON状態の場合
            ret = 0
        else:
            ret = -1

        return ret

    #---------------------------------------------------------------------------
    #
    #---------------------------------------------------------------------------
    def __del__(self):
        pass

# FOMA power on
def FOMA_power_on():
    foma = FomaUM01HW()

    for cnt in range(3):
        # FOMA電源ON
        res = foma.power_on()
        if res == 0:
            break
        else:
            if cnt == 2:
                break
            else:
                msg = "Power on failure."
                syslog.syslog(syslog.LOG_INFO, "%s: %s" % \
                    ("[MA_AppServer.py]FOMA_power_on", str(msg)))

    if res != 0:
        errmsg = "! FOMA module: power-on failed."
        syslog.syslog(syslog.LOG_ERR, errmsg)

#===============================================================================
#
# main
#
#===============================================================================

syslog.openlog("MAAS")

debug = config.debug

if debug == True:
    syslog.syslog("DebugMode")
    INIT_CONFIG = '/tmp/MA_conf.yaml'
else:
    syslog.syslog("ProductionMode")
    INIT_CONFIG = '/etc/MA_conf.yaml'

mod_account = Account.Account(debug)
mod_configdata = ConfigData.ConfigData(debug)

mod_configdata.set_Auth(mod_account)

# MA-E250/FZの場合
foma = os.path.isfile("/proc/driver/foma_status")
if foma == True:
    syslog.syslog(syslog.LOG_INFO, "%s" % "MA-E250/FZ Start")
    # FOMA power on
    #FOMA_power_on()

#---------------------------------------
# WSSE header URLs
#---------------------------------------
# /service
root = Empty()
root.service = Empty()

# /service/conf
root.service.conf = Empty()

# /service/conf/net
root.service.conf.net = Network.Network(debug)

root.service.conf.net.ethernet = Ethernet.Ethernet(debug)
root.service.conf.net.dns      = DNSServer.DNSServer(debug)
root.service.conf.net.dhcp     = DHCPServer.DHCPServer(debug)
root.service.conf.net.firewall = Firewall.Firewall(debug)
#root.service.conf.net.ocnv6    = OCNv6.OCNv6(debug)
root.service.conf.net.route    = Route.Route(debug)
root.service.conf.net.gre      = Gre.Gre(debug)

root.service.conf.net.ppp         = Empty()
root.service.conf.net.ppp.callout = PPP_callout.PPP_callout(debug)
root.service.conf.net.ppp.dialin  = PPP_dialin.PPP_dialin(debug)

#mod_commodule = ComModule.ComModule(debug)
#root.service.conf.net.commodule = mod_commodule

# /service/conf/datetime
root.service.conf.datetime = DateTime.DateTime(debug)

# /service/daemon
root.service.daemon = Empty()

root.service.daemon.ntp = NTP.NTP(debug)
#root.service.daemon.ftp = FTP.FTP(debug)

# /service/apps
root.service.apps = Empty()

mod_ser2net = Ser2net.Ser2net(debug)
root.service.apps.ser2net = mod_ser2net

mod_monitor = Monitor.Monitor(debug)
root.service.apps.monitor = mod_monitor

mod_pppmon = PPPMonitor.PPPMonitor()

# /service/admin
root.service.admin = Empty()

root.service.admin.syslog  = Syslog.Syslog(debug)
root.service.admin.ping    = Ping.Ping(debug)
root.service.admin.reboot  = Reboot.Reboot(debug)
root.service.admin.account = mod_account

#---------------------------------------
# No WSSE header URLs
#---------------------------------------
# /service_nowsse
root.service_nowsse = Empty()

# /service_nowsse/firmup
root.service_nowsse.firmup  = Firmup.Firmup()
root.service_nowsse.firmup.set_Auth(mod_account)

# /service_nowsse/config
root.service_nowsse.config = mod_configdata

#---------------------------------------
# Monitoring Application
#---------------------------------------
#monitorApp = MonitorApp.MonitorApp()
#mod_monitor.set_receiver(monitorApp)

# Register modules for configuration file update
mod_configdata.register_module('Net', root.service.conf.net)
mod_configdata.register_module('gre', root.service.conf.net.gre)
mod_configdata.register_module('dnsmasq', root.service.conf.net.dhcp)
mod_configdata.register_module('shorewall', root.service.conf.net.firewall)
#mod_configdata.register_module('OCNIPv6', root.service.conf.net.ocnv6)
mod_configdata.register_module('ntpd', root.service.daemon.ntp)
mod_configdata.register_module('ppp_callout', root.service.conf.net.ppp.callout)
mod_configdata.register_module('ppp_dialin', root.service.conf.net.ppp.dialin)
mod_configdata.register_module('Reboot', root.service.admin.reboot)
mod_configdata.register_module('ser2net', root.service.apps.ser2net)

# PPP link monitoring thread
mod_pppmon.start()

cp_conf = {'global' :
           {'engine.autoreload_on'        : False,
            #'engine.autoreload_frequency' : 1,
            #'request.show_tracebacks'     : False,
            'request.show_tracebacks'     : True,
            #'server.socket_host'          : '127.0.0.1',
            'server.socket_port'          : 8080,
            'response.timeout'            : 86400},
           '/service_nowsse' :
           {'request.dispatch' : cherrypy.dispatch.MethodDispatcher(),

             'tools.dejsonifier.on'   : True,
             'tools.jsonifier.on'     : True,
             'tools.noBodyProcess.on' : True,},
           '/service' :
           {'request.dispatch' : cherrypy.dispatch.MethodDispatcher(),

            'tools.wsse_auth.on'           : True,
            'tools.wsse_auth.realm'        : 'This Site',
            'tools.wsse_auth.get_password' : mod_account.getPassword,

            'tools.dejsonifier.on'  : True,
            'tools.jsonifier.on'    : True,

            'tools.gzip.on'         : True,
            'tools.gzip.mime_types' : ['text/html', 'application/json']
            },}

# Limit POST size to 33MB
cherrypy.server.max_request_body_size = 33 * 1024 * 1024

# Listen any interface
cherrypy.server.socket_host = '0.0.0.0'

try:
    cherrypy.quickstart(root, '/', config=cp_conf)
except TypeError, msg:
    syslog.syslog("cherrypy received SIGTERM, orz... KILL myself.")
    os.kill(os.getpid(), signal.SIGKILL)

mod_monitor.kill_monitor()
#mod_commodule.kill_monitor()
os.kill(os.getpid(), signal.SIGKILL)
