#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#
# Copyright (C) 2007 Century Systems Co.,Ltd.
#
# Last Modified: 2010/05/10 15:18:19 kikuchi
#

import random
import datetime, time
import base64, sha
import httplib
import urllib

class RESTClient:
    def __init__(self):
        self.__endpoint = None
        self.__wsse = None

    def set_account(self, user, password):
        self.__user = user
        self.__password = password

    def set_endpoint(self, endpoint):
        self.__endpoint = endpoint

    def __make_credentials(self):
        password_digest = sha.sha(self.__password).hexdigest()
        nonce = sha.sha(str(time.time() + random.random())).digest()
        now = datetime.datetime.now().isoformat() + "Z"
        digest = sha.sha(nonce + now + password_digest).digest()

        wsse = ('UsernameToken Username="%(u)s", PasswordDigest="%(p)s", '
                'Nonce="%(n)s", Created="%(c)s"')
        value = dict(u=self.__user, p=base64.encodestring(digest).strip(),
                     n=base64.encodestring(nonce).strip(), c=now)

        self.__wsse = wsse % value

    def request(self, method, URI, body, content_type='text/json'):
        conn = httplib.HTTPConnection(self.__endpoint)

        self.__make_credentials()

        header = {'X-WSSE' : self.__wsse, 'Content-Type' : content_type}

        conn.request(method, URI, body, header)
        r = conn.getresponse()

        response = dict(status=r.status, reason=r.reason, data=r.read())

        conn.close()
        self.__wsse = None

        return response

    def nowsse_request(self, method, URI, args, content_type='text/json'):
        if type(args) != dict:
            raise Exception, "dict arguments required."

        conn = httplib.HTTPConnection(self.__endpoint)
        self.__make_credentials()

        # Quote arguments
        args_encoded = ["X-WSSE=%s" % urllib.quote_plus(self.__wsse)]
        for arg in args.items():
            args_encoded.append("%s=%s" % (arg[0], urllib.quote_plus(arg[1])))

        body = '&'.join(args_encoded)

        #print body
        #header = { 'X-WSSE' : self.__wsse, 'Content-Type' : content_type }
        conn.request(method, URI, body)
        r = conn.getresponse()

        response = dict(status=r.status, reason=r.reason, data=r.read())

        conn.close()
        self.__wsse = None

        return response
