#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   /etc/shorewall/interfaces output module
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
#
# Copyright (C) 2008 Century Systems Co.,Ltd.
# Last Modified: 2010/05/10 15:14:10 kikuchi
#

from datetime import datetime
import re
import syslog

class interfaces(object):
    def __init__(self, searchList, append=None):
        params = searchList.get('params', {})
        fw_interfaces = params.get('fw_interface', [])

        response = [
            u"#",
            u"# Shorewall version 4.0 - Sample Interfaces File for two-interface configuration.",
            u"# Copyright (C) 2006 by the Shorewall Team",
            u"#",
            u"# This library is free software; you can redistribute it and/or",
            u"# modify it under the terms of the GNU Lesser General Public",
            u"# License as published by the Free Software Foundation; either",
            u"# version 2.1 of the License, or (at your option) any later version.",
            u"#",
            u"# See the file README.txt for further details.",
            u"#------------------------------------------------------------------------------",
            u"# For information about entries in this file, type \"man shorewall-interfaces\"",
            u"#",
            u"# For additional information, see",
            u"# http://shorewall.net/Documentation.htm#Interfaces",
            u"#",
            u"###############################################################################",
            u"#ZONE INTERFACE   BROADCAST   OPTIONS"]

        for intf in fw_interfaces:
            intf_name = intf['name'].lower()

            if intf_name.startswith('eth') == True:
                # Ethernet
                if intf['zone'] == 'net':
                    options = u"dhcp,tcpflags,routefilter,nosmurfs,logmartians"
                else:
                    #options = u"dhcp,tcpflags,nosmurfs"
                    # Shorewall FAQ No.2 "loc2loc static route"
                    options = u"dhcp,tcpflags,nosmurfs,routeback"
            elif intf_name.startswith('ppp') == True:
                # PPP
                options = u"tcpflags,nosmurfs"
            elif intf_name.startswith('gtn') == True:
                # GRE
                options = u"tcpflags,nosmurfs"
            else:
                # ??? todo
                syslog.syslog("Shorewall.interfaces: '%s' not supported." % intf['name'])
                continue

            response.append(u"%-8s%-12s%-12s%s" % (
                intf['zone'], intf_name, 'detect', options))

        response.append(u"#LAST LINE -- ADD YOUR ENTRIES BEFORE THIS ONE -- DO NOT REMOVE")

        self.__response = response

    def respond(self):
        return u"%s\n" % '\n'.join(self.__response)
