#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   /etc/shorewall/rules output module
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
#
# Copyright (C) 2008 Century Systems Co.,Ltd.
# Last Modified: 2010/05/10 15:14:34 kikuchi
#

import syslog

syslog_header = "Shorewall.rules"

class rules(object):
    def __init__(self, searchList, append=None):
        params = searchList.get('params', {})
        fw_rules = params.get('fw_rule', [])

        response = [
            u"#",
            u"# Shorewall version 4.0 - Sample Rules File for two-interface configuration.",
            u"# Copyright (C) 2006,2007 by the Shorewall Team",
            u"#",
            u"# This library is free software; you can redistribute it and/or",
            u"# modify it under the terms of the GNU Lesser General Public",
            u"# License as published by the Free Software Foundation; either",
            u"# version 2.1 of the License, or (at your option) any later version.",
            u"#",
            u"# See the file README.txt for further details.",
            u"#------------------------------------------------------------------------------",
            u"# For information about entries in this file, type \"man shorewall-rules\"",
            u"#",
            u"# For more information, see http://www.shorewall.net/Documentation.htm#Rules",
            u"#",
            u"#" * 124,
            u"#ACTION      SOURCE               DEST                 PROTO   DEST    SOURCE   ORIGINAL         RATE        USER/    MARK",
            u"#                                                              PORT    PORT(S)  DEST             LIMIT       GROUP"]

        for fw_rule in fw_rules:
            svc_macro     = fw_rule.get('svc_macro', '')
            svc_protocol  = fw_rule.get('svc_protocol', '')
            svc_port      = fw_rule.get('svc_port', 0)
            action        = fw_rule.get('action', '')
            src_address   = fw_rule.get('src_address', '')
            src_port      = fw_rule.get('src_port', 0)
            dst_address   = fw_rule.get('dst_address', '')
            dst_port      = fw_rule.get('dst_port', 0)
            original_dest = fw_rule.get('original_dest', '')

            rule_params = {}

            #print "@@ fw_rule: %s" % str(fw_rule)

            # ACTION
            if action != '':
                if svc_macro != '' and svc_port == 0:
                    rule_params['ACTION']   = u"%s/%s" % (svc_macro, action)
                    rule_params['PROTOCOL'] = ''
                    rule_params['PORT']     = 0
                elif svc_macro == '' and svc_port != 0 and svc_protocol != '':
                    rule_params['ACTION']   = u"%s" % action
                    rule_params['PROTOCOL'] = svc_protocol.lower()
                    rule_params['PORT']     = svc_port
                else:
                    syslog.syslog("%s: 'ACTION' error, (macro, protocol, port)" \
                                  "= ('%s', '%s', %d)" % \
                                  (syslog_header, svc_macro, svc_protocol, svc_port))
                    continue
            else:
                syslog.syslog("%s: 'ACTION' error, action == ''" % syslog_header)
                continue
            
            # SOURCE
            tmp = [u'net',]
            if src_address != '':
                tmp.append(src_address)

                if src_port != 0:
                    tmp.append("%d" % src_port)

            rule_params['SOURCE'] = ':'.join(tmp)

            # DEST
            if dst_address != '':
                tmp = [u'loc',]
                tmp.append(dst_address)

                if dst_port != 0:
                    tmp.append("%d" % dst_port)
            else:
                tmp = [u'$FW',]

            rule_params['DEST'] = ':'.join(tmp)

            if svc_macro != '':
                # Macro
                if original_dest == '':
                    rule_line = u"%-12s %-20s %-20s" % \
                                                (rule_params['ACTION'],
                                                    rule_params['SOURCE'],
                                                    rule_params['DEST'])
                else:
                    rule_line = u"%-12s %-20s %-20s %-7s %-7s %-8s %-8s" % \
                                                (rule_params['ACTION'],
                                                  rule_params['SOURCE'],
                                                  rule_params['DEST'],
                                                  '-',
                                                  '-',
                                                  '-',
                                                  original_dest)
            else:
                if original_dest == '':
                    rule_line = u"%-12s %-20s %-20s %-7s %-7s" % \
                                                (rule_params['ACTION'],
                                                  rule_params['SOURCE'],
                                                  rule_params['DEST'],
                                                  rule_params['PROTOCOL'],
                                                  rule_params['PORT'])
                else:
                    rule_line = u"%-12s %-20s %-20s %-7s %-7s %-8s %-8s" % \
                                                (rule_params['ACTION'],
                                                  rule_params['SOURCE'],
                                                  rule_params['DEST'],
                                                  rule_params['PROTOCOL'],
                                                  rule_params['PORT'],
                                                  '-',
                                                  original_dest)

            #print "** rule: %s" % rule_line
            response.append(rule_line)

        response.append(u"#LAST LINE - ADD YOUR ENTRIES ABOVE THIS ONE - DO NOT REMOVE")
        self.__response = response

    def respond(self):
        return u"%s\n" % '\n'.join(self.__response)
