#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   /etc/ppp/peers/mgetty output module
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
#
# Copyright (C) 2008 Century Systems Co.,Ltd.
# Last Modified: 2010/05/10 15:15:08 kikuchi
#

from datetime import datetime
import pyiproute

class peers_mgetty(object):
    def __init__(self, searchList, append=None):
        ipaddr = pyiproute.ipaddr()

        params = searchList.get('params', {})

        mgetty_conf = params.get('PPP_dialin', {})

        response = [
            u"#" * 80,
            u"## /etc/ppp/peers/ppp50 (for mgetty dialin)",
            u"## Generated by MA-E2xx WebUI at %s" % datetime.now().isoformat(' '),
            u"#" * 80,
            u""]

        response.append(u"%s" % mgetty_conf['dev_node'])

        response.extend([u"#login",
                         u"debug",
                         u"kdebug 7",
                         u"#ipv6",
                         u"crtscts",
                         u"modem",
                         u"lock"])

        # auth_method
        auth = False
        auth_method = mgetty_conf.get('auth_method', True)
        if auth_method:
            auth = True
            response.append(u"auth")
        else:
            response.append(u"noauth")

        # user
        useaccount = mgetty_conf.get('useaccount', False)
        if useaccount:
            auth = True
            response.append(u"user %s" % mgetty_conf.get('user', 'nanashi-san'))

        # chap/pap
        if auth == False:
            response.extend([u"refuse-chap",
                    u"refuse-pap"])

        # ipconfig
        ip_local  = mgetty_conf.get('ip_local', '0.0.0.0')
        ip_remote = mgetty_conf.get('ip_remote', '0.0.0.0')
        if ip_local == '':
            ip_local = '0.0.0.0'
        if ip_remote == '':
            ip_remote = '0.0.0.0'

        if ip_local != '0.0.0.0' and ip_remote != '0.0.0.0':
            response.append(u"%s:%s" % (ip_local, ip_remote))
        elif ip_local != '0.0.0.0':
            response.append(u"%s:" % ip_local)
        elif ip_remote != '0.0.0.0':
            response.append(u":%s" % ip_remote)
        else:
            #ip_local = ipaddr.get_IPv4address('eth0')[0]['local']
            #response.append(u"%s:" % ip_local)
            response.append(u"noipdefault")

        # netmask
        ip_netmask = mgetty_conf.get('ip_netmask', '0.0.0.0')
        if ip_netmask != '0.0.0.0' and ip_netmask != '':
            response.append(u"netmask %s" % ip_netmask)

        # idle
        idle = mgetty_conf.get('idle_time', 0)
        if idle:
            response.append(u"idle %d" % idle)

        # dns
        ipcp_dns = mgetty_conf.get('ipcp_dns', 0)
        if ipcp_dns == 0:
            response.append(u"# ms-dns")

        elif ipcp_dns == 1:
            primary_dns = mgetty_conf.get('primary_dns', '0.0.0.0')
            secondary_dns = mgetty_conf.get('secondary_dns', '0.0.0.0')
            if primary_dns == '':
                primary_dns = '0.0.0.0'
            if secondary_dns == '':
                secondary_dns = '0.0.0.0'

            if primary_dns != '0.0.0.0':
                response.append(u"ms-dns %s" % primary_dns)
            if secondary_dns != '0.0.0.0':
                response.append(u"ms-dns %s" % secondary_dns)

        elif ipcp_dns == 2:
            if ip_local != '0.0.0.0':
                response.append(u"ms-dns %s" % ip_local)
            else:
                eth0_ip = ipaddr.get_IPv4address('eth0')[0]['local']
                response.append(u"ms-dns %s" % eth0_ip)

        # defaultroute
        defaultroute = mgetty_conf.get('defaultroute', False)
        if defaultroute:
            response.append(u"defaultroute")
        else:
            response.append(u"nodefaultroute")

        # lcp-echo
        lcp_check = mgetty_conf.get('lcp_check', True)
        if lcp_check:
            lcp_interval = mgetty_conf.get('lcp_interval', 30)
            lcp_failure = mgetty_conf.get('lcp_failure', 3)
            response.extend([u"lcp-echo-interval %d" % lcp_interval,
                             u"lcp-echo-failure %d" % lcp_failure])

        # bps
        if mgetty_conf.get('dev_name', 'NTT docomo(UM02-F)') == 'NTT docomo(UM02-F)':
            # UM02-F̏ꍇ
            response.append(u"460800")
        else:
            response.append(u"115200")

        # vj
        vjcomp = mgetty_conf.get('vjcomp', True)
        if vjcomp:
            response.append(u"# novjccomp")
        else:
            response.append(u"novjccomp")

        # deflate
        deflatecomp = mgetty_conf.get('deflatecomp', True)
        if deflatecomp:
            response.append(u"# nodeflate")
        else:
            response.append(u"nodeflate")

        # mtu,mru
        mtu = mgetty_conf.get('mtu', 1500)
        if mtu > 1500:
            mtu = 1500
        response.extend([u"mtu %d" % mtu,
                         u"mru %d" % mtu])

        response.extend([u"name ppp50",
                         u"unit 50"])

        self.__response = response

    def respond(self):
        return u"%s\n" % '\n'.join(self.__response)
