#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   /etc/conf.d/net output module (replaced)
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
#
# Copyright (C) 2008 Century Systems Co.,Ltd.
# Last Modified: 2010/05/10 15:07:06 kikuchi
#

from datetime import datetime

DHCP_val2opt = {2 ** 0 : 'release',
                2 ** 1 : 'nodns',
                2 ** 2 : 'nontp',
                2 ** 3 : 'nogateway'}

class net(object):
    def __init__(self, searchList, append=None):
        response = [
            "#" * 80,
            "## /etc/conf.d/net",
            "## Generated by MA-E2xx WebUI at %s" % datetime.now().isoformat(' '),
            "#" * 80,
            ""]

        params = searchList.get('params', [])

        DNS_Server = params.get('DNSServer', {})
        dns = []

        for key in ('primary', 'secondary'):
            dns_tmp = DNS_Server.get(key, '')
            if dns_tmp != '':
                dns.append(dns_tmp)

        for eth_conf in params.get('Ethernet', {}):
            #-------------------------------------
            # modules_eth?
            #-------------------------------------
            module_list   = []
            dhcp_options  = []

            use_dhcp = eth_conf.get('use_dhcp', False)

            if use_dhcp == True:
                module_list.append('dhcpcd')

                if len(eth_conf.get('address', '')) > 0:
                    fallback = True
                    module_list.append('iproute2')
                else:
                    fallback = False

                # dhcp options
                dhcp_opts = eth_conf.get('dhcp_opts', 0)

                for (val, opt) in DHCP_val2opt.items():
                    if dhcp_opts & val:
                        dhcp_options.append(opt)
            else:
                module_list.append('iproute2')

            response.append('modules_%s="%s"' %
                            (eth_conf['name'],
                             ' '.join(['%s' % module for module in module_list])))

            #-------------------------------------
            # config_eth? / fallback_eth?
            #-------------------------------------
            if use_dhcp == True:
                response.append("config_%s=\"dhcpcd\"" % eth_conf['name'])

                if fallback == True:
                    response.append("fallback_%s=\"%s/%d\"" %
                                    (eth_conf['name'],
                                     eth_conf['address'],
                                     eth_conf['prefixlen']))

                response.append("dhcpcd_%s=\"-t 10\"" % eth_conf['name'])
            else:
                response.append("config_%s=\"%s/%d\"" %
                                (eth_conf['name'],
                                 eth_conf['address'],
                                 eth_conf['prefixlen']))

            #-------------------------------------
            # dhcp_eth?
            #-------------------------------------
            if len(dhcp_options) > 0:
                response.append("dhcp_%s=\"%s\"" %
                                (eth_conf['name'],
                                 ' '.join(dhcp_options)))

            #-------------------------------------
            # dns_servers_eth0
            #-------------------------------------
            if len(dns) > 0 and eth_conf['name'] == 'eth0':
                response.append("dns_servers_eth0=\"%s\"" %
                                (' '.join([x for x in dns])))

            # blank line
            response.append('')

        #-------------------------------------
        # routes_eth?
        #-------------------------------------
        for (gw_dev, route_list) in params.get('Gateways', {}).items():
            response.append('routes_%s="%s"' %
                            (gw_dev,
                             '\n'.join(['%s' % route for route in route_list])))

        # blank line
        response.append('')

        #-------------------------------------
        # iptunnel(GRE)
        #-------------------------------------
        for gre in params.get('GRE', {}):
            # iptunnel
            tmpl = "iptunnel_%(NAME)s=\"mode gre %(REMOTE)s %(LOCAL)s %(TTL)s\""
            arg = {}
            arg['NAME'] = gre['name']
            arg['REMOTE'] = "remote %s" % gre['remote_ip']

            if ("local_ip" in gre) and (gre['local_ip'] != ""):
                arg['LOCAL'] = "local %s" % gre['local_ip']
            else:
                arg['LOCAL'] = ""

            if ("ttl" in gre) and (gre['ttl'] > 0 and gre['ttl'] <= 255):
                arg['TTL'] = "ttl %d" % gre['ttl']
            else:
                arg['TTL'] = ""

            response.append(tmpl % arg)

            # config
            tmpl = "config_%(NAME)s=\"%(MY_ADDR)s\""
            arg = {}
            arg['NAME'] = gre['name']
            if ("if_address" in gre) and (gre["if_address"] != ""):
                arg['MY_ADDR'] = gre['if_address']
            else:
                arg['MY_ADDR'] = "null"

            response.append(tmpl % arg)

            # mtu
            if "mtu" in gre:
                response.append("mtu_%s=\"%d\"" % (gre['name'], gre['mtu']))

            # blank line
            response.append('')

        self.__response = response

    def __repr__(self):
        return "%s\n" % '\n'.join(self.__response)
