#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   /etc/ppp/chatscripts/ppp? output module
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
#
# Copyright (C) 2008-2013 Century Systems Co.,Ltd.
# Last Modified: 2013/10/22 09:25:21 kikuchi
#

from datetime import datetime

# Dialing AT command (key: tone)
dialtype = {False : 'ATDP',
            True  : 'ATDT',
            None  : 'ATD'}

pdptype = {0 : 'PPP',
           1 : 'IP'}

class ppp_chat(object):
    def __init__(self, searchList, append=None):
        params = searchList.get('params', {})

        if append == None:
            print "PPP_chat: UnitNo required."
            return

        unitNo = append

        callouts = params.get('PPP_callout', [])
        callout = {}

        for tmp in callouts:
            if tmp.get('unit', None) == unitNo:
                callout = tmp
                break

        if callout == {}:
            print "Error: configuration for UnitNo.%d not found." % unitNo
            return

        connectionName = callout.get('conn_name', u"'not set.'")
        #print "connectionName: %s" % connectionName

        if callout['is_modem'] == False:
            # PPPoE
            self.__response = [
                #-----------------
                # header
                u"#" * 80,
                u"## /etc/ppp/chatscript/ppp%d (name: %s)" % (unitNo, connectionName),
                u"##  *** ppp%d is PPPoE, chatscript not required." % (unitNo),
                u"## Generated by MA-E2xx WebUI at %s" % datetime.now().isoformat(' '),
                u"#" * 80]
            return

        apn = callout.get('apn', '')
        if apn:
            pdp_type = pdptype[callout.get('pdptype', 0)]
            apn_cmd = 'AT+CGDCONT=1,"%s","%s"' % (pdp_type, apn)
            atd_cmd = '%s' % dialtype[None]
        else:
            apn_cmd = 'AT'
            cipl_usable = callout.get('cipl_usable', False)
            if cipl_usable:
                # KCMV-200
                atd_cmd = '%s' % dialtype[None]
            else:
                atd_cmd = '%s' % dialtype[callout.get('tone', None)]

        if callout.get('dev_name', 'NTT docomo(UM03-KO)') == 'NTT docomo(UM03-KO)':
            # UM03-KOの場合
            if apn:
                pdp_type = pdptype[callout.get('pdptype', 0)]
                apn_cmd = 'AT+CGDCONT=1,"%s","%s"' % (pdp_type, apn)
            else:
                apn_cmd = 'AT'

            atd_cmd = '%s' % dialtype[None]

            self.__response = [
                #-----------------
                # header
                u"#" * 80,
                u"## /etc/ppp/chatscript/ppp%d (name: %s)" % (unitNo, connectionName),
                u"## Generated by MA-E2xx WebUI at %s" % datetime.now().isoformat(' '),
                u"#" * 80,
                u"",
                #-----------------
                # chatscript
                u"ABORT 'RING'",
                u"ABORT 'ERROR'",
                u"ABORT 'REWRITING'",
                u"ABORT 'RESTRICTION'",
                u"ABORT 'IN PACKET SERVICE'",
                u"ABORT 'OUT OF SERVICE'",
                u"ABORT 'PACKET SERVICE IN USE'",
                u"ABORT 'BUSY'",
                u"ABORT 'NO ANSWER'",
                u"ABORT 'NO DIALTONE'",
                u"ABORT 'DELAYED'",
                u"ABORT 'BLACKLISTED'",
                u"ABORT '+PNRII:1'",
                u"ABORT '*DANTE: 0'",
                u"TIMEOUT '5'",
                u"''    '\\d\\r\\nAT*DIFCHG'",
                u"OK    '\\d%s'" % callout.get('modeminit', 'AT&FS0=0'),
                u"ABORT 'NO CARRIER'",
                u"OK    '\\dATE0'",
                u"OK    '\\dAT*DANTE'",
                u"OK    '\\d%s'" % apn_cmd,
                u"OK    '\\d%s%s'" % (atd_cmd, callout.get('phone_number', '')),
                u"TIMEOUT '125'",
                u"CONNECT '\\d'"]
        else:
            self.__response = [
                #-----------------
                # header
                u"#" * 80,
                u"## /etc/ppp/chatscript/ppp%d (name: %s)" % (unitNo, connectionName),
                u"## Generated by MA-E2xx WebUI at %s" % datetime.now().isoformat(' '),
                u"#" * 80,
                u"",
                #-----------------
                # chatscript
                u"ABORT 'RING'",
                u"ABORT 'ERROR'",
                u"ABORT 'NO DIALTONE'",
                u"ABORT 'BUSY'",
                u"ABORT 'NO ANSWER'",
                u"ABORT 'IN PACKET SERVICE'",
                u"ABORT 'OUT OF SERVICE'",
                u"ABORT 'PACKET SERVICE IN USE'",
                u"ABORT 'RESTRICTION'",
                u"ABORT 'DELAYED'",
                u"ABORT 'BLACKLISTED'",
                u"TIMEOUT '5'",
                u"'' '\\r\\nAT'",
                u"OK '%s'" % callout.get('modeminit', 'AT&FS0=0'),
                u"ABORT 'NO CARRIER'",
                u"OK '%s'" % apn_cmd,
                u"OK '%s%s'" % (atd_cmd, callout.get('phone_number', '')),
                u"TIMEOUT 60",
                u"CONNECT ''"]

    def respond(self):
        return u"%s\n" % '\n'.join(self.__response)
