#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   /etc/ppp/peers/ppp? output module
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
#
# Copyright (C) 2008 Century Systems Co.,Ltd.
# Last Modified: 2013/07/31 12:51:24 kikuchi
#

from datetime import datetime

parameters = {
    'defaultroute' : {True  : 'defaultroute',
                      False : 'nodefaultroute'},
    'conn_type' : {0 : '# persist/demand',
                   1 : 'persist',
                   2 : 'demand'},
    'usepeerdns' : {False : '# usepeerdns',
                    True  : 'usepeerdns'},
    'vjcomp' : {True  : '# novjccomp',
                False : 'novjccomp'},
    'deflatecomp' : {True  : '# nodeflate',
                     False : 'nodeflate'},
    'ipparam' : {0 : 'normal',
                 1 : 'persist',
                 2 : 'demand'}
    }

class ppp_peers(object):
    def __init__(self, searchList, append=None):
        params = searchList.get('params', {})

        if append == None:
            print "PPP_peers: UnitNo required."
            return

        unitNo = append

        callouts = params.get('PPP_callout', [])
        callout = {}

        for tmp in callouts:
            if tmp.get('unit', None) == unitNo:
                callout = tmp
                break

        if callout == {}:
            print "Error: configuration for UnitNo.%d not found." % unitNo
            return

        connectionName = callout.get('conn_name', u"'not set.'")
        #print "connectionName: %s" % connectionName

        is_modem = callout.get('is_modem', True)

        response = [
            u"#" * 80,
            u"## /etc/ppp/peers/ppp%d (name: %s)" % (unitNo, connectionName),
            u"## Generated by MA-E2xx WebUI at %s" % datetime.now().isoformat(' '),
            u"#" * 80,
            u""]

        if is_modem == True:
            #-------------
            # device node
            response.append(u"%s" % callout['dev_node'])

        response.extend([u"maxfail 0",
                         u"debug",
                         u"kdebug 7",
                         u"noauth"])

        response.append(parameters['defaultroute'][callout.get('defaultroute', False)])
        response.append(parameters['usepeerdns'][callout.get('usepeerdns', False)])
        response.append(parameters['conn_type'][callout.get('conn_type', 0)])
        response.append("ipparam %s" % parameters['ipparam'][callout.get('conn_type', 0)])

        if callout.get('conn_type', 0) == 2:
            # demand
            response.append(u"idle %d" % callout.get('idle_time', 30))

        ip_local  = callout.get('ip_local', '0.0.0.0')
        ip_remote = callout.get('ip_remote', '0.0.0.0')

        if ip_local != '0.0.0.0' and ip_remote != '0.0.0.0':
            response.append(u"%s:%s" % (ip_local, ip_remote))
        elif ip_local != '0.0.0.0':
            response.append(u"%s:" % ip_local)
        elif ip_remote != '0.0.0.0':
            response.append(u":%s" % ip_remote)
        else:
            response.append(u"noipdefault")

        ip_netmask = callout.get('ip_netmask', '0.0.0.0')
        if ip_netmask != '0.0.0.0':
            response.append(u"netmask %s" % ip_netmask)

        response.extend([u"ipcp-accept-local",
                         u"ipcp-accept-remote"])

        response.append(u"holdoff 3")

        # LCP ECHO
        if callout.get('lcp_check', True) == True:
            lcp_interval = callout.get('lcp_interval', 30)
            lcp_failure  = callout.get('lcp_failure', 3)

            response.extend([u"lcp-echo-interval %d" % lcp_interval,
                             u"lcp-echo-failure %d" % lcp_failure])

        if is_modem == True:
            response.append(u"lock")
            if callout.get('dev_name', 'NTT docomo(UM02-F)') == 'NTT docomo(UM02-F)':
                # UM02-Fの場合
                response.append(u"460800")
            else:
                response.append(u"115200")
            response.extend([u"modem",
                             u"crtscts"])

            response.append(parameters['vjcomp'][callout.get('vjcomp', True)])
            response.append(parameters['deflatecomp'][callout.get('deflatecomp', True)])
        else:
            response.extend([u"asyncmap 0",
                             u"noccp",
                             u"nopcomp",
                             u"novj",
                             u"novjccomp",
                             u"nobsdcomp",
                             u"nodeflate",
                             u"noaccomp"])
            # rp-pppoe plugin
            response.append(u"plugin rp-pppoe.so %s" % callout.get('dev_node', 'eth1'))

        useaccount = callout.get('useaccount', False)
        if useaccount:
            response.append(u"user %s" % callout.get('account', 'nanashi-san'))

        if callout.get('domain', '') != '':
            response.append(u"domain %s" % callout['domain'])

        if is_modem == True:
            if callout.get('dev_name', 'NTT docomo(UM02-F)') == 'NTT docomo(UM02-F)':
                # UM02-Fの場合
                connect_str = [u"connect \"/opt/mae2xx/ppp/chat_wrapper_um02f.py"]
                connect_str.append(u"-v -f /etc/ppp/chatscripts/%s" % \
                        callout.get('name', 'ppp99'))
            else:
                connect_str = [u"connect \"/usr/sbin/chat -v -s -f"]
                connect_str.append(u"/etc/ppp/chatscripts/%s" % \
                        callout.get('name', 'ppp99'))
            #connect_str = [u"connect \"/etc/ppp/scripts/connect"]
            #connect_str.append(callout.get('name', 'ppp99'))
            #connect_str.append(callout.get('dev_node', '/dev/ttyXXX'))
            connect_str.append("\"")
            response.append(' '.join(connect_str))

        if is_modem == True:
            mtu = callout.get('mtu', 1500)
            if mtu > 1500:
                mtu = 1500
        else:
            mtu = callout.get('mtu', 1492)
            if mtu > 1492:
                mtu = 1492

        response.extend([u"mtu %d" % mtu,
                         u"mru %d" % mtu])

        response.append(u"unit %d" % unitNo)
        response.append(u"remotename ppp%d" % unitNo)

        self.__response = response

    def respond(self):
        return u"%s\n" % '\n'.join(self.__response)
