#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   Template FrontEnd Base
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
# Copyright (C) 2007-2008 Century Systems Co.,Ltd.
#
# Last Modified: 2010/05/10 15:13:27 kikuchi
#

import os
import copy

#===============================================================================
#
# TemplateModule FrontEnd Base
#
#===============================================================================
class Template_Base:
    def __init__(self, debug=False, prefix=''):
        self.debug = debug
        self.prefix = prefix
        self.params_set = False

    #-------------------------------------------------------
    #
    #-------------------------------------------------------
    def set_params(self, params):
        self.params = copy.copy(params)
        self.params_set = True

    #-------------------------------------------------------
    #
    #-------------------------------------------------------
    def get_response(self, params=None):
        if params:
            self.set_params(params)

        if self.params_set == False:
            return None

        results = {}

        #print "get_response(): debug = %s" % str(self.debug)

        if self.debug == False:
            idx = 0
            prefix = ''
        else:
            idx = 1
            prefix = self.prefix

        for module in self.module_list:
            # ファイル毎個別パラメータ
            append = module.get('append', None)

            #print "*** module : %s, append=%s" % (str(module), str(append))
            try:
                mod = module['module'](searchList={'params' : self.params},
                                       append=append)
            except Exception, msg:
                print "Error '%s'" % str(msg)
                # Cheetah module ?
                mod = module['module'](searchList={'params' : self.params})

            try:
                response = mod.respond()
            except:
                response = u"%s" % str(mod)

            file_mode = module.get('mode', 0644)

            results[module['name']] = {'filename' : "%s%s" % (prefix, module['file'][idx]),
                                       'file_mode' : file_mode,
                                       'contents' : response}

        return results

    #-------------------------------------------------------
    #
    #-------------------------------------------------------
    def write_config(self, params=None):
        results = self.get_response(params)

        if not results:
            raise Exception, "Parameters is not set."

        for (key, entry) in results.items():
            try:
                fileName = entry['filename']
                fileMode = entry['file_mode']
                contents = entry['contents'].encode('utf-8')

                chmod_flag = True
                if os.path.exists(fileName):
                    # 既存ファイルのモードは変更しない
                    chmod_flag = False

                fw = open(fileName, 'w')
                fw.write(contents)
                fw.close()

                if chmod_flag:
                    os.chmod(fileName, fileMode)

            except Exception, msg:
                print "Error: %s" % str(msg)
