#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   Template FrontEnd (Net)
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
# Copyright (C) 2007-2008 Century Systems Co.,Ltd.
#
# Last Modified: 2010/05/10 15:13:22 kikuchi
#

from Utils import Utils

# Template FrontEnd Base
from Template_Base import Template_Base

# Template modules
from Modules.net        import net
#from Modules.resolvconf import resolvconf

#===============================================================================
#
# TemplateModule FrontEnd (Network)
#
#===============================================================================
class Template_Net(Template_Base):
    def __init__(self, debug=False, prefix=''):
        self.super = Template_Base

        print ("Template_Net: debug = %s, prefix = '%s'"
               % (str(debug), prefix))

        self.module_list = [
            {'name'   : 'net',
             'file'   : ('/etc/conf.d/net',
                         'etc_confd_net'),
             'module' : net}]

        # Superclass Constructor
        self.super.__init__(self, debug, prefix)

    #-------------------------------------------------------
    #
    #-------------------------------------------------------
    def set_params(self, params):
        # 前処理
        gateways  = {}

        for eth in params['Ethernet']:
            gateway = str(eth.get('gateway', ''))
            if len(gateway) > 0:
                gateways[str(eth['name'])] = ["default via %s" % gateway,]

        for route in params['Route']:
            intf = str(route['gw_dev'])

            netmask = Utils.prefixlen2netmask(int(route.get('prefixlen', 24)))

            if not intf in gateways:
                gateways[intf] = []

            if route.get('gw_address', '') != '':
                # via Ethernet
                gateways[intf].append("%s netmask %s via %s" %
                                      (route['address'], netmask, route['gw_address']))
            else:
                # via PPP
                gateways[intf].append("%s netmask %s" %
                                      (route['address'], netmask))

        params['Gateways'] = gateways

        # Call Superclass
        self.super.set_params(self, params)


#===============================================================================
# self test
#===============================================================================
if __name__ == '__main__':
    #import syck
    import yaml

    conf_yaml1 = '\n'.join((
        "Ethernet:",
        "  eth0:",
        "    config: {dhcp: true, ipaddr: 192.168.253.253, netmask: 255.255.255.0}",
        "    dhcp_opts: [release, nontp]",
        "    modules: [dhcpcd, ifconfig]",
#       "  eth1:",
#       "    config: {dhcp: false, gateway: 192.168.254.2, ipaddr: 192.168.254.254, netmask: 255.255.255.0}",
#       "    modules: [ifconfig]",
        "Route:",
        "  route_info:",
        "    routes:",
        "    - {address: 172.16.0.0, gateway: 192.168.253.5, intf: eth0, netmask: 255.255.0.0}",
        "    - {address: 10.2.0.0, gateway: 192.168.254.10, intf: eth1, netmask: 255.255.0.0}",
        "DNSServer:",
        "  dns_info:",
        "    - '202.247.130.5'",
        "    - ''"))

    conf_yaml2 = '\n'.join((
        "Ethernet: ",
        "  eth0: ",
        "    modules: ",
        "    - ifconfig",
        "    config: ",
        "      ipaddr: 192.168.253.253",
        "      dhcp: False",
        "      netmask: 255.255.255.0",
        "      gateway: \"\"",
        "    dhcp_opts: []",
#       "  eth1: ",
#       "    dhcp_opts: []",
#       "    modules: ",
#       "    - ifconfig",
#       "    config: ",
#       "      netmask: 255.255.255.0",
#       "      dhcp: False",
#       "      ipaddr: 192.168.254.254",
#       "      gateway: \"\"",
        "DNSServer: ",
        "  dns_info: ",
        "  - 202.247.130.5",
        "  - \"\"",
        "Route: ",
        "  route_info: ",
        "    routes: ",
        "    - netmask: 255.255.0.0",
        "      address: 172.16.0.0",
        "      intf: eth0",
        "      gateway: 192.168.253.2",
        ""))

    params = syck.load(conf_yaml2)

    print params

    net_tmpl = Template_Net(debug=True, prefix='output_test/')

    net_tmpl.set_params(params)

    res = net_tmpl.get_response()

    for mod in res:
        print "### ModuleName: %s\n%s" % (mod, res[mod])

    net_tmpl.write_config()
