#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   Template FrontEnd (OCN IPv6)
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
# Copyright (C) 2007-2008 Century Systems Co.,Ltd.
#
# Last Modified: 2010/05/10 15:13:33 kikuchi
#

# Template FrontEnd Base
from Template_Base import Template_Base

# Template modules
from Modules.chap_secrets import chap_secrets
from Modules.conf_ocnv6   import conf_ocnv6
from Modules.l2tp_conf    import l2tp_conf
from Modules.dhcp6c_conf  import dhcp6c_conf
from Modules.v6tun        import v6tun

#===============================================================================
#
# TemplateModule FrontEnd (OCN IPv6)
#
#===============================================================================
class Template_OCNIPv6(Template_Base):
    def __init__(self, debug=False, prefix=''):
        self.super = Template_Base

        self.module_list = [
            {'name'   : 'chap_secrets',
             'file'   : ('/etc/ppp/chap-secrets',
                         'etc_ppp_chap-secrets'),
             'module' : chap_secrets},
            {'name'   : 'conf_ocnv6',
             'file'   : ('/etc/OCNIPv6/ocnv6',
                         'etc_confd_ocnv6'),
             'module' : conf_ocnv6},
            {'name'   : 'rp-l2tp_conf',
             'file'   : ('/etc/rp-l2tp/rp-l2tpd.conf',
                         'etc_rp-l2tp_rp-l2tpd_conf'),
             'module' : l2tp_conf},
            {'name'   : 'dhcp6c_conf',
             'file'   : ('/etc/dhcp6c.conf',
                         'etc_dhcp6c_conf'),
             'module' : dhcp6c_conf},
            {'name'   : 'v6tun',
             'file'   : ('/etc/ppp/peers/v6tun',
                         'etc_ppp_peers_v6tun'),
             'module' : v6tun}]

        # Superclass Constructor
        self.super.__init__(self, debug, prefix)


#===============================================================================
# self test
#===============================================================================
if __name__ == '__main__':
    #import syck
    import yaml

    conf_yaml1 = '\n'.join((
        "OCNv6: ",
        "  &id001 account_info: ",
        "    use_ocnv6: True",
        "    LNS_sel: \"0\"",
        "    L2TP_server: v6tffd1.ocn.ne.jp",
        "    L2TP_port: \"1701\"",
        "    userID: test@ocn.ne.jp",
        "    password: hogehoge\n"))

    params = syck.load(conf_yaml1)

    print params

    ocnv6_tmpl = Template_OCNIPv6(debug=True, prefix='output_test/')

    ocnv6_tmpl.set_params(params)

    res = ocnv6_tmpl.get_response()

    for mod in res:
        print "### ModuleName: %s\n%s" % (mod, res[mod])

    ocnv6_tmpl.write_config()
