#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   Template FrontEnd (PPP callout)
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
# Copyright (C) 2008 Century Systems Co.,Ltd.
#
# Last Modified: 2010/05/10 15:13:40 kikuchi
#

from Utils import Utils
import syslog

# Template FrontEnd Base
from Template_Base import Template_Base

# Template modules
from Modules.ppp_peers    import ppp_peers
from Modules.ppp_chat     import ppp_chat
from Modules.chap_secrets import chap_secrets

syslog_header = "Template_PPP_callout"

#===============================================================================
#
# TemplateModule FrontEnd (PPP callout)
#
#===============================================================================
class Template_PPP_callout(Template_Base):
    def __init__(self, debug=False, prefix=''):
        self.super = Template_Base

        self.module_list_base = [
            {'name'   : 'chap_secrets',
             'file'   : ('/etc/ppp/chap-secrets',
                         'etc_ppp_chap-secrets'),
             'module' : chap_secrets},
            {'name'   : 'pap_secrets',
             'file'   : ('/etc/ppp/pap-secrets',
                         'etc_ppp_pap-secrets'),
             'module' : chap_secrets}]

        # Superclass Constructor
        self.super.__init__(self, debug, prefix)

    #-------------------------------------------------------
    #
    #-------------------------------------------------------
    def set_params(self, params):
        # 前処理
        self.module_list = []
        self.module_list.extend(self.module_list_base)

        callouts = params.get('PPP_callout', [])

        for callout in callouts:
            try:
                unit = callout['unit']
            except:
                syslog.syslog("%s: 'unit' not defined, continue." % syslog_header)

            print "unitNo: %d" % unit
            self.module_list.append({'name'   : 'ppp_peers_ppp%d' % unit,
                                     'file'   : ('/etc/ppp/peers/ppp%d' % unit,
                                                 'etc_ppp_peers-ppp%d' % unit),
                                     'append' : unit,
                                     'module' : ppp_peers})
            self.module_list.append({'name'   : 'ppp_chat_ppp%d' % unit,
                                     'file'   : ('/etc/ppp/chatscripts/ppp%d' % unit,
                                                 'etc_ppp_chatscripts-ppp%d' % unit),
                                     'append' : unit,
                                     'module' : ppp_chat})

        # Call Superclass
        self.super.set_params(self, params)
