#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   Template FrontEnd (dnsmasq)
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
# Copyright (C) 2007-2008 Century Systems Co.,Ltd.
#
# Last Modified: 2010/05/10 15:13:53 kikuchi
#

# Template FrontEnd Base
from Template_Base import Template_Base

# Template modules
from Modules.dnsmasq_conf import dnsmasq_conf

#===============================================================================
#
# TemplateModule FrontEnd (dnsmasq)
#
#===============================================================================
class Template_dnsmasq(Template_Base):
    def __init__(self, debug = False, prefix = ''):
        self.super = Template_Base

        self.module_list = [
            {'name'   : 'dnsmasq.conf',
             'file'   : ('/etc/dnsmasq.conf',
                         'etc_dnsmasq_conf'),
             'module' : dnsmasq_conf}]

        # Superclass Constructor
        self.super.__init__(self, debug, prefix)


#===============================================================================
# self test
#===============================================================================
if __name__ == '__main__':
    #import syck
    import yaml

    conf_yaml1 = '\n'.join((
        "Ethernet:",
        "  eth0:",
        "    config: {dhcp: true, ipaddr: 192.168.253.253, netmask: 255.255.255.0}",
        "    dhcp_opts: [release, nontp]",
        "    modules: [dhcpcd, ifconfig]",
        "  eth1:",
        "    config: {dhcp: false, gateway: 192.168.254.2, ipaddr: 192.168.254.254, netmask: 255.255.255.0}",
        "    modules: [ifconfig]",
        "Route:",
        "  route_info:",
        "    routes:",
        "    - {address: 172.16.0.0, gateway: 192.168.253.5, intf: eth0, netmask: 255.255.0.0}",
        "    - {address: 10.2.0.0, gateway: 192.168.254.10, intf: eth1, netmask: 255.255.0.0}",
        "DNSServer:",
        "  dns_info:",
        "    - '202.247.130.5'",
        "    - ''"))

    conf_yaml2 = '\n'.join((
        "Ethernet:",
        "  eth0:",
        "    config:",
        "      dhcp: False",
        "      gateway: \"\"",
        "      ipaddr: 192.168.253.253",
        "      netmask: 255.255.255.0",
        "    modules:",
        "    - ifconfig",
        "    dhcp_opts: []",
        "",
        "  eth1:",
        "    modules:",
        "    - ifconfig",
        "    config:",
        "      netmask: 255.255.255.0",
        "      dhcp: False",
        "      ipaddr: 192.168.254.254",
        "",
        "DHCPServer:",
        "  dhcp_info:",
        "    lease:",
        "    - 192.168.253.2",
        "    - 192.168.253.50",
        "    dhcp_server: True",
        "",
        "NTP:",
        "  ntp_info:",
        "    use_ntp: True",
        "    ntp_servers:",
        "    - ntp.nict.jp",
        "    - \"\"",
        "    act_server: True",
        ""))

    params = syck.load(conf_yaml2)

    print params

    net_tmpl = Template_dnsmasq(debug=True, prefix='output_test/')

    net_tmpl.set_params(params)

    res = net_tmpl.get_response()

    for mod in res:
        print "### ModuleName: %s\n%s" % (mod, res[mod])

    net_tmpl.write_config()
