#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   X-WSSE Authentication Module
#
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
# Copyright (C) 2007-2008 Century Systems Co.,Ltd.
#
# Last Modified: 2010/05/10 15:17:12 kikuchi
#

import cherrypy
import datetime, time
import base64, sha
import httplib
import urllib2

import syslog

#===============================================================================
#
#===============================================================================
class queue:
    def __init__(self, depth = 100):
        self.__queue = []
        self.__timeInfo = {}
        self.__depth = depth

    def put(self, obj):
        try:
            idx = self.__queue.index(obj)
            return
        except ValueError:
            self.__queue.append(obj)
            self.__timeInfo[str(obj)] = datetime.datetime.now()
            self.__check_depth()

    def auth_check(self, obj):
        try:
            idx = self.__queue.index(obj)

            now = datetime.datetime.now()
            digest_time = self.__timeInfo[str(obj)]
            diff = now - digest_time
            diff_seconds = diff.days * (3600 * 24) + diff.seconds

            if diff_seconds < 3:
                print "digest/time is same, ok"
                return False

            return True
        except ValueError:
            print "now found, ok"
            self.put(obj)
            return False

    def get_length(self):
        return len(self.__queue)

    def __check_depth(self):
        over = len(self.__queue) - self.__depth

        print "queue length: %d" % (over + self.__depth)
        for i in range(over):
            key = self.__queue.pop(0)
            del(self.__timeInfo[key])

__password_hist = queue(100)

#===============================================================================
#
# WSSE Authentication tool
#
#===============================================================================
def wsse_auth(realm, get_password):
    if cherrypy.request.headers.has_key('X-WSSE'):
        wsse = cherrypy.request.headers['X-WSSE']

        return wsse_auth_lib(realm, get_password, wsse)
    else:
        raise cherrypy.HTTPError(401, "You are not authorized to access that resource")

def wsse_auth_lib(realm, get_password, wsse):
    auth_scheme, auth_params = wsse.split(' ', 1)

    if auth_scheme == "UsernameToken":
        params = urllib2.parse_keqv_list(urllib2.parse_http_list(auth_params))

        p_digest = params['PasswordDigest']
        username = params['Username']
        created  = params['Created']
        nonce    = params['Nonce']

        try:
            password = get_password(username)
        except KeyError:
            raise cherrypy.HTTPError(401, "Account '%s' not registered." % username)

        #print "PasswordDigest : '%s'" % p_digest
        #print "Username       : '%s'" % username
        #print "Created        : '%s'" % created
        #print "Nonce          : '%s'" % nonce

        nonce = base64.decodestring(nonce)

        digest_calc = base64.encodestring(sha.sha(nonce + created + password).digest()).strip()
        #digest_calc = base64.encodestring(sha.sha(nonce + created + pass_digest).digest()).strip()

        #print "CalculatedDigest : '%s'" % digest_calc

        if p_digest == digest_calc:
            print "Authentication OK!"

            # check replay attack
            if __password_hist.auth_check(digest_calc):
                raise cherrypy.HTTPError(401, "Replay attack detected.")

            return

    raise cherrypy.HTTPError(401, "You are not authorized to access that resource")

cherrypy.tools.wsse_auth = cherrypy.Tool('on_start_resource', wsse_auth)
