#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#   PPP status management module
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
#
# Copyright (C) 2008-2009 Century Systems Co.,Ltd.
# Last Modified: 2013/09/03 16:52:21 kikuchi
#

import glob
import os
import re
import pyiproute

# Configuration DB
from DB import config, models

# pyiproute ipaddr/iplink instance
ipaddr = pyiproute.ipaddr()

# DeviceName --> DeviceNode
DeviceNameTable = [
    # PPPoE (eth0)
    {"devname"   : "PPPoE (eth0)",
     "device"    : "eth0",
     "modem"     : False,
     "type"      : 0,
     "nocheck"   : True,
     "exclusive" : False},
    # PPPoE (eth1)
#    {"devname"   : "PPPoE (eth1)",
#     "device"    : "eth1",
#     "modem"     : False,
#     "type"      : 0,
#     "nocheck"   : True,
#     "exclusive" : False},
    # External Modem (USB)
    {"devname"   : "Modem(USB)",
     "device"    : "/dev/ttyUSBModem",
     "modem"     : True,
     "type"      : 0,
     "nocheck"   : False,
     "pulseok"   : True,
     "dialin"    : True},
    # External Modem (PORT0)
    {"devname"   : "Modem(PORT0)",
     "device"    : "/dev/ttymxc0",
     "modem"     : True,
     "type"      : 0,
     "nocheck"   : True,
     "pulseok"   : True,
     "dialin"    : True},
    # External Modem (PORT1)
    {"devname"   : "Modem(PORT1)",
     "device"    : "/dev/ttymxc1",
     "modem"     : True,
     "type"      : 0,
     "nocheck"   : True,
     "pulseok"   : True,
     "dialin"    : True},
    # NTT DoCoMo (CF)
    {"devname"   : "NTT DoCoMo(CF)",
     "device"    : "/dev/ttyNTTDoCoMoPCMCIA",
     "modem"     : True,
     "type"      : 0,
     "nocheck"   : False,
     "pulseok"   : False,
     "dialin"    : False},
    # NTT DoCoMo (USB)
    {"devname"   : "NTT DoCoMo(USB)",
     "device"    : "/dev/ttyNTTDoCoMoUSB",
     "modem"     : True,
     "type"      : 0,
     "nocheck"   : False,
     "pulseok"   : False,
     "dialin"    : False},
    # NTT docomo (CF)
    {"devname"   : "NTT docomo(CF)",
     "device"    : "/dev/ttyNTTdocomoPCMCIA",
     "modem"     : True,
     "type"      : 0,
     "nocheck"   : False,
     "pulseok"   : False,
     "dialin"    : False},
    # NTT docomo (USB)
    {"devname"   : "NTT docomo(USB)",
     "device"    : "/dev/ttyNTTdocomoUSB",
     "modem"     : True,
     "type"      : 0,
     "nocheck"   : False,
     "pulseok"   : False,
     "dialin"    : False},
    # NTT docomo (UM01-F)
#   {"devname"   : "NTT docomo(UM01-F)",
#    "device"    : "/dev/ttyUM01F",
#    "modem"     : True,
#    "type"      : 1,
#    "nocheck"   : False,
#    "pulseok"   : False,
#    "dialin"    : True,
#    "force_lcp_check" : True,
#    "no_persist" : True,
#    "need_apn" : True,
#    "account_default" : {
#       "modeminit" : "AT&FS0=0",
#       "phone_no"  : "*99***1#"}},
    # NTT docomo (UM02-F)
    {"devname"   : "NTT docomo(UM02-F)",
     "device"    : "/dev/ttyUM02F",
     "modem"     : True,
     "type"      : 1,
     "nocheck"   : False,
     "pulseok"   : False,
     "dialin"    : True,
     "force_lcp_check" : True,
     "need_apn" : True,
     "account_default" : {
        "modeminit" : "AT&FS0=0",
        "phone_no"  : "*99***1#"}},
    # NTT docomo (UM01-HW)
    {"devname"   : "NTT docomo(UM01-HW)",
     "device"    : "/dev/ttyUM01HW",
     "modem"     : True,
     "type"      : 1,
     "nocheck"   : False,
     "pulseok"   : False,
     "dialin"    : True,
     "force_lcp_check" : True,
     "need_apn" : True,
     "account_default" : {
        "modeminit" : "AT&FS0=0",
        "phone_no"  : "*99***1#"}},
    # NTT docomo (UM03-KO)
    {"devname"   : "NTT docomo(UM03-KO)",
     "device"    : "/dev/ttyUM03KO",
     "modem"     : True,
     "type"      : 1,
     "nocheck"   : False,
     "pulseok"   : False,
     "dialin"    : True,
     "force_lcp_check" : True,
     "need_apn" : True,
     "account_default" : {
        "modeminit" : "AT&FS0=0",
        "phone_no"  : "*99***1#"}},
    # Softbank (CF)
    {"devname"   : "Softbank(CF)",
     "device"    : "/dev/ttySoftbankPCMCIA",
     "modem"     : True,
     "type"      : 0,
     "nocheck"   : False,
     "pulseok"   : False,
     "dialin"    : False,
     "account_default" : {"modeminit" : "ATZ",
                          "phone_no"  : "*99#",
                          "account"   : "softbank",
                          "password"  : "softbank"}},
    # Softbank (USB)
    {"devname"   : "Softbank(USB)",
     "device"    : "/dev/ttySoftbankUSB",
     "modem"     : True,
     "type"      : 0,
     "nocheck"   : False,
     "pulseok"   : False,
     "dialin"    : False,
     "account_default" : {"phone_no" : "*99***1#",
                          "account"  : "softbank",
                          "password" : "softbank"}},
    # Willcom (CF)
    {"devname"   : "Willcom(CF)",
     "device"    : "/dev/ttyWillcomPCMCIA",
     "modem"     : True,
     "type"      : 0,
     "nocheck"   : False,
     "pulseok"   : False,
     "dialin"    : True,
     "account_default" : {"phone_no" : "0570570711##64",
                          "account"  : "prin",
                          "password" : "prin"}},
    # Willcom (USB)
    {"devname"   : "Willcom(USB)",
     "device"    : "/dev/ttyWillcomUSB",
     "modem"     : True,
     "type"      : 0,
     "nocheck"   : False,
     "pulseok"   : False,
     "dialin"    : True,
     "account_default" : {"phone_no" : "0570570711##64",
                          "account"  : "prin",
                          "password" : "prin"}},
    # KDDI (CF)
    {"devname"   : "KDDI(CF)",
     "device"    : "/dev/ttyKDDIPCMCIA",
     "modem"     : True,
     "type"      : 0,
     "nocheck"   : False,
     "pulseok"   : False,
     "dialin"    : False,
     "account_default" : {"phone_no" : "*99**24#",
                          "account"  : "au@au-win.ne.jp",
                          "password" : "au"}},
    # KDDI (USB)
    {"devname"   : "KDDI(USB)",
     "device"    : "/dev/ttyKDDIUSB",
     "modem"     : True,
     "type"      : 0,
     "nocheck"   : False,
     "pulseok"   : False,
     "dialin"    : False,
     "account_default" : {"phone_no" : "*99**24#",
                          "account"  : "au@au-win.ne.jp",
                          "password" : "au"}},
#   # KDDI (KCMV-200)
#   {"devname"   : "KDDI(KCMV-200)",
#    "device"    : "/dev/ttyKCMV200a",
#    "modem"     : True,
#    "type"      : 2,
#    "nocheck"   : False,
#    "pulseok"   : False,
#    "dialin"    : False,
#    "account_default" : {
#        "modeminit" : "ATQ0E0V1",
#        "phone_no" : "*99**24#"}},
    # emobile (CF)
    {"devname"   : "emobile(CF)",
     "device"    : "/dev/ttyEmobileCF",
     "modem"     : True,
     "type"      : 0,
     "nocheck"   : False,
     "pulseok"   : False,
     "dialin"    : False,
     "account_default" : {"phone_no"  : "*99***1#",
                          "account"   : "em",
                          "password"  : "em"}},
    # emobile (USB)
    {"devname"   : "emobile(USB)",
     "device"    : "/dev/ttyEmobileUSB",
     "modem"     : True,
     "type"      : 0,
     "nocheck"   : False,
     "pulseok"   : False,
     "dialin"    : False,
     "account_default" : {"modeminit" : 'AT+cgdcont=1,"ip","emb.ne.jp"',
                          "phone_no"  : "*99***1#",
                          "account"   : "em",
                          "password"  : "em"}},
    ]

if config.debug == False:
    PPP_stat_dir = "/var/state/ppp"
else:
    PPP_stat_dir = "./state_ppp"


#---------------------------------------------------------------------
# 使用可能デバイス一覧取得
# [
#  {'devname':         デバイス名(画面表示),
#   'device':          デバイス名(/dev/...),
#   'modem':           モデムかどうか(pppoeはfalse),
#   'type':            種別(UM02-F=1, KCMV200=2, その他=0),
#   'pulseok':         パルス指定可,
#   'execlusive':      同時利用不可(pppoeは可),
#   'dialin':          着信可,
#   'force_lcp_check': LCP Echoを強制,
#   'no_persist':      persist選択不可,
#   'need_apn':        電話番号の代わりにAPNを指定,
#   'account_dafault': デフォルトパラメータ
#     {
#       'modeminit':     初期化ATコマンド,
#       'phone_no':      電話番号,
#       'account':       アカウント,
#       'password':      パスワード
#     }
#  }, ...
# ]
#---------------------------------------------------------------------
def get_available_devices(dialin=False):
    res = []

    for entry in DeviceNameTable:
        dev = {'devname'           : entry['devname'],
               'device'            : entry['device'],
               'modem'             : entry.get('modem', False),
               'type'              : entry.get('type', 0),
               'pulseok'           : entry.get('pulseok', False),
               'exclusive'         : entry.get('exclusive', True),
               'dialin'            : entry.get('dialin', False),
               'force_lcp_check'   : entry.get('force_lcp_check', False),
               'no_persist'        : entry.get('no_persist', False),
               'need_apn'          : entry.get('need_apn', False),
               'account_default'   : entry.get('account_default', {})}

        # MA-E250/Fの場合
        foma = os.path.isfile("/proc/driver/foma_status")
        if foma == True:
            # チェック不要デバイス もしくは デバイスが存在し、UM-02Fである
            if entry['nocheck'] == True or \
               (__find_device(entry['device']) == True and \
                entry.get('need_apn', False) == True):
                # 着信ではない もしくは 着信可能デバイス
                if dialin == False or entry.get('dialin', False) == True:
                    res.append(dev)
        else:
            # チェック不要デバイス もしくは デバイスが存在
            if entry['nocheck'] == True or __find_device(entry['device']) == True:
                # 着信ではない もしくは 着信可能デバイス
                if dialin == False or entry.get('dialin', False) == True:
                    if entry['device'] != "/dev/ttyUM02F":
                        res.append(dev)

    return res

PPP_status_table = {
    "NOTEXIST"     : 0,
    "NOTEXIST_UP"  : 1,
    "STOPPED"      : 2,
    "BOOTED"       : 3,
    "DORMANT"      : 4,
    "INUSE"        : 5,
    "CONNECTED"    : 6,
    "DISCONNECTED" : 7,

    0 : u"デバイス無し",
    1 : u"デバイス無し(待機中)",
    2 : u"未接続",
    3 : u"接続処理中",
    4 : u"休止",
    5 : u"他で使用中",
    6 : u"接続中",
    7 : u"回線断"}

#---------------------------------------------------------------------
# PPP(発信) 状態取得
#---------------------------------------------------------------------
def get_PPP_callout_status(intf='all'):
    ppp_confs = models.vPPP.select_all()
    ppp_devices = get_available_devices()
    ppp_stats = __check_status()

    devices_in_use = []

    for entry in ppp_confs:
        status = 0
        address_info = {}

        for dev_info in DeviceNameTable:
            if entry['dev_node'] == dev_info['device']:
                entry['exclusive'] = dev_info.get('exclusive', True)
                break

        ### デバイス存在確認
        for dev in ppp_devices:
            if entry['dev_node'] == dev['device']:
                # device exists
                status = 2
                break

        ### PPP接続状態確認 (/var/state/ppp/ppp?)
        if status == 2:
            pppname = entry['name']
            print "# pppname = %s" % pppname

            if pppname in ppp_stats:
                print " found in ppp_stats: ppp_stats[%s] = %s" % (pppname,
                                                                   ppp_stats[pppname])
                address_info = ppp_stats[pppname]['address']

                if ppp_stats[pppname]['status'] == 'UP':
                    # PPP link is UP
                    status = 6
                elif ppp_stats[pppname]['status'] == 'BOOT':
                    # PPP connecting...
                    status = 3
                elif ppp_stats[pppname]['status'] == 'DORMANT':
                    # PPP link is DORMANT
                    status = 4
                elif ppp_stats[pppname]['status'] == 'DISCONNECTED':
                    # pppd was finished.
                    status = 7

                devices_in_use.append(ppp_stats[pppname]['device'])

        entry['status'] = {'value'   : status,
                           'string'  : PPP_status_table[status],
                           'address' : address_info}

    ### 他のLinkで使用中か確認
    # 重複起動確認は行わない
    #for entry in ppp_confs:
    #   if entry['exclusive'] == False:
    #       # 共有可能デバイス (PPPoE)
    #       continue

    #   if entry['status']['value'] == 2:
    #       if entry['dev_node'] in devices_in_use:
    #           status = 5

    #           entry['status']['value'] = status
    #           entry['status']['string'] = PPP_status_table[status]

    if intf != 'all':
        for entry in ppp_confs:
            if entry['name'] == intf:
                return entry
        # not found
        return None

    return ppp_confs

#---------------------------------------------------------------------
# PPP(着信) 状態取得
#---------------------------------------------------------------------
def get_PPP_dialin_status():
    ppp_confs = models.DialIn.select_all()
    ppp_devices = get_available_devices()
    ppp_stats = __check_status()

    devices_in_use = []

    for entry in ppp_confs:
        status = 0
        address_info = {}

        ### デバイス存在確認
        for dev in ppp_devices:
            if entry['dev_node'] == dev['device']:
                # device exists
                status = 2
                break

        ### PPP接続状態確認 (/var/stat/ppp/ppp?)
        if status == 2:
            pppname = entry['name']

            print "# pppname = %s" % pppname

            if pppname in ppp_stats:
                print " found in ppp_stats: ppp_stats[%s] = %s" % (pppname,
                                                                   ppp_stats[pppname])

                address_info = ppp_stats[pppname]['address']

                if ppp_stats[pppname]['status'] == 'UP':
                    # PPP link is UP
                    status = 6
                elif ppp_stats[pppname]['status'] == 'DORMANT':
                    # PPP link is DORMANT
                    status = 4

                devices_in_use.append(ppp_stats[pppname]['device'])

        entry['status'] = {'value'   : status,
                           'string'  : PPP_status_table[status],
                           'address' : address_info}

    ### 他のLinkで使用中か確認
    # 重複起動確認は行わない
    #for entry in ppp_confs:
    #   if entry['status']['value'] == 1:
    #       if entry['dev_node'] in devices_in_use:
    #           status = 5

    #           entry['status']['value'] = status
    #           entry['status']['string'] = PPP_status_table[status]

    return ppp_confs
    

#---------------------------------------------------------------------
#
#---------------------------------------------------------------------
def __find_device(device):
    result = False
    files = glob.glob(device)

    if len(files) == 1:
        result = True

    return result

#---------------------------------------------------------------------
#
#---------------------------------------------------------------------
def __check_status():
    res = {}

    cwd = os.getcwd()

    # サービス起動状態
    try:
        os.chdir('/lib/rc/init.d/started')
    except OSError, msg:
        print "os.chdir(): %s" % msg
        return {}

    statfiles = glob.glob("net.ppp*")

    ppps = models.vPPP.select_all()

    for booted in statfiles:
        try:
            pppname = re.match('net.(ppp[0-9]+)', booted).groups()[0]

            device = 'UNKNOWN'

            for ppp in ppps:
                if ppp['name'] == pppname:
                    device = ppp['dev_node']
                    break

            res[pppname] = {'status'  : 'BOOT',
                            'device'  : device,
                            'address' : {}}
        except Exception, msg:
            print "Error, '%s'" % str(msg)

    os.chdir(cwd)

    # /var/state/ppp/ 
    try:
        os.chdir(PPP_stat_dir)
    except OSError, msg:
        print "os.chdir(): %s" % msg
        return {}

    # ip-up/down status file
    statfiles = glob.glob("ppp*")

    for pppname in statfiles:
        try:
            if res.get(pppname, {}).get('status', '') != 'BOOT':
                continue

            f = open(pppname, 'r')
        except:
            continue

        try:
            info = f.read().split()

            print "info: %s" % str(info)
            stat = res[pppname]
            if len(info) == 4:
                # UP
                stat['status'] = info[0]
                stat['address'] = {'local' : info[2], 'peer' : info[3]}
            elif len(info) == 2 and info[0] == 'DOWN':
                # DORMANT
                stat['status'] = 'BOOT'
        finally:
            f.close()

    # ppp_wrapper result file
    resultfiles = glob.glob("result*")
    for resultfile in resultfiles:
        try:
            pppname = re.match('result.(ppp[0-9]+)', resultfile).groups()[0]
            if res.get(pppname, {}).get('status', '') == 'BOOT':
                stat = res[pppname]
                stat['status'] = 'DISCONNECTED'
        except Exception, msg:
            print "Error, '%s'" % str(msg)

    os.chdir(cwd)

    # from pyiproute
    infos = ipaddr.get()

    for (idx, info) in infos.iteritems():
        link  = info.get('link', {})
        addrs = info.get('addr', None)

        if type(addrs) != list:
            continue

        addr = addrs[0]

        if link.get('link', '') == 'ppp':
            pppname = addr.get('label', '')

            if pppname == '':
                continue

            if res.get(pppname, {}).get('status', '') == 'BOOT':
                conn_type = 0
                for ppp in ppps:
                    if ppp['name'] == pppname:
                        conn_type = int(ppp['conn_type'])
                        break

                if conn_type == 2: # demand
                    # Demand 'Dormant' state
                    stat = res[pppname]

                    stat['status']  = 'DORMANT'
                    stat['address'] = {'local' : addr['local'],
                                       'peer'  : addr['peer']}

                    res[pppname] = stat

    print "__check_status: %s" % str(res)
    return res
