#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-4xx/8xx/E2xx Web Application Server Module
#
# Copyright (C) 2008 Century Systems Co.,Ltd.
#

import threading
import os
import time

class Serlock(object):

    def __init__(self, device):
        self.__device = os.path.basename(device)
        if not self.__device:
            raise Exception('device name error.')

        self.__lockfile_name = '/var/lock/LCK..%s' % self.__device
        self.__pid = os.getpid()
        self.__mutex = threading.Lock()

    # ロック取得
    def acquire(self, wait = True):
        r = self.__mutex.acquire(wait)
        if not r:
            return False

        try:
            pid = self.__readLockFile()
            if pid < 0:
                self.__createLockFile()
                return True

        except Exception, err:
            self.__log("acquire() error: '%s'" % str(err))
            self.__mutex.release()
            raise err

        if pid == self.__pid:
            return True

        # ノンブロックモード
        if wait == False:
            self.__mutex.release()
            return False

        # ロックファイルが消えるのを待つ。
        while True:
            time.sleep(1)
            if not os.path.exists(self.__lockfile_name):
                self.__createLockFile()
                break

        return True

    # ロック解放
    def release(self):
        try:
            pid = self.__readLockFile()
            if pid == self.__pid:
                self.__removeLockFile()
        except Exception, err:
            self.__log("release() error: '%s'" % str(err))

        self.__mutex.release()

    def isLocked(self):
        return os.path.exists(self.__lockfile_name)

    # ロギング
    def __log(self, msg):
        print "Serlock:", msg

    # ロックファイル読み出し
    def __readLockFile(self):
        if os.path.exists(self.__lockfile_name):
            try:
                f = open(self.__lockfile_name, 'r')
                tmp = f.read()
                pid = int(tmp)
                f.close()
            except Exception, err:
                self.__log("__readLockFile() error: '%s'" % str(err))
                return -1

            return pid

        else:
            return -1

    # ロックファイル作成
    def __createLockFile(self):
        try:
            f = open(self.__lockfile_name, 'w+')
            f.write('%d' % self.__pid)
            f.close()
        except Exception, err:
            self.__log("__createLockFile() error: '%s'" % str(err))

    # ロックファイル削除
    def __removeLockFile(self):
        try: 
            os.unlink(self.__lockfile_name)
        except Exception, err:
            self.__log("__removeLockFile() error: '%s'" % str(err))
