//==============================================================================
// MA-4xx/8xx/E2xx WebUI Module
//
// WebUI パスワード設定
// Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
// Copyright (C) 2007-2010 Century Systems Co., Ltd.
//
// Last Modified: 2010/05/28 15:03:30 kikuchi
//==============================================================================

mod_account = {
    //--------------------------------------------
    // HTML template
    //--------------------------------------------
    template : {
	    HTMLtemplate : [
	        "<h1 class='title'>WebUIパスワード設定</h1>",
	        "<table id='account_table' class='zebra'>",
	        "<tr>",
	        "  <th id='th_account'>アカウント</th>",
	        "  <td class='noborder'>",
	        "    <input type='text' id='txt_account' class='text' value='${account}'",
	        "     style='width:200px;' tabindex=1 />",
	        "  </td>",
	        "</tr>",
	        "<tr>",
	        "  <th>パスワード</th>",
	        "  <td class='noborder'>",
	        "    <input type='password' id='txt_password1' class='text'",
	        "     style='width:200px;' tabindex=2 />",
	        "  </td>",
	        "</tr>",
	        "<tr>",
	        "  <th>パスワード (確認)</th>",
	        "  <td class='noborder'>",
	        "    <input type='password' id='txt_password2' class='text'",
	        "     style='width:200px;' tabindex=3 />",
	        "  </td>",
	        "</tr>",
	        "<tr>",
	        "  <td colspan=2 class='noborderright'>",
	        "    <input type='button' id='bt_update' value='変更'",
	        "     class='button' tabindex=4 />",
	        "  </td>",
	        "</tr>",
	        "</table>"
	    ].join("\n"),
	    handlers : [
	        [
		        // "Update" button handler
		        'bt_update', 'click', function() {
		            mod_account.onUpdate();
		        }]
	    ],
	    tooltips : [
	        [
		        //
		        'th_account', 'Note', ["WebUIにログインするためのアカウントを指定します。",
				                       "ここで指定したアカウント以外ではログインできません。"].join('')
	        ]
	    ]
    },

    //----------------------------------
    // dummy data
    //----------------------------------
    data : {
	    account  : '',
	    password : '',
	    digest   : ''
    },

    //--------------------------------------------
    // Initialize
    //--------------------------------------------
    init : function(elem) {
	    var auth_data;
	    this.elem = elem;

	    auth_data = mod_auth.getAccountInfo();

	    if (auth_data == null || auth_data.authorized != 'true') {
	        load_module("auth", "content", true);

	        return;
	    }

	    this.data.account = auth_data.account;

	    this.render();
    },

    //--------------------------------------------
    // Render HTML
    //--------------------------------------------
    render : function() {
	    var template, html, parser;

	    template = this.template;

	    parser = new ZParse(Implementation);
	    parser.parse(template.HTMLtemplate);
	    html = parser.process(this.data);

	    this.elem.innerHTML = html;

	    set_handlers(template);
	    set_tooltips(template, true);
    },

    //--------------------------------------------
    // Clear Form
    //--------------------------------------------
    clearForm : function() {
	    this.data.password = '';
	    this.data.digest = '';

	    this.render();
    },

    //==========================================================================
    //
    // Ajax functions
    //
    //==========================================================================

    //--------------------------------------------
    // Update
    //--------------------------------------------
    updateAccount : function(new_auth) {
	    var auth_data, put_data;
	    var url, params, wsse_header;
	    var json;

	    auth_data = mod_auth.getAccountInfo();
	    wsse_header = wsseHeader(auth_data.account, auth_data.digest);

	    url = "/service/admin/account";

	    put_data = { account  : new_auth.account,
		             password : new_auth.password };

	    json = Object.toJSON(put_data);
	    debug($jq.sprintf("updateAccount(): %s", json));

	    params = {
	        method: "PUT",
	        postBody: json,
	        requestHeaders: [
		        'If-Modified-Since', 'Wed, 15 Nov 1995 00:00:00 GMT',
		        'X-WSSE', wsse_header,
		        'Content-type', 'application/json'
	        ],
	        onFailure: function(httpObj) {
		        var status = httpObj.status;

		        switch (status) {
		        case 401:
		            alert("Authentication failed @ getInterfaces. (401)");
		            load_module('auth', 'content', true);
		            break;

		        default:
		            debug($jq.sprintf("updateAccount: Error %d.", status));
		            break;
		        }
	        },
	        onSuccess: function(httpObj) {
		        var auth_data;
		        var dialog_param = {
		            className : "alphacube",
		            width: 240,
		            showEffect: Element.show,
		            hideEffect: Element.hide
		        };

		        //auth_data = mod_auth.getAccountInfo();
		        //auth_data.account = new_auth.account;
		        //auth_data.digest = new_auth.password;
		        mod_auth.setAccountInfo(new_auth.account, new_auth.password);

		        Dialog.alert("アカウント、パスワード設定を<br/>更新しました。", dialog_param);
	        }.bind(this)
	    };

	    new Ajax.Request(url, params);

	    // dummy
	    //this.clearForm();
    },

    //==========================================================================
    //
    // Event handlers
    //
    //==========================================================================

    //--------------------------------------------
    // "Update" button handler
    //--------------------------------------------
    onUpdate : function() {
	    var account, pass1, pass2;
	    var pass_digest, errMessage = '';
	    var params;

	    account = strip_escapeHTML($('txt_account').value);
	    pass1 = $('txt_password1').value.strip();
	    pass2 = $('txt_password2').value.strip();
	    pass_digest = hex_sha1(pass1);

	    if (account == '')
	        errMessage = "Account を入力してください。";
	    else if (pass1 != pass2)
	        errMessage = "Password が一致しません。";
	    else if (pass1 == '' || pass2 == '')
	        errMessage = "Password を入力してください。";

	    if (errMessage != '') {
	        var dialog_param = {
		        className : "alphacube",
		        width: 240,
		        showEffect: Element.show,
		        hideEffect: Element.hide
	        };

	        Dialog.alert(errMessage, dialog_param);

	        return;
	    }

	    params = {
	        className : "alphacube",
	        width: 300,
	        showEffect: Element.show,
	        hideEffect: Element.hide,

	        onOk : function() {
		        var new_auth;

		        debug("Confirm: OK");

		        // Update AuthData
		        new_auth = {
		            account : account,
		            password : pass_digest
		        };

		        this.updateAccount(new_auth);

		        return true;
	        }.bindAsEventListener(this),
	        onCancel : function() {
		        debug("Confirm: Cancel");

		        return false;
	        }
	    };

	    Dialog.confirm("更新してもよろしいですか ?", params);
    }
};
