//==============================================================================
// MA-4xx/8xx/E2xx WebUI Module
//
// dnsmasq設定 (DHCP Server + DNS proxy)
// Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
// Copyright (C) 2007-2010 Century Systems Co., Ltd.
//
// Last Modified: 2010/05/28 15:04:49 kikuchi
//==============================================================================

mod_dnsmasq = {
    //--------------------------------------------
    // HTML template
    //--------------------------------------------
    template : {
	    HTMLtemplate : [
	        "<h1 class='title'>DHCPサーバ設定 (dnsmasq)</h1>",
	        "<h3>編集されるファイル&nbsp;<img src='img/ark_help.png' id='help_file' /></h3><br/>", 
	        "<table id='dnsmasq_table' class='zebra'>",
	        "<tr>",
	        "  <th colspan='2' class='header'>",
	        $jq.sprintf("現在のLAN側(%s) 設定", MA4xx_conf.net.lan),
	        "  </th>",
	        "</tr>",
	        "<tr>",
	        "  <th>DHCPクライアント機能</th>",
	        "  <td>",
	        "    <input type='checkbox' id='chk_dhcp_client' disabled='disabled' ",
	        "     <:if config.use_dhcp == true:>checked='checked'<:/if:> />",
	        "    利用する",
	        "  </td>",
	        "</tr>",
	        "<tr>",
	        "  <th>IPアドレス</th>",
	        "  <td>",
	        "     <:if config.use_dhcp==false:>${config.address}<:/if:>",
	        "     <:else:>DHCPサーバより取得<:/else:>",
	        "  </td>",
	        "</tr>",
	        "<tr>",
	        "  <th>ネットマスク</th>",
	        "  <td>",
	        "     <:if config.use_dhcp==false:>${config.netmask}<:/if:>",
	        "     <:else:>DHCPサーバより取得<:/else:>",
	        "  </td>",
	        "</tr>",
	        "<tr>",
	        "  <th>DHCPサーバ利用可否</th>",
	        "  <td>",
	        "     <:if config.use_dhcp==false:>利用可能です<:/if:>",
	        "     <:else:>LAN側がDHCP設定になっているため<br/>利用できません<:/else:>",
	        "  </td>",
	        "</tr>",

	        "<:if config.use_dhcp==false:>",
	        "<tr><br/></tr>",
	        "<tr>",
	        "  <th colspan='2' class='header'>",
	        "    DHCPサーバ設定",
	        "  </th>",
	        "</tr>",
	        "<tr>",
	        "  <th>DHCPサーバ機能</th>",
	        "  <td>",
	        "   <ul style='list-style-type: none;'>",
	        "    <li>",
	        "    <label for='chk_use_dnsmasq'>",
	        "    <input type='checkbox' id='chk_use_dnsmasq' ",
	        "     <:if settings.use_dhcp==true:>checked='checked'<:/if:>",
	        "     tabindex='1' />",
	        "    利用する",
	        "    </label>",
	        "    </li>",
	        "   </ul>",
	        "  </td>",
	        "</tr>",
	        "<tr>",
	        "  <th rowspan='2'>リース範囲設定</th>",
	        "  <td>",
	        "    <input type='text' id='lease_from' class='text' style='width:120px;' ",
	        "     <:if settings.use_dhcp==true:>",
	        "      value='${settings.lease_from}'",
	        "     <:/if:>",
	        "     <:else:>",
	        "      value='' disabled='disabled'",
	        "     <:/else:> tabindex='2'/> から",
	        "  </td>",
	        "</tr>",
	        "<tr>",
	        "  <td>",
	        "    <input type='text' id='lease_to' class='text' style='width:120px;' ",
	        "     <:if settings.use_dhcp==true:>",
	        "      value='${settings.lease_to}'",
	        "     <:/if:>",
	        "     <:else:>",
	        "      value='' disabled='disabled'",
	        "     <:/else:> tabindex='3' /> まで",
	        "  </td>",
	        "</tr>",
	        "<tr>",
	        "  <td colspan='2' class='noborderright'>",
	        "   <img id='indicator' src='img/indicator.gif' ",
	        "    style='visibility: hidden;' />",
	        "   <input type='button' class='button' id='btn_update'",
	        "    value='設定' tabindex='4' />",
	        "  </td>",
	        "</tr>",
	        "<:/if:>",

	        "</table>"
	    ].join("\n"),
	    handlers : [
	        [
		        // 'DHCP Client' checkbox
		        'chk_use_dnsmasq', 'click', function() {
		            var input_disable = true;

		            if ($('chk_use_dnsmasq').checked)
			            input_disable = false;

		            $('lease_from').disabled = input_disable;
		            $('lease_to').disabled = input_disable;
		        } ],
	        [
		        // '設定' button
		        'btn_update', 'click', function() {
		            debug("'更新' button clicked.");

		            mod_dnsmasq.updateSettings(true);
		        } ],
	        [
		        // '適用' button
		        'btn_reboot', 'click', function() {
		            debug("'適用' button clicked.");

		            mod_dnsmasq.rebootDHCPServer();
		        } ]
	    ],
	    //------------------------------
	    // Tooltip
	    //------------------------------
	    tooltips : [
	        [
		        // --- 編集されるファイル ---
		        'help_file', 'Note', [ "下記のファイルが編集対象です。<br/>",
				                       "/etc/dnsmasq.conf" ].join('\n')
	        ]
	    ]
    },

    //----------------------------------
    // dummy data
    //----------------------------------
    data : {
	    // LAN側設定状態
	    // /service/conf/net/ethernet/eth1 から取得
	    config : {
	        dhcp    : false,
	        ipaddr  : '192.168.254.254',
	        netmask : '255.255.255.0'
	    },

	    settings : {
	        use_dhcp  : true,
	        interface : 'eth0',
	        lease : [
		        '192.168.253.100',
		        '192.168.253.199'
	        ]
	    }
    },

    //--------------------------------------------
    // Initialize
    //--------------------------------------------
    init : function(elem) {
	    var auth_data;
	    this.elem = elem;

	    auth_data = mod_auth.getAccountInfo();

	    if (auth_data == null || auth_data.authorized != 'true') {
	        load_module("auth", "content", true);

	        return;
	    }

	    // validator
	    if (!this.validator)
	        this.validator = new validateUtil();

	    // Get LAN configurations (Ajax)
	    this.getLANconfig();
    },

    //--------------------------------------------
    // Render HTML
    //--------------------------------------------
    render : function() {
	    var template, html, parser;

	    template = this.template;

	    parser = new ZParse(Implementation);
	    parser.parse(template.HTMLtemplate);
	    html = parser.process(this.data);

	    this.elem.innerHTML = html;

	    set_handlers(template);
	    set_tooltips(template);
    },

    //--------------------------------------------
    // Disable DHCP Server
    //--------------------------------------------
    disableDHCPServer : function() {
	    var settings;

	    settings = this.data.settings;

	    settings.use_dhcp    = false;
	    settings.interface   = MA4xx_conf.net.lan;
	    settings.lease_from  = '';
	    settings.lease_to    = '';

	    this.updateSettings(false);
    },

    //--------------------------------------------
    // update data from Form
    //--------------------------------------------
    updateFromForm : function() {
	    var settings;
	    debug("updateFromForm()");

	    settings = this.data.settings;

	    settings.interface = MA4xx_conf.net.lan;
	    // DHCP Server 利用する ?
	    if ($('chk_use_dnsmasq').checked) {
	        settings.use_dhcp   = true;
	        settings.lease_from = strip_escapeHTML($('lease_from').value);
	        settings.lease_to   = strip_escapeHTML($('lease_to').value);
	    } else {
	        settings.use_dhcp   = false;
	        settings.lease_from = '';
	        settings.lease_to   = '';
	    }
    },

    //--------------------------------------------
    // update Form from data
    //--------------------------------------------
    updateForm : function() {
	    var settings;

	    settings = this.data.settings;

	    if ($('btn_reboot') != null) {
	        // DHCP Server 利用する ?
	        if (settings.use_dhcp == true) {
		        $('btn_reboot').disabled = false;
	        } else {
		        $('btn_reboot').disabled = true;
	        }
	    }
    },

    //==========================================================================
    //
    // Ajax functions
    //
    //==========================================================================

    //--------------------------------------------
    // Get LAN configuration
    //--------------------------------------------
    getLANconfig : function() {
	    var auth_data;
	    var url, params, wsse_header;

	    auth_data = mod_auth.getAccountInfo();

	    wsse_header = wsseHeader(auth_data.account, auth_data.digest);

	    url = $jq.sprintf("/service/conf/net/ethernet/%s", MA4xx_conf.net.lan);
	    debug($jq.sprintf("getLANconfig: URL -> '%s'", url));

	    params = {
	        method: "GET",
	        requestHeaders: [
		        'If-Modified-Since', 'Wed, 15 Nov 1995 00:00:00 GMT',
		        'X-WSSE', wsse_header
	        ],
	        onFailure: function(httpObj) {
		        var status = httpObj.status;

		        switch (status) {
		        case 401:
		            alert("Authentication failed @ getLANconfig (401)");
		            load_module('auth', 'content', true);
		            break;

		        default:
		            debug($jq.sprintf("Error %d.", status));
		            break;
		        }
	        },
	        onSuccess: function(httpObj) {
		        var res = eval("(" + httpObj.responseText + ")");
		        this.data.config = res.config;
		        this.data.config = res;

		        // get DHCP configuration
		        mod_dnsmasq.getDHCPconfig();
	        }.bind(this)
	    };

	    new Ajax.Request(url, params);
    },

    //--------------------------------------------
    // Get DHCP Server configuration
    //--------------------------------------------
    getDHCPconfig : function() {
	    var auth_data;
	    var url, params, wsse_header;

	    auth_data = mod_auth.getAccountInfo();

	    wsse_header = wsseHeader(auth_data.account, auth_data.digest);

	    url = "/service/conf/net/dhcp";
	    debug($jq.sprintf("getDHCPconfig: URL -> '%s'", url));

	    params = {
	        method: "GET",
	        requestHeaders: [
		        'If-Modified-Since', 'Wed, 15 Nov 1995 00:00:00 GMT',
		        'X-WSSE', wsse_header
	        ],
	        onFailure: function(httpObj) {
		        var status = httpObj.status;

		        switch (status) {
		        case 401:
		            alert("Authentication failed @ getDHCPconfig (401)");
		            load_module('auth', 'content', true);
		            break;

		        default:
		            debug($jq.sprintf("Error %d.", status));
		            break;
		        }
	        },
	        onSuccess: function(httpObj) {
		        var res = eval("(" + httpObj.responseText + ")");
		        debug($jq.sprintf("getDHCPconfig : %s", httpObj.responseText));

		        this.data.settings = res.settings;

		        // get DHCP configuration
		        this.render();
	        }.bind(this)
	    };

	    new Ajax.Request(url, params);
    },

    //--------------------------------------------
    // Update
    //--------------------------------------------
    updateSettings : function(fromForm) {
	    var auth_data, put_data;
	    var url, params, wsse_header;
	    var json;

	    if (fromForm == true)
	        this.updateFromForm();

	    if (this.validateForm() == false)
	        return;

	    auth_data = mod_auth.getAccountInfo();
	    wsse_header = wsseHeader(auth_data.account, auth_data.digest);

	    url = "/service/conf/net/dhcp";

	    json = Object.toJSON(this.data.settings);
	    debug($jq.sprintf("updateSettings: %s", json));

	    params = {
	        method: "PUT",
	        postBody: json,
	        requestHeaders: [
		        'If-Modified-Since', 'Wed, 15 Nov 1995 00:00:00 GMT',
		        'X-WSSE', wsse_header,
		        'Content-type', 'application/json'
	        ],
	        onFailure: function(httpObj) {
		        var status = httpObj.status;

		        this.hide_indicator();

		        switch (status) {
		        case 401:
		            alert("Authentication failed @ getInterfaces. (401)");
		            load_module('auth', 'content', true);
		            break;

		        default:
		            debug($jq.sprintf("Error %d.", status));
		            break;
		        }
	        }.bind(this),
	        onSuccess: function(httpObj) {
		        var dialog_param = {
		            className  : "alphacube",
		            width      : 240,
		            showEffect : Element.show,
		            hideEffect : Element.hide
		        };

		        this.hide_indicator();

		        if (fromForm == true)
		            Dialog.alert("DHCPサーバ設定を<br/>更新しました。", dialog_param);

		        // '適用' ボタン状態更新
		        mod_dnsmasq.updateForm();
	        }.bind(this)
	    };

	    this.show_indicator();
	    new Ajax.Request(url, params);
    },

    //--------------------------------------------
    // Reboot
    //--------------------------------------------
    rebootDHCPServer : function() {
	    var auth_data, put_data;
	    var url, params, wsse_header;
	    var json;

	    auth_data = mod_auth.getAccountInfo();
	    wsse_header = wsseHeader(auth_data.account, auth_data.digest);

	    url = "/service/conf/net/dhcp";

	    params = {
	        method: "POST",
	        postBody: "command=restart",
	        requestHeaders: [
		        'If-Modified-Since', 'Wed, 15 Nov 1995 00:00:00 GMT',
		        'X-WSSE', wsse_header
	        ],
	        onFailure: function(httpObj) {
		        var status = httpObj.status;

		        switch (status) {
		        case 401:
		            alert("Authentication failed @ rebootDHCPServer. (401)");
		            load_module('auth', 'content', true);
		            break;

		        default:
		            debug($jq.sprintf("Error %d.", status));
		            break;
		        }
	        },
	        onSuccess: function(httpObj) {
		        var dialog_param = {
		            className : "alphacube",
		            width: 240,
		            showEffect: Element.show,
		            hideEffect: Element.hide
		        };

		        Dialog.alert("DHCPサーバを<br/>再起動しました。", dialog_param);
	        }.bind(this)
	    };

	    new Ajax.Request(url, params);
    },

    //================================================================
    //
    // Utility functions
    //
    //================================================================

    // AJAX 処理開始
    show_indicator : function() {
	    $('indicator').style.visibility = 'visible';
    },

    // AJAX 処理完了
    hide_indicator : function() {
	    $('indicator').style.visibility = 'hidden';
    },

    //--------------------------------------------
    // Form validator
    //--------------------------------------------
    validateForm : function() {
	    var config, settings;
	    var errMsgArray = [], errMsg;

	    // shortcuts
	    config = this.data.config;
	    settings = this.data.settings;

	    // DHCPサーバ機能を使用する場合のみ
	    // リース範囲のアドレスを確認
	    if (settings.use_dhcp == true) {
	        if (this.validator.checkIPAddress(settings.lease_from) == false)
		        errMsgArray.push("・リース範囲 先頭アドレスが不正です。");
	        else if (this.validator.checkReachability(config.address, config.netmask,
						                              settings.lease_from) == false)
		        errMsgArray.push("・リース範囲 先頭アドレスが\n" +
				                 "　範囲外のアドレスにされています。");

	        if (this.validator.checkIPAddress(settings.lease_to) == false)
		        errMsgArray.push("・リース範囲 終了アドレスが不正です。");
	        else if (this.validator.checkReachability(config.address, config.netmask,
						                              settings.lease_to) == false)
		        errMsgArray.push("・リース範囲 終了アドレスが\n" +
				                 "　範囲外のアドレスに設定されています。");
	    } else {
	        // insure
	        settings.lease_from = '';
	        settings.lease_to   = '';
	    }

	    if (errMsgArray.length > 0) {
	        errMsg = errMsgArray.join("\n");

	        alert(errMsg);
	        return false;
	    }

	    return true;
    }
};
