//==============================================================================
// MA-4xx/8xx/E2xx WebUI Module
//
// NTP設定
// Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
// Copyright (C) 2007-2010 Century Systems Co., Ltd.
//
// Last Modified: 2010/05/28 18:16:03 kikuchi
//==============================================================================

mod_ntp = {
    //--------------------------------------------
    // HTML template
    //--------------------------------------------
    template : {
	    HTMLtemplate : [
	        "<h1 class='title'>NTP設定</h1>",
	        "<h3>編集されるファイル&nbsp;<img src='img/ark_help.png' id='help_file' /></h3><br/>",
	        "<form id='ntp_form'>",
	        "<table id='ntp_table' class='zebra'>",
	        "<tr>",
	        "  <th>全体設定</th>",
	        "  <td>",
	        "   <ul style='list-style-type: none;'>",
	        "    <li>",
	        "    <label for='use_ntp'>",
	        "    <input id='use_ntp' type=checkbox ",
	        "     <:if use_ntp==true:>checked='checked'<:/if:>/>",
	        "     NTPを使用する",
	        "    </label>",
	        "    </li>",
	        "   </ul>",
	        "  </td>",
	        "</tr>",
	        "<tr class='when_enable'>",
	        "  <th>サーバ動作</th>",
	        "  <td>",
	        "   <ul style='list-style-type: none;'>",
	        "    <li>",
	        "    <label for='act_server'>",
	        "    <input id='act_server' type=checkbox ",
	        "     <:if act_server==true:>checked='checked'<:/if:>/>",
	        "     NTPサーバとして動作する",
	        "    </label>",
	        "    </li>",
	        "   </ul>",
	        "  </td>",
	        "</tr>",
	        "<tr class='when_enable'>",
	        "  <th colspan='2' class='header'>NTPサーバ設定</th>",
	        "</tr>",
	        "<tr class='when_enable'>",
	        "  <th>NTPサーバ(1)</th>",
	        "  <td class='noborder'>",
	        "    <input type='text' id='ntp_server0' class='text' style='width:100%;'",
	        "     value='${ntp_server_primary}' />",
	        "  </td>",
	        "</tr>",
	        "<tr class='when_enable'>",
	        "  <th>NTPサーバ(2)</th>",
	        "  <td class='noborder'>",
	        "    <input type='text' id='ntp_server1' class='text' style='width:100%;'",
	        "     value='${ntp_server_secondary}' />",
	        "  </td>",
	        "</tr>",
	        "<tr>",
	        "  <td colspan='2' class='noborderright'>",
	        "    <img id='indicator' src='img/indicator.gif' ",
	        "     style='visibility: hidden;' />",
	        "    <input type='button' id='bt_update' value='設定' class='button' />",
	        "  </td>",
	        "</tr>",
	        "</table>",
	        "</form>"
	    ].join(""),
	    //------------------------------
	    // Event Handlers
	    //------------------------------
	    handlers : [
	        [
		        // NTP 使用する/しない 切替
		        'use_ntp', 'click', function() {
		            var tr_list, tr, i;

		            tr_list = $$('tr.when_enable');

		            for (i = 0; i < tr_list.length; i++) {
			            tr = tr_list[i];

			            if ($('use_ntp').checked)
			                new Effect.Appear(tr);
			            else
			                new Effect.Fade(tr);
		            }
		        }],
	        [
		        // NTP設定更新
		        'bt_update', 'click', function() {
		            mod_ntp.updateSettings();
		        }]
	    ],
	    //------------------------------
	    // Tooltips
	    //------------------------------
	    tooltips : [
	        [
		        'bt_update', 'Note', "設定を装置へ反映します。"
	        ],
	        [
		        // --- 編集されるファイル ---
		        'help_file', 'Note', [ "下記のファイルが編集対象です。<br/>",
				                       "/etc/ntp.conf" ].join('\n')
	        ]
	    ]
    },

    //----------------------------------
    // dummy data
    //----------------------------------
    data : {
	    use_ntp              : true,
	    act_server           : true,
	    ntp_server_primary   : "ntp.nict.jp",
	    ntp_server_secondary : "pool.ntp.org"
    },

    //--------------------------------------------
    // Initialize
    //--------------------------------------------
    init : function(elem) {
	    var auth_data;
	    this.elem = elem;

	    auth_data = mod_auth.getAccountInfo();

	    if (auth_data == null || auth_data.authorized != 'true') {
	        load_module("auth", "content", true);

	        return;
	    }

	    // Validator
	    if (!this.validator)
	        this.validator = new validateUtil();

	    // Get NTP configurations (Ajax)
	    this.getNTPconfig();
    },

    //--------------------------------------------
    // Render HTML
    //--------------------------------------------
    render : function() {
	    var template, html, parser;

	    template = this.template;

	    parser = new ZParse(Implementation);
	    parser.parse(template.HTMLtemplate);
	    html = parser.process(this.data);

	    this.elem.innerHTML = html;

	    this.updateDisplay(this.data.use_ntp);

	    set_handlers(template);
	    set_tooltips(template, true);
    },

    //--------------------------------------------
    // NTP Use True/False
    //--------------------------------------------
    updateDisplay : function(use_ntp) {
        if (use_ntp)
	        $jq(".when_enable").fadeIn("fast");
        else
	        $jq(".when_enable").fadeOut("fast");
    },

    //--------------------------------------------
    // update data from Form
    //--------------------------------------------
    updateFromForm : function(use_ntp) {
	    var elements, elm, i;

	    this.data.use_ntp = $('use_ntp').checked;

	    if (this.data.use_ntp) {
	        this.data.act_server           = $('act_server').checked;
	        this.data.ntp_server_primary   = strip_escapeHTML($('ntp_server0').value);
	        this.data.ntp_server_secondary = strip_escapeHTML($('ntp_server1').value);
	    } else {
	        this.data.act_server           =  false;
	        this.data.ntp_server_primary   = 'ntp.nict.jp';
	        this.data.ntp_server_secondary = '';

	        $('ntp_server0').value = 'ntp.nict.jp';
	        $('ntp_server1').value = '';
	    }
    },

    //================================================================
    //
    // Utility functions
    //
    //================================================================

    //
    show_indicator : function() {
	    $('indicator').style.visibility = 'visible';
    },

    //
    hide_indicator : function() {
	    $('indicator').style.visibility = 'hidden';
    },

    //==========================================================================
    //
    // Ajax functions
    //
    //==========================================================================

    //--------------------------------------------
    // Get NTP configuration
    //--------------------------------------------
    getNTPconfig : function() {
	    var auth_data;
	    var url, params, wsse_header;

	    auth_data = mod_auth.getAccountInfo();

	    wsse_header = wsseHeader(auth_data.account, auth_data.digest);

	    url = "/service/daemon/ntp";

	    params = {
	        method: "GET",
	        requestHeaders: [
		        'If-Modified-Since', 'Wed, 15 Nov 1995 00:00:00 GMT',
		        'X-WSSE', wsse_header
	        ],
	        onFailure: function(httpObj) {
		        var staus = httpObj.status;

		        switch (status) {
		        case 401:
		            alert("Authentication failed @ getInterfaces. (401)");
		            load_module('auth', 'content', true);
		            break;

		        default:
		            alert($jq.sprintf("Error %d", status));
		            break;
		        }
	        },
	        onSuccess: function(httpObj) {
		        var res = eval("(" + httpObj.responseText + ")");

		        debug($jq.sprintf("getNTP: '%s'", httpObj.responseText));
		        this.data = res['ntp_info'];

		        this.render();
	        }.bind(this)
	    };

	    new Ajax.Request(url, params);
    },

    //--------------------------------------------
    // Update
    //--------------------------------------------
    updateSettings : function() {
	    var auth_data, put_data;
	    var url, params, wsse_header;
	    var json;

	    this.updateFromForm();

	    // Form validation
	    if (this.validateForm() == false)
	        return;

	    auth_data = mod_auth.getAccountInfo();
	    wsse_header = wsseHeader(auth_data.account, auth_data.digest);

	    url = "/service/daemon/ntp";

	    json = Object.toJSON(this.data);
	    debug($jq.sprintf("setNTPserver(): %s", json));

	    params = {
	        method: "PUT",
	        postBody: json,
	        requestHeaders: [
		        'If-Modified-Since', 'Wed, 15 Nov 1995 00:00:00 GMT',
		        'X-WSSE', wsse_header,
		        'Content-type', 'application/json'
	        ],
	        onFailure: function(httpObj) {
		        var status = httpObj.status;

		        this.hide_indicator();

		        switch (status) {
		        case 401:
		            alert("Authentication failed @ updateSettings. (401)");
		            load_module('auth', 'content', true);
		            break;

		        default:
		            alert($jq.sprintf("Error %d.", status));
		            break;
		        }
	        }.bind(this),
	        onSuccess: function(httpObj) {
		        var dialog_param = {
		            className  : "alphacube",
		            width      : 240,
		            showEffect : Element.show,
		            hideEffect : Element.hide
		        };

		        this.hide_indicator();
		        Dialog.alert("NTP設定を<br/>更新しました。", dialog_param);
	        }.bind(this)
	    };

	    this.show_indicator();
	    new Ajax.Request(url, params);
    },

    //--------------------------------------------
    // Reboot
    //--------------------------------------------
    rebootNTPServer : function() {
	    alert("reboot button clicked.");
    },

    //--------------------------------------------
    // Form validation
    //--------------------------------------------
    validateForm : function() {
	    var useNTP, server, i, servers = [];
	    var errMsgArray = [], errMsg;

	    useNTP = $('use_ntp').checked;

	    if (useNTP == false)
	        return true;

	    server = strip_escapeHTML($('ntp_server0').value);
	    if (server != "")
	        servers.push(server);

	    server = strip_escapeHTML($('ntp_server1').value);
	    if (server != "")
	        servers.push(server);

	    if (servers.length == 0)
	        errMsgArray.push("・NTPサーバが指定されておりません。");
	    else {
	        for (i = 0; i < servers.length; i++) {
		        if (this.validator.checkFQDN(servers[i]) == false &&
		            this.validator.checkIPAddress(servers[i]) == false) {

		            errMsgArray.push($jq.sprintf("・NTPサーバのアドレスが不正です : \" %s \"", servers[i]));
		        }
	        }
	    }

	    if (errMsgArray.length > 0) {
	        errMsg = errMsgArray.join('\n');
	        alert(errMsg);

	        return false;
	    }

	    return true;
    }
};
