//==============================================================================
// MA-4xx/8xx/E2xx WebUI Module
//
// 装置再起動
// Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
// Copyright (C) 2007-2010 Century Systems Co., Ltd.
//
// Last Modified: 2010/05/28 15:06:35 kikuchi
//==============================================================================

mod_reboot = {
	//--------------------------------------------
	// HTML template
	//--------------------------------------------
	template : {
		HTMLtemplate : [
			"<h1 class='title'>装置再起動/シャットダウン</h1>",
			" <h2>再起動設定</h2>",
			"  <form id='form_reboot_setting'>",
			"   <table id='reboot_table' class='zebra'>",
			"   <tr>",
			"    <th class='header'>rootfs mount mode</th>",
			"    <td>",
			"     <select name='sel_rootfs_mode' id='sel_rootfs_mode'",
			"      class='select' style='width:100px;'/>",
			"      <option value='RW'",
			"       <:if rootfs_mode=='RW':>selected='selected'<:/if:> >",
			"        読書可能</option>",
			"       <option value='RO'",
			"        <:if rootfs_mode=='RO':>selected='selected'<:/if:> >",
			"        読込専用</option>",
			"     </select>",
			"    </td>",
			"    <td>",
			"     ルートファイルシステム(FLASHメモリ)への<br/>",
            "     書き込みモードを設定します。",
			"    </td>",
			"   </tr>",

			"   <tr>",
			"    <td colspan='3' class='noborderright'>",
			"     <img id='indicator' src='img/indicator.gif'",
			"      style='visibility: hidden;'/>",
			"     <input type='button' id='btn_submit' class='button'",
			"      style='width:100px' value='設定'/>",
			"    </td>",
			"   </tr>",
			"   </table>",
			"  </form>",

			" <h2>再起動/シャットダウンの実行</h2>",
			"  <table class='zebra'>",
			"  <tr>",
			"   <th>装置再起動</th>",
			"   <td>",
			"    <input type='button' id='btn_reboot' class='button'",
			"     value='再起動実行' style='width:120px;' />",
			"   </td>",
			"  </tr>",
			"  <tr>",
			"   <th>シャットダウン</th>",
			"   <td>",
			"    <input type='button' id='btn_shutdown' class='button'",
			"     value='シャットダウン実行' style='width:120px;' />",
			"   </td>",
			"  </tr>",
			"  </table>"
		].join("\n"),
		//------------------------------
		// Event Handlers
		//------------------------------
		handlers : [
			[
				'btn_submit', 'click', function() {
					mod_reboot.onSubmit();
				}],
			[
				// 'シャットダウン' button
				'btn_shutdown', 'click', function() {
					//debug("shutdown button clicked.");

					if (confirm("シャットダウンしてもよろしいですか ?"))
						mod_reboot.reboot('shutdown');
				}],
			[
				// '再起動' button
				'btn_reboot', 'click', function() {
					//debug("reboot button clicked.");

					if (confirm("再起動してもよろしいですか ?"))
						mod_reboot.reboot('reboot');
				}]
		]
	},

	//----------------------------------
	// dummy data
	//----------------------------------
	data : {
		'rootfs_mode': 'RW'
	},

	//--------------------------------------------
	// Initialize
	//--------------------------------------------
	init : function(elem) {
		var auth_data;
		this.elem = elem;

        auth_data = mod_auth.getAccountInfo();

		if (auth_data == null || auth_data.authorized != 'true') {
			load_module("auth", "content", true);

			return;
		}

		this.get_settings();
	},

	//--------------------------------------------
	// Render HTML
	//--------------------------------------------
	render : function() {
		var template, html, parser;

		template = this.template;

		parser = new ZParse(Implementation);
		parser.parse(template.HTMLtemplate);
		html = parser.process(this.data);

		this.elem.innerHTML = html;

		set_handlers(template);
	},

	onSubmit : function() {
		this.data.rootfs_mode = $('sel_rootfs_mode').value;

		this.put_settings();
	},

	show_indicator : function(elem) {
		$(elem).style.visibility = 'visible';
	},

	hide_indicator : function(elem) {
		$(elem).style.visibility = 'hidden';
	},

	//==========================================================================
	//
	// Ajax functions
	//
	//==========================================================================

	//--------------------------------------------
	// Reboot/Shutdown request
	//--------------------------------------------
	get_settings : function() {
		var auth_data;
		var url, params, wsse_header;

        auth_data = mod_auth.getAccountInfo();
		wsse_header = wsseHeader(auth_data.account, auth_data.digest);

		url = "/service/admin/reboot";

		params = {
			method: "GET",
			requestHeaders: [
				'If-Modified-Since', 'Wed, 15 Nov 1995 00:00:00 GMT',
				'X-WSSE', wsse_header
			],
			onFailure: function(httpObj) {
				var status = httpObj.status;

				switch (status) {
				case 401:
					alert("Authentication failed @ get_settings. (401)");
					load_module('auth', 'content', true);
					break;

				default:
					alert($jq.sprintf("Error %d", status));
					break;
				}
			}.bind(this),
			onSuccess: function(httpObj) {
				var res = eval("(" + httpObj.responseText + ")");
				debug($jq.sprintf("reboot() --> %s", httpObj.responseText));
				this.data = res;
				this.render();
			}.bind(this)
		};

		new Ajax.Request(url, params);
	},

	put_settings : function() {
		var auth_data;
		var url, params, wsse_header, json;

        auth_data = mod_auth.getAccountInfo();
		wsse_header = wsseHeader(auth_data.account, auth_data.digest);

		url = "/service/admin/reboot";
		json = Object.toJSON(this.data);
		debug($jq.sprintf("json: '%s'", json));

		params = {
			method: 'PUT',
			contentType: 'application/json',
			postBody: json,
			requestHeaders: [
				'If-Modified-Since', 'Wed, 15 Nov 1995 00:00:00 GMT',
				'X-WSSE', wsse_header,
				'Content-type', 'application/json'
			],
			onFailure: function(httpObj) {
				var status = httpObj.status;
				this.hide_indicator('indicator');

				switch (status) {
				case 401:
					alert("Authentication failed @put_settings (401)");
					load_module('auth', 'content', true);
					break;

				default:
					alert($jq.sprintf("Error %d", status));
					break;
				}
			}.bind(this),
			onSuccess: function(httpObj) {
				var result;

				this.hide_indicator('indicator');
				debug($jq.sprintf("put_settings: %s", httpObj.responseText));

				if (confirm("設定を更新しました。設定を反映させるには再起動が必要です。再起動しますか?"))
					mod_reboot.reboot('reboot');

				// reload
				this.get_settings();
			}.bind(this)
		};

		this.show_indicator('indicator');
		new Ajax.Request(url, params);
	},

	//--------------------------------------------
	// Reboot/Shutdown request
	//--------------------------------------------
	reboot : function(mode) {
		var auth_data;
		var url, params, rootfs_mode, postBody, wsse_header;

        auth_data = mod_auth.getAccountInfo();
		wsse_header = wsseHeader(auth_data.account, auth_data.digest);

		url = "/service/admin/reboot";

		postBody = $jq.sprintf("mode=%s", mode, rootfs_mode);
		debug($jq.sprintf("reboot: postBody = '%s'", postBody));

		params = {
			method: "POST",
			postBody: postBody,
			requestHeaders: [
				'If-Modified-Since', 'Wed, 15 Nov 1995 00:00:00 GMT',
				'X-WSSE', wsse_header
			],
			onFailure: function(httpObj) {
				var status = httpObj.status;

				switch (status) {
				case 401:
					alert("Authentication failed @ reboot. (401)");
					load_module('auth', 'content', true);
					break;

				default:
					//alert($jq.sprintf("Error %d", status));
					break;
				}
			},
			onSuccess: function(httpObj) {
				var res = eval("(" + httpObj.responseText + ")");

				debug($jq.sprintf("reboot() --> %s", httpObj.responseText));

				alert(res.msg);
				load_module('auth', 'content', true);
			}
		};

		new Ajax.Request(url, params);
	}
};
