//==============================================================================
// MA-4xx/8xx/E2xx WebUI
//
// Utility Functions
// Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
// Copyright (C) 2007-2010 Century Systems Co.,Ltd.
//
// Last Modified: 2010/05/28 15:07:31 kikuchi
//==============================================================================

//----------------------------------------------------------
// get HTTP Server IP address
//----------------------------------------------------------
get_hostaddr = function() {
    return document.location.host;
};

//----------------------------------------------------------
// set HTML template event handlers
//----------------------------------------------------------
set_handlers = function(template) {
    var i, elm, event, func, handler;

    for (i = 0; i < template.handlers.length; i++) {
	    handler = template.handlers[i];
	    elm = handler[0];
	    event = handler[1];
	    func = handler[2];

	    if ($(elm) != null)
	        Element.observe($(elm), event, func);
    }
};

//----------------------------------------------------------
// set HTML template Tooltips (GLT)
//----------------------------------------------------------
set_tooltips_GLT = function(template) {
    var i, elem, title, content, options, tooltip;
    var old_elems, parent;

    for (i = 0; i < template.tooltips.length; i++) {
	    tooltip = template.tooltips[i];

	    elem = tooltip[0];
	    title = tooltip[1];
	    content = tooltip[2];

	    options = {
	        title: title,
	        className: 'tip'
	    };

	    if ($(elm) != null)
	        $(elem).title = content;
    }

    GLT.init();
};

//----------------------------------------------------------
// set HTML template Tooltips (Effect)
//----------------------------------------------------------
set_tooltips = function(template, clear) {
    var i, elem, title, content, options, tooltip;
    var old_elems, parent;

    if (clear == true) {
	    old_elems = $$('div.tip');

	    for (i = 0; i < old_elems.length; i++) {
	        elem = old_elems[i];
	        parent = elem.parentNode;

	        parent.removeChild(elem);
	    }

	    //		if (old_elems.length > 0)
	    //			debug($jq.sprintf("remove %d old tooltip(s).", old_elems.length));
    }

    for (i = 0; i < template.tooltips.length; i++) {
	    tooltip = template.tooltips[i];

	    elem    = tooltip[0];
	    title   = tooltip[1];
	    content = tooltip[2];

	    options = {
	        title:     title,
	        className: 'tip',
	        effect:    'appear'
	    };

	    if ($(elem) != null)
	        //new Effect.Tooltip(elem, content, options);
	        new Tip(elem, content, options);

	    // 		Object.extend($(elem));
	    // 		$(elem).tooltip_content = content;

	    // 		Element.observe($(elem), "mouseover", function(event) {
	    // 							showToolTip(event, this.tooltip_content);
	    // 						}.bindAsEventListener($(elem)));

	    // 		Element.observe($(elem), "mouseout", function(event) {
	    // 							hideToolTip();
	    // 						});
    }
};

ffs8 = function(val) {
    var i, mask;

    if (val < 1 || val > 255)
	    return -1;

    mask = 128;
    for (i = 7; i >= 0; i--) {
	    if (val & mask)
	        break;

	    mask >>= 1;
    }

    return i;
};

fls8 = function(val) {
    var i, mask;

    if (val < 1 || val > 255)
	    return -1;

    mask = 1;
    for (i = 0; i < 8; i++) {
	    if (val & mask)
	        break;

	    mask <<= 1;
    }

    return i;
};

//==========================================================
//
// Validator
//
//==========================================================
var validateUtil = Class.create();

validateUtil.prototype = {
    initialize : function() {
	    this.re_IPAddress = new RegExp("");
	    //		this.re_IPAddress.compile('([0-9]+)\\.([0-9]+)\\.([0-9]+)\\.([0-9]+)');
	    this.re_IPAddress.compile('^([0-9]+)\\.([0-9]+)\\.([0-9]+)\\.([0-9]+)$');

	    this.re_CIDR = new RegExp("");
	    //		this.re_CIDR.compile('([0-9]+)\\.([0-9]+)\\.([0-9]+)\\.([0-9]+)/([12]?[0-9])');
	    this.re_CIDR.compile('^([0-9]+)\\.([0-9]+)\\.([0-9]+)\\.([0-9]+)/([12]?[0-9])$');

	    this.re_FQDN = new RegExp("");
	    this.re_FQDN.compile('^([a-zA-Z0-9][a-zA-Z0-9\\-\\.]+)\\.([a-z]{2,4}$)');

	    this.re_OCN = new RegExp("");
	    this.re_OCN.compile('^([a-zA-Z0-9]+)@?([a-zA-Z0-9\\-\\.]+)$');

	    this.re_Email = new RegExp("");
	    this.re_Email.compile(/^([a-zA-Z0-9_\.\-])+\@(([a-zA-Z0-9\-])+\.)+([a-zA-Z0-9]{2,4})+$/);
    },

    //--------------------------------------------
    // Check Range (Integer ver.)
    //--------------------------------------------
    checkRangeInt : function(val, min, max) {
	    if (isNaN(val))
	        return false;

	    if (min > max)
	        return false;

	    if (min > val || max < val)
	        return false;

	    return true;
    },

    //--------------------------------------------
    // Check Range (String ver.)
    //--------------------------------------------
    checkRange : function(str, min, max) {
	    var val;

	    val = parseInt(str);

	    return this.checkRangeInt(val, min, max);
    },

    //--------------------------------------------
    // IP -> long
    //--------------------------------------------
    IP2long : function(ip) {
	    var ips = ip.split('.');
	    var iplong = 0, i;

	    for (i = 0; i < 4; i++)
	        iplong += parseInt(ips[i]) * Math.pow(256, 3 - i);

	    return iplong;
    },

    //--------------------------------------------
    // long -> IP
    //--------------------------------------------
    long2IP : function(l) {
	    var ip = [], i, ip, base;

	    for (i = 0; i < 4; i++) {
	        base = Math.pow(256, 3 - i);

	        ip[i] = Math.floor(l / base);
	        l -= ip[i] * base;
	    }

	    ip = ip.join('.');

	    return ip;
    },

    //--------------------------------------------
    // CIDR -> long
    //--------------------------------------------
    CIDR2long : function(num) {
	    var cidr_array = [], cidr_str;
	    var i, l;

	    for (i = 0; i < 32; i++) {
	        if (i < num)
		        cidr_array.push('1');
	        else
		        cidr_array.push('0');
	    }

	    cidr_mask = cidr_array.join('');
	    l = parseInt(cidr_mask, 2);

	    return l;
    },

    //--------------------------------------------
    // CIDR -> long (bit reverse)
    //--------------------------------------------
    CIDR2long_rev : function(num) {
	    var cidr_array = [], cidr_str;
	    var i, l;

	    for (i = 0; i < 32; i++) {
	        if (i < num)
		        cidr_array.push('0');
	        else
		        cidr_array.push('1');
	    }

	    cidr_mask = cidr_array.join('');
	    l = parseInt(cidr_mask, 2);

	    return l;
    },

    //--------------------------------------------
    // Check IP Address
    //--------------------------------------------
    checkIPAddress : function(str) {
	    var addr, octet, i, min, max;
	    var l, ip;

	    if (str == '')
	        return false;

	    if (this.re_IPAddress.exec(str) == null) {
	        debug("checkIPAddress: regexp not match.");
	        return false;
	    }

	    addr = str.split('.');
	    
	    for (i = 0; i < 4; i++) {
	        switch (i) {
	        case 0:
		        min = 1, max = 223;
		        break;

	        case 3:
		        min = 1, max = 254;
		        break;
		        
	        default:
		        min = 0, max = 255;
		        break;
	        }

	        if (this.checkRange(addr[i], min, max) == false) {
		        debug($jq.sprintf("checkIPAddress: octet %d range over (%s).",
			                  i + 1, addr[i]));
		        return false;
	        }
	    }

	    // local
	    if (addr[0] == '127') {
	        debug("checkIPAddress: local address.");
	        return false;
	    }

	    // APIPA
	    if (addr[0] == '169' && addr[1] == '254') {
	        debug("checkIPAddress: APIPA.");
	        return false;
	    }

	    return true;
    },

    //--------------------------------------------
    // Check Netmask
    //--------------------------------------------
    checkNetmask : function(str) {
	    var addr, octet, notmask, i;
	    var set = [];

	    if (str == '')
	        return false;

	    notmask = -1;

	    if (this.re_IPAddress.exec(str) != null) {
	        addr = str.split('.');
	        
	        for (i = 0; i < 4; i++) {
		        octet = parseInt(addr[i]);

		        if (this.checkRange(addr[i], 0, 255) == false)
		            return false;

		        if (octet != 255) {
		            if (notmask != -1 && octet != 0) {
			            debug("not 255");
			            return false;
		            } else {
			            if (octet != 0) {
			                set[0] = fls8(octet);		// '1' 一番右
			                set[1] = ffs8(255 - octet);	// '0' 一番左

			                if (set[0] != set[1] + 1) {
				                debug($jq.sprintf("octet: %d, (%d, %d)",
					                          i, set[0], set[1]));
				                return false;
			                }
			            }

			            notmask = i;
		            }
		        }
	        }

	        return true;
	    }

	    return false;
    },

    //--------------------------------------------
    // Check CIDR
    //--------------------------------------------
    checkCIDR : function(str) {
	    var addr, octets;
	    var addr_long, mask_long, masked_long;

	    result = false;

	    if (str == '')
	        return false;

	    if (this.re_CIDR.exec(str) == null)
	        return false;

	    addr = str.split('/');
	    octets = addr[0].split('.');

	    // check IP address
	    for (i = 0; i < 4; i++) {
	        if (this.checkRange(octets[i], 0, 255) == false) {
		        debug($jq.sprintf("checkCIDR: %d octets range over (%s)",
			                  i + 1, octets[i]));
		        return false;
	        }
	    }

	    addr_long = this.IP2long(addr[0]);
	    debug($jq.sprintf("addr[0](IP)       %s / long = %d", addr[0], addr_long));
	    mask_long = this.CIDR2long_rev(addr[1]);
	    debug($jq.sprintf("addr[1](CIDR_rev) %s / long = %d", addr[1], mask_long));
	    masked_long = addr_long & mask_long;
	    debug($jq.sprintf("masked_long = %d", masked_long));

	    if (masked_long != 0) {
	        debug("checkCIDR(): Network error.");
	        return false;
	    }

	    return true;
    },

    //--------------------------------------------
    // Check Network Address
    //--------------------------------------------
    checkNetworkAddress : function(str) {
	    var addr, i, result, min, max;

	    if (str == '')
	        return false;

	    min = 0, max = 255;
	    result = false;

	    if (this.re_IPAddress.exec(str) != null) {
	        addr = str.split('.');

	        for (i = 0; i < 4; i++) {
		        if (this.checkRange(addr[i], min, max) == false)
		            break;
	        }

	        if (i == 4)
		        result = true;
	    }

	    return result;
    },

    //--------------------------------------------
    // Check Route target Address
    //--------------------------------------------
    checkRouteTarget : function(addr_str, mask_str) {
	    var addr, mask, addr_masked, i;

	    if (this.checkNetworkAddress(addr_str) == false)
	        return false;
	    if (this.checkNetmask(mask_str) == false)
	        return false;

	    addr = addr_str.split('.');
	    mask = mask_str.split('.');

	    for (i = 0; i < 4; i++) {
	        addr_masked = parseInt(addr[i]) & parseInt(mask[i]);

	        if (addr_masked != parseInt(addr[i]))
		        return false;
	    }

	    return true;
    },

    //--------------------------------------------
    // Check Network reachability
    //--------------------------------------------
    checkReachability : function(src_str, mask_str, dst_str) {
	    var src, mask, dst;
	    var src_masked, dst_masked, i;

	    if (this.checkIPAddress(src_str) == false)
	        return false;
	    if (this.checkIPAddress(dst_str) == false)
	        return false;
	    if (this.checkNetmask(mask_str) == false)
	        return false;

	    src  = src_str.split('.');
	    mask = mask_str.split('.');
	    dst  = dst_str.split('.');

	    for (i = 0; i < 4; i++) {
	        src_masked = parseInt(src[i]) & parseInt(mask[i]);
	        dst_masked = parseInt(dst[i]) & parseInt(mask[i]);

	        if (src_masked != dst_masked) {
		        debug($jq.sprintf("checkReachability: octet %d not match, (%d : %d) ",
			                  i + 1, src_masked, dst_masked));
		        return false;
	        }
	    }

	    return true;
    },

    //--------------------------------------------
    // Check FQDN
    //--------------------------------------------
    checkFQDN : function(str) {
	    var result = false;

	    if (str.match(this.re_FQDN) != null) {
	        debug($jq.sprintf("match: '%s' '%s'", RegExp.$1, RegExp.$2));
	        result = true;
	    }

	    return result;
    },

    //--------------------------------------------
    // Check OCN IPv6 account
    //--------------------------------------------
    checkOCN_Account : function(str) {
	    var result = false;

	    if (str.match(this.re_OCN) != null)
	        result = true;

	    return result;
    },

    //--------------------------------------------
    // Check E-mail address
    //--------------------------------------------
    checkEmailAddress : function(str) {
	    var result = false;

	    if (str.match(this.re_Email) != null)
	        result = true;

	    return result;
    }
};

//==========================================================
//
// Array.prototype extend
//
//==========================================================

Array.prototype.removeByIndex = function(idx) {
    if (idx < 0 || idx >= this.length)
	    return this;

    return this.slice(0, idx).concat(this.slice(idx + 1));
};

Array.prototype.insertByIndex = function(idx, value) {
    if (idx < 0 || idx >= this.length)
	    idx = this.length;

    return this.slice(0, idx).concat(value, this.slice(idx));
};

Array.prototype.replaceByIndex = function(idx, value) {
    if (idx < 0 || idx >= this.length)
	    idx = this.length;

    return this.slice(0, idx).concat(value, this.slice(idx + 1));
};

//==========================================================
//
// miscellaneous functions
//
//==========================================================
strip_escapeHTML = function(str) {
    return str.strip().escapeHTML();
};

show_indicator = function(elem) {
    $(elem).style.visibility = 'visible';
};

hide_indicator = function(elem) {
    $(elem).style.visibility = 'hidden';
};
