#! /usr/bin/env python
# -*- coding: utf-8 -*-

import re
import time
import traceback
import serial

LED_BASE = "/sys/class/leds/um01hw_%s%d/brightness"

LED_ANTENNA_R = LED_BASE % ('r', 1)
LED_ANTENNA_G = LED_BASE % ('g', 1)

LED_PACKET_R = LED_BASE % ('r', 2)
LED_PACKET_G = LED_BASE % ('g', 2)


class UM01HW_Antenna(object):
    def __init__(self, port):
        self._port = port
        self._ser = serial.Serial(self._port, 115200, timeout=0.5)

        self._cmd_pat = {
            "level" : ["AT%BAR", re.compile("%BAR:(\d)"), self.set_antennalevel],
            "stat"  : ["AT+CGREG?", re.compile("\+CGREG: \d,(\d)"),
                       self.set_servicestat]}

        self.clear_leds()

    def __del__(self):
        if self._ser:
            self._ser.close()

    def set_led(self, path, val):
        open(path, 'w').write("%d\n" % val)

    def clear_leds(self):
        self.set_led(LED_ANTENNA_R, 0)
        self.set_led(LED_ANTENNA_G, 0)
        self.set_led(LED_PACKET_R, 0)
        self.set_led(LED_PACKET_G, 0)

    def set_antennalevel(self, level):
        if level > 0:
            val_r = 0
            val_g = 1
        else:
            val_r = 1
            val_g = 0

        self.set_led(LED_ANTENNA_R, val_r)
        self.set_led(LED_ANTENNA_G, val_g)

    def set_servicestat(self, stat):
        if stat in [1, 5]:
            # 1: 圏内, 5: ローミングネットワーク圏内
            val_r = 0
            val_g = 1
        else:
            val_r = 1
            val_g = 0

        self.set_led(LED_PACKET_R, val_r)
        self.set_led(LED_PACKET_G, val_g)

    def mainloop(self):
        while True:
            for kind, defs in self._cmd_pat.iteritems():
                self._ser.write("%s\r\n" % defs[0])
                buf = self._ser.readlines()

                for s in buf:
                    match = defs[1].match(s)
                    if match:
                        val = int(match.groups()[0])
                        #print("match!: %s --> %d" % (s, val))
                        defs[2](val)

            time.sleep(2)


if __name__ == "__main__":
    while True:
        try:
            um01 = UM01HW_Antenna("/dev/ttyUM01HWcmd")
            um01.mainloop()
        except Exception, msg:
            err_traceback = traceback.format_exc()
            print msg, err_traceback

            if um01:
                um01.clear_leds()

        time.sleep(5)
