#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# MA-430 mgetty(dialin) wrapper
# Author : Takeyoshi Kikuchi <kikuchi@centurysys.co.jp>
# Copyright (C) 2007-2008 Century Systems Co.,Ltd.
#
# Last Modified: 2008/01/30 09:36:26 kikuchi
#

import os
import sys
import re
import glob
import time
import subprocess
import signal
import syslog
import uevent

debug = False
#debug = True

pppunit = 50
localdata = {}

#---------------------------------------
#
#---------------------------------------
def make_pidfile(ppp):
	pidfile = "/var/run/mgetty_wrapper.%s" % str(ppp)
	pid = os.getpid()

	fd = open(pidfile, 'w')
	fd.write("%d" % pid)
	fd.close()

	localdata['pidfile'] = pidfile

#---------------------------------------
#
#---------------------------------------
def del_pidfile():
	pidfile = localdata.get('pidfile', None)

	if pidfile != None:
		syslog.syslog("delete pidfile: %s" % pidfile)
		os.unlink(pidfile)

#---------------------------------------
#
#---------------------------------------
def device_exists(dev):
	count = len(glob.glob("/dev/%s" % dev))

	if count > 0:
		return True
	else:
		return False

#---------------------------------------
#
#---------------------------------------
def wait_for_device(dev):
	ev = uevent.uevent()

	while True:
		(result, event) = ev.receive()

		if result == True and event[0] == 'add':
			if re.match("/class/tty/(tty.+)", event[1]) == None:
				continue

			# wait for udev
			time.sleep(1)

			if device_exists(dev) == True:
				break

	return True

def sig_handler(signum, frame):
	syslog.syslog("mgetty_wrapper: signal no. %s" % str(signum))
	del_pidfile()

	child = localdata.get('child', None)
	if child:
		print "kill mgetty: pid = %d" % child.pid
		os.kill(child.pid, signal.SIGTERM)

	sys.exit(0)
		

#===========================================================
# main
#===========================================================
if __name__ == '__main__':
	if debug == False:
		mgetty_conf = "/etc/mgetty+sendfax/mgetty.config"
	else:
		mgetty_conf = "./mgetty.config"

	fd = open(mgetty_conf, 'r')
	dev = None

	while True:
		line = fd.readline()

		try:
			dev = re.match("^port (tty.+)$", line).groups()[0]
			break
		except:
			pass

	fd.close()

	if dev == None:
		print "'port tty...' not found."
		sys.exit(3)

	print "device is '%s'" % dev

	# make pidfile
	make_pidfile("mgetty")

	signal.signal(signal.SIGTERM, sig_handler)
	signal.signal(signal.SIGINT, sig_handler)

	if device_exists(dev) == False:
		wait_for_device(dev)

	# device ok, exec pppd
	cmdline = ["/usr/sbin/mgetty", "/dev/%s" % dev]
	child = subprocess.Popen(cmdline, close_fds=True)

	localdata['child'] = child
	child.wait()

	del_pidfile()
